#!/usr/bin/env python3
# -*- coding:utf-8 -*-

import re

typoLIST = [["一樣", "依樣", "一", "依樣畫葫蘆"],
            ["自己人", "自已人", "己", ""],
            ["已是", "以是", "已", "所以是"],
            ["已然", "以然", "已", ""],
            ["已經", "以經", "已", "以經理的、以經驗、以經常"],
            ["不能自已", "不能自己", "已", "不能自己先、不能自己來、不能自己去、不能自己做、"],
            ["而已", "而以", "已", "反而以"],
            ["什麼", "捨麼", "什", ""],
            ["什麼", "神麼", "什", ""],
            ["辦公室", "辦工室", "公", ""],
            ["化妝", "畫妝", "化", ""],
            ["來不及", "來不急", "及", ""],
            ["迫不及待", "迫不急待", "及", ""],
            ["尤其", "由其", "尤", "交由其他"],
            ["以前", "已前", "以", ""],
            ["以為", "已為", "以", ""],
            ["一旦", "一但", "旦", ""],
            ["生硬", "深硬", "生", ""],
            ["由於", "尤於", "由", ""],
            ["名字", "名子", "字", "姓周名子瑜"],
            ["類似於", "類是於", "似", ""],
            ["作息", "做息", "作", ""],
            ["作業", "做業", "作", ""],
            ["莫名其妙", "莫名奇妙", "其", ""],
            ["折價券", "折價卷", "券", ""],
            ["彩券", "彩卷", "券", ""],
            ["優惠券", "優惠卷", "券", ""],
            ["問卷", "問券", "卷", ""],
            ["姑且", "辜且", "姑", ""],
            ["上弦月", "上玄月", "弦", ""],
            ["下弦月", "下玄月", "弦", ""],
            ["弦樂團", "玄樂團", "弦", ""],
            ["弦樂器", "玄樂器", "弦", ""],
            ["管弦樂", "管玄樂", "弦", ""],
            ["明信片", "名信片", "明", ""],
            ["明信片", "名信片", "明", ""],
            ["目的", "目地", "的", "盲目地"],
            ["真的", "真得", "的", "真得意、真得寵"],
            ["不知道", "不之道", "知", ""],
            ["我知道", "我之道", "知", ""],
            ["近來可好", "進來可好", "近", ""],
            ["靠近", "靠進", "近", ""],
            ["長久", "常久", "長", "常久坐、常久站"],
            ["附件", "付件", "附", ""],
            ["有關係", "有關西", "係", "有關西腔"],
            ["沒關係", "沒關西", "係", "沒關西腔"],
            ["然後", "然候", "後", ""],
            ["重新", "從新", "重", "從新莊、從新店、從新竹、從新加坡、從新營、從新家"],
            ["重複", "從複", "重", "從複雜"],
            ["倒是", "到是", "倒", "沒想到是這樣"],
            ["時候", "時後", "候", "個小時後"],
            ["氣候", "氣後", "候", "生氣後、充氣後、打氣後、沒氣後、消氣後、賭氣後"],
            ["等候", "等後", "候", "等後面的"],
            ["剝殼", "撥殼", "剝", ""],
            ["剝蝦", "撥蝦", "剝", ""],
            ["座位", "坐位", "座", ""],
            ["座標", "坐標", "座", "坐標準"],
            ["根本", "跟本", "根", "跟本人"],
            ["好衰", "好雖", "衰", "好雖然"],
            ["哈密瓜", "哈蜜瓜", "密", ""],
            ["經常", "經長", "常", "已經長"],
            ["跑得快", "跑的快", "得", "用跑的快"],
            ["跑得慢", "跑的慢", "得", "用跑的慢"],
            ["說得好", "說的好", "得", "他說的好像"],
            ["覺得", "覺的", "得", "睡覺的"],
            ["從今以後", "重今以後", "從", ""],
            ["從頭", "重頭", "從", "重頭戲"],
            ["轉捩點", "轉淚點", "捩", ""],
            ["竟然", "盡然", "竟", "不盡然"],
            ["尊重", "遵重", "尊", ""],
            ["嵌入", "坎入", "嵌", ""],
            ["嵌入", "崁入", "嵌", ""],
            ["必須的", "必虛的", "須", ""],
            ["搜尋", "收尋", "搜", ""],
            ["不禁", "不經", "禁", "不經歷、不經一番、荒誕不經、寧失不經、不經常、不經過"],
            ["詢問", "尋問", "詢", "千尋問"],
            ["中文造詣", "中文造旨", "詣", ""],
            ["中文造詣", "中文造紙", "詣", ""],
            ["國文造詣", "國文造紙", "詣", ""],
            ["語文造詣", "語文造旨", "詣", ""],
            ["語文造詣", "語文造紙", "詣", ""],
            ["難道", "難到", "道", "好難到、難到達、困難到"],
            ["一鼓作氣", "一股作氣", "鼓", ""],
            ["抱歉", "抱欠", "歉", ""],
            ["盡量", "竟量", "盡", "竟量不到"],
            ["趕快", "敢快", "趕", "不敢快"],
            ["影響", "引響", "影", "吸引響尾蛇"],
            ["撨看看", "喬看看", "撨", ""],
            ["播放", "撥放", "播", ""],
            ["去潛水", "去淺水", "潛", ""],
            ["潛力", "淺力", "潛", ""],
            ["潛水夫", "淺水夫", "潛", ""],
            ["學潛水", "學淺水", "潛", ""],
            ["儘管", "僅管", "儘", ""],
            ["錄取", "入取", "錄", ""],
            ["應該", "因該", "應", "死因該、原因該、因該名眾、因該嫌犯、因該男、因該女、因該人"],
            ["戳到笑點", "搓到笑點", "戳", ""],
            ["戳破謊言", "搓破謊言", "戳", ""],
            ["荒謬", "荒繆", "謬", ""]
]

def typoChecker(inputSTR):
    resultLIST = []
    for t_lst in typoLIST:
        targetPat = re.compile(t_lst[1])
        scanLIST = [[s.start(), s.end(), s.group(), t_lst[0]] for s in list(targetPat.finditer(inputSTR))]
        if scanLIST == []:
            pass
        else:
            if t_lst[3] == "":
                resultLIST.extend(scanLIST)
                #resultLIST[-1].append(t_lst[0])
            else:
                whitePat = re.compile("|".join(t_lst[3].split("、")))
                whiteLIST = [[w.start(), w.end(), w.group()] for w in list(whitePat.finditer(inputSTR))]
                removeLIST = []
                for s_tpl in scanLIST:
                    for w_tpl in whiteLIST:
                        if set(range(s_tpl[0], s_tpl[1]+1)).intersection(set(range(w_tpl[0], w_tpl[1]+1))) == set():
                            pass
                        else:
                            removeLIST.append(s_tpl)
                for r_tpl in removeLIST:
                    scanLIST.remove(r_tpl)
                if scanLIST != []:
                    resultLIST.extend(scanLIST)
    resultLIST = sorted(resultLIST)
    return resultLIST


if __name__ == "__main__":
    inputSTR = "因該吧，我不之道是否因該男子如此的表現會有所不同，所以我覺得只是因該而已。"
    resultLIST = typoChecker(inputSTR)
    print(resultLIST)
