#!/usr/bin/env python3
# -*- coding:utf-8 -*-

"""
    Loki module for Zai_QMarker

    Input:
        inputSTR      str,
        utterance     str,
        args          str[],
        resultDICT    dict

    Output:
        resultDICT    dict
"""

import json
import os

DEBUG_Zai_QMarker = True
try:
    userDefinedDICT = json.load(open(os.path.join(os.path.dirname(__file__), "USER_DEFINED.json"), encoding="utf-8"))
except:
    userDefinedDICT = {"_FAC":["HResort","國家音樂廳","市政府","法說會","營業場所","網路媒體","縣政府","電腦網路","電腦網路媒體","非保險醫療機構","音樂廳"],"_asNoun":["7-11","@","WPPT","《","》","「","」","『","』","【","】","七星山","中山堂","五大都會區","伺服器","八位元環境","六都","前提","坡度","太祖媽廟","太興社區","學習時空","山上人家","已開發國家","師生","新光三越","時機點","申報戶","白玉","老湖口","舊檢查哨","虎頭埤","親等","資金","過季廣場","鐵砧山","飛比價格","（","）"],"_asVerb":["上廁所","便民","兩稅合一","日趨多樣","欺壓"],"_schoolTW":["一甲國中","一甲國小","一甲國民中學","一甲國民小學","七堵國小","七堵國民小學","七星國小","七星國民小學","七美國中","七美國小","七美國民中學","七美國民小學","七美鄉立托兒所","七股國小","七股國民小學","七股托兒所","七賢國中","七賢國小","七賢國民中學","七賢國民小學","三仙國小","三仙國民小學","三侯國小","三侯國民小學","三信家事商業職業學校","三信家商","三光國中","三光國小","三光國民中學","三光國民小學","三和國中","三和國小","三和國民中學","三和國民小學","三地門鄉立托兒所","三坑國小","三坑國民小學","三埤國小","三埤國民小學","三多國中","三多國小","三多國民中學","三多國民小學","三峽國中","三峽國小","三峽國民中學","三峽國民小學","三峽幼兒園","三峽托兒所","三崙國小","三崙國民小學","三慈國小","三慈國民小學","三星國中","三星國小","三星國民中學","三星國民小學","三星鄉立托兒所","三春國小","三春國民小學","三村國小","三村國民小學","三條國小","三條國民小學","三棧國小","三棧國民小學","三民商職","三民國中","三民國小","三民國民中學","三民國民小學","三民家商","三民高中","三民高商","三民高級中學","三民高級家事商業職業學校","三民高職","三江國小","三江國民小學","三潭國小","三潭國民小學","三灣國中","三灣國小","三灣國民中學","三灣國民小學","三灣鄉立幼兒園","三灣鄉立托兒所","三玉國小","三玉國民小學","三田國小","三田國民小學","三義鄉立托兒所","三義高中","三義高級中學","三股國小","三股國民小學","三育實驗完全中學","三育高中","三興國小","三興國民小學","三芝國中","三芝國小","三芝國民中學","三芝國民小學","三芝幼兒園","三芝托兒所","三重商工","三重國小","三重國民小學","三重幼兒園","三重托兒所","三重高中","三重高級中學","三重高級商工職業學校","三間國小","三間國民小學","上大國小","上大國民小學","上安國小","上安國民小學","上岐國小","上岐國民小學","上平國小","上平國民小學","上林國小","上林國民小學","上楓國小","上楓國民小學","上湖國小","上湖國民小學","上田國小","上田國民小學","上石國小","上石國民小學","上館國小","上館國民小學","下坑國小","下坑國民小學","下崙國小","下崙國民小學","下楫國小","下楫國民小學","下潭國小","下潭國民小學","下營國中","下營國小","下營國民中學","下營國民小學","下營幼兒園","下營托兒所","世新","世新大學","世界新聞傳播學院","世界高中","世界高級中學","世賢國小","世賢國民小學","中信國小","中信國民小學","中原國小","中原國民小學","中原大學","中原理工學院","中台科大","中台科技大學","中台醫護技術學院","中和國中","中和國小","中和國民中學","中和國民小學","中和幼兒園","中和托兒所","中和高中","中和高級中學","中國技術學院","中國文化大學","中國文化學院","中國科大","中國科技大學","中國醫","中國醫藥大學","中國醫藥學院","中園國小","中園國民小學","中坑國小","中坑國民小學","中城國小","中城國民小學","中埔國中","中埔國小","中埔國民中學","中埔國民小學","中埔鄉立托兒所","中壇國小","中壇國民小學","中壢商業高級中等學校","中壢國中","中壢國小","中壢國民中學","中壢國民小學","中壢家事商業高級中等學校","中壢家商","中壢幼兒園","中壢高中","中壢高商","中壢高鐵幼兒園","中大","中大壢中","中大附中","中央大學","中央大學醫學院","中央大學附屬中壢高級中學","中央警察大學","中央軍官學校","中央軍校","中寮國中","中寮國小","中寮國民中學","中寮國民小學","中寮鄉立托兒所","中屯國小","中屯國民小學","中山國中","中山國小","中山國民中學","中山國民小學","中山大學","中山大學附屬國光高級中學","中山女子高級中學","中山女高","中山小學","中山幼兒園","中山托兒所","中山牙醫專科學校","中山社區大學","中山社大","中山醫","中山醫學大學","中山醫學專科學校","中山醫學院","中山醫學院專科學校","中山附中","中山高中","中山高級中學","中峰國小","中峰國民小學","中崙國中","中崙國小","中崙國民中學","中崙國民小學","中崙高中","中崙高級中學","中州國小","中州國民小學","中州技術學院","中師","中平國中","中平國小","中平國民中學","中平國民小學","中庄國中","中庄國小","中庄國民中學","中庄國民小學","中教","中教大","中教大實小","中教大附小","中林國小","中林國民小學","中正","中正中小學","中正國中","中正國小","中正國民中學","中正國民中小學","中正國民小學","中正國防幹部預備學校","中正大學","中正幼兒園","中正托兒所","中正理工學院","中正社區大學","中正社大","中正預校","中正高中","中正高工","中正高級中學","中正高級工業職業學校","中泰國小","中泰國民小學","中洲國小","中洲國民小學","中港國小","中港國民小學","中港高中","中港高級中學","中湖國小","中湖國民小學","中溪國小","中溪國民小學","中營國小","中營國民小學","中科國小","中科國民小學","中科大","中科實中","中科實驗高級中學","中臺科大","中臺科技大學","中臺醫護技術學院","中興商工","中興國中","中興國小","中興國民中學","中興國民小學","中興大學","中興大學附屬台中高級農業職業學校","中興大學附屬臺中高級農業職業學校","中興大學附屬高級中學","中興高中","中興高級中學","中興高級商工職業學校","中芸國中","中芸國小","中芸國民中學","中芸國民小學","中華國中","中華國小","中華國民中學","中華國民小學","中華大學","中華工商","中華工學院","中華技術學院","中華科大","中華科技大學","中華藝術學校","中華醫事學院","中華醫事科大","中華醫事科技大學","中華高級工商職業學校","中華高級藝術職業學校","中角國小","中角國民小學","中路國小","中路國民小學","中道高中","中道高級中學","丹路國小","丹路國民小學","丹鳳國小","丹鳳國民小學","丹鳳高中","丹鳳高級中學","久安國小","久安國民小學","久美國小","久美國民小學","九份國小","九份國民小學","九如國中","九如國小","九如國民中學","九如國民小學","九德國小","九德國民小學","九曲國小","九曲國民小學","九湖國小","九湖國民小學","九芎國小","九芎國民小學","乾峰國小","乾峰國民小學","乾華國小","乾華國民小學","二信高中","二信高級中學","二城國小","二城國民小學","二崙國中","二崙國小","二崙國民中學","二崙國民小學","二崙鄉立托兒所","二林國小","二林國民小學","二林工商","二林鎮立托兒所","二林高中","二林高級中學","二林高級工商職業學校","二橋國小","二橋國民小學","二水國中","二水國小","二水國民中學","二水國民小學","二水鄉立托兒所","二溪國小","二溪國民小學","二苓國小","二苓國民小學","二重國中","二重國小","二重國民中學","二重國民小學","互助國小","互助國民小學","五光國小","五光國民小學","五城國小","五城國民小學","五堵國小","五堵國民小學","五寮國小","五寮國民小學","五峰國中","五峰國小","五峰國民中學","五峰國民小學","五峰鄉立托兒所","五常國中","五常國小","五常國民中學","五常國民小學","五德國小","五德國民小學","五林國小","五林國民小學","五權國中","五權國小","五權國民中學","五權國民小學","五湖國小","五湖國民小學","五溝國小","五溝國民小學","五王國小","五王國民小學","五甲國中","五甲國小","五甲國民中學","五甲國民小學","五福國中","五福國小","五福國民中學","五福國民小學","五穀國小","五穀國民小學","五結國中","五結國小","五結國民中學","五結國民小學","五結鄉立托兒所","五股國中","五股國小","五股國民中學","五股國民小學","五股幼兒園","五股托兒所","五育高中","五育高級中學","五華國小","五華國民小學","五龍國小","五龍國民小學","亞大","亞東學院","亞東技術學院","亞洲大學","亞洲餐旅","亞洲高級餐旅職業學校","交大","交通大學","人和國小","人和國民小學","人文中小學","人文國民中小學","仁光國小","仁光國民小學","仁和國中","仁和國小","仁和國民中學","仁和國民小學","仁善國小","仁善國民小學","仁德國中","仁德國小","仁德國民中學","仁德國民小學","仁德幼兒園","仁德托兒所","仁德文賢國中","仁德文賢國民中學","仁德醫專","仁德醫護管理專科學校","仁德高級醫事職業學校","仁愛國中","仁愛國小","仁愛國民中學","仁愛國民小學","仁愛鄉立托兒所","仁愛高級農業職業學校","仁愛高農","仁武國小","仁武國民小學","仁武幼兒園","仁武特教","仁武特殊教育學校","仁武高中","仁武高級中學","仁美國中","仁美國小","仁美國民中學","仁美國民小學","仁義高中","仁義高級中學","仁豐國小","仁豐國民小學","介壽中小學","介壽國中","介壽國小","介壽國民中學","介壽國民中小學","介壽國民小學","介達國小","介達國民小學","仕絨國小","仕絨國民小學","仕隆國小","仕隆國民小學","仙吉國小","仙吉國民小學","仙洞國小","仙洞國民小學","仙草國小","仙草國民小學","以栗國小","以栗國民小學","仰德高中","仰德高級中學","伸仁國小","伸仁國民小學","伸東國小","伸東國民小學","伸港國中","伸港國民中學","伸港鄉立托兒所","何厝國小","何厝國民小學","何浦國小","何浦國民小學","佛光","佛光人文社會學院","佛光大學","佛公國小","佛公國民小學","佳佐國小","佳佐國民小學","佳冬國中","佳冬國小","佳冬國民中學","佳冬國民小學","佳冬鄉立托兒所","佳冬高級農業職業學校","佳冬高農","佳林國中","佳林國民中學","佳民國小","佳民國民小學","佳義國小","佳義國民小學","佳興國中","佳興國小","佳興國民中學","佳興國民小學","佳里國中","佳里國小","佳里國民中學","佳里國民小學","來吉國小","來吉國民小學","來惠國小","來惠國民小學","來義國小","來義國民小學","來義鄉立托兒所","來義高中","來義高級中學","依仁國小","依仁國民小學","保東國小","保東國民小學","保生國小","保生國民小學","保西國小","保西國民小學","保長國小","保長國民小學","信勢國小","信勢國民小學","信德國小","信德國民小學","信義中小學","信義國中","信義國小","信義國民中學","信義國民中小學","信義國民小學","信義幼兒園","信義托兒所","信義社區大學","信義社大","信義鄉立托兒所","修平技術學院","修德國小","修德國民小學","健康國小","健康國民小學","健民國小","健民國民小學","健行國小","健行國民小學","健行技術學院","僑仁國小","僑仁國民小學","僑信","僑信國民小學","僑光","僑光國小","僑光國民小學","僑光技術學院","僑光科技大學","僑勇國小","僑勇國民小學","僑和國小","僑和國民小學","僑善國小","僑善國民小學","僑孝國小","僑孝國民小學","僑平國小","僑平國民小學","僑建國小","僑建國民小學","僑德國小","僑德國民小學","僑忠國小","僑忠國民小學","僑愛國小","僑愛國民小學","僑成國小","僑成國民小學","僑文國小","僑文國民小學","僑智國小","僑智國民小學","僑榮國小","僑榮國民小學","僑樂國小","僑樂國民小學","僑真國小","僑真國民小學","僑美國小","僑美國民小學","僑義","僑義國民小學","僑育國小","僑育國民小學","僑興國小","僑興國民小學","億載國小","億載國民小學","優佳國中","優佳國民中學","元培科大","元培科學技術學院","元培科技大學","元智","元智大學","元智工學院","元生國小","元生國民小學","元長國中","元長國小","元長國民中學","元長國民小學","元長鄉立托兒所","兆湘國小","兆湘國民小學","光仁小學","光復中小學","光復商工","光復國中","光復國小","光復國民中學","光復國民中小學","光復國民小學","光復實小","光復生態實驗小學","光復鄉立托兒所","光復高中","光復高級中學","光復高級商工職業學校","光德國中","光德國民中學","光明國中","光明國小","光明國民中學","光明國民小學","光春國中","光春國小","光春國民中學","光春國民小學","光榮國中","光榮國小","光榮國民中學","光榮國民小學","光正國中","光正國小","光正國民中學","光正國民小學","光武國中","光武國小","光武國民中學","光武國民小學","光武學院","光武技術學院","光武技術學院附設專科進修學校","光興國小","光興國民小學","光華國中","光華國小","光華國民中學","光華國民小學","光華女中","光華女子高中","光華女子高級中學","光華女高","光華高工","光華高級工業職業學校","光隆商職","光隆國小","光隆國民小學","光隆家商","光隆高商","光隆高級家事商業職業學校","光隆高職","內垵國小","內垵國民小學","內城中小學","內城國民中小學","內埔國中","內埔國小","內埔國民中學","內埔國民小學","內埔農工","內埔高級農工職業學校","內壢國中","內壢國小","內壢國民中學","內壢國民小學","內壢高中","內壢高級中等學校","內安國小","內安國民小學","內定國小","內定國民小學","內工","內思高工","內思高級工業職業學校","內惟國小","內惟國民小學","內新國小","內新國民小學","內柵國小","內柵國民小學","內海國小","內海國民小學","內湖國中","內湖國小","內湖國民中學","內湖國民小學","內湖幼兒園","內湖托兒所","內湖社區大學","內湖社大","內湖高級中學","內湖高級工業職業學校","內灣國小","內灣國民小學","內獅國小","內獅國民小學","內甕國小","內甕國民小學","內角國小","內角國民小學","內門國中","內門國小","內門國民中學","內門國民小學","內門幼兒園","內高","全德國小","全德國民小學","八卦國小","八卦國民小學","八堵國小","八堵國民小學","八德國中","八德國小","八德國民中學","八德國民小學","八德幼兒園","八斗國小","八斗國民小學","八斗高中","八斗高級中學","八里國中","八里國小","八里國民中學","八里國民小學","八里幼兒園","八里托兒所","公務人力發展學院","公園國小","公園國民小學","公埔國小","公埔國民小學","公明國中","公明國小","公明國民中學","公明國民小學","公東高工","公東高級工業職業學校","公正國中","公正國小","公正國民中學","公正國民小學","公誠國小","公誠國民小學","公館國中","公館國小","公館國民中學","公館國民小學","六信高中","六信高級中學","六合國小","六合國民小學","六嘉國中","六嘉國民中學","六家國小","六家國民小學","六家高中","六家高級中學","六寶國小","六寶國民小學","六甲國中","六甲國小","六甲國民中學","六甲國民小學","六甲幼兒園","六甲托兒所","六美國小","六美國民小學","六腳國小","六腳國民小學","六腳鄉立托兒所","六龜國小","六龜國民小學","六龜幼兒園","六龜高中","六龜高級中學","共和國小","共和國民小學","再興小學","再興高中","再興高級中學","冬山國中","冬山國小","冬山國民中學","冬山國民小學","冬山鄉立托兒所","凌雲國中","凌雲國小","凌雲國民中學","凌雲國民小學","凱旋國中","凱旋國小","凱旋國民中學","凱旋國民小學","初來國小","初來國民小學","初鄉國小","初鄉國民小學","初鹿國中","初鹿國小","初鹿國民中學","初鹿國民小學","利嘉國小","利嘉國民小學","利澤國中","利澤國小","利澤國民中學","利澤國民小學","前山國小","前山國民小學","前峰國中","前峰國小","前峰國民中學","前峰國民小學","前進國小","前進國民小學","前金國中","前金國小","前金國民中學","前金國民小學","前金幼兒園","前金幼稚園","前鎮國中","前鎮國小","前鎮國民中學","前鎮國民小學","前鎮高中","前鎮高級中學","劍潭國小","劍潭國民小學","力社國小","力社國民小學","力行國小","力行國民小學","力里國小","力里國民小學","加拿國小","加拿國民小學","加昌國小","加昌國民小學","加祿國小","加祿國民小學","勝利國小","勝利國民小學","勤益","勤益技術學院","勤益技術學院附設進修學院","勤益科大附設進修學院","勤益科技大學","勤益科技大學附設進修學院","化仁國中","化仁國小","化仁國民中學","化仁國民小學","北一女","北勢國中","北勢國小","北勢國民中學","北勢國民小學","北區立人國小","北區立人國民小學","北商","北商大","北回國小","北回國民小學","北園國中","北園國小","北園國民中學","北園國民小學","北城科大","北埔國中","北埔國小","北埔國民中學","北埔國民小學","北埔鄉立托兒所","北大","北大國小","北大國民小學","北大高中","北大高級中學","北安國中","北安國民中學","北寮國小","北寮國民小學","北屯國小","北屯國民小學","北山國中","北山國小","北山國民中學","北山國民小學","北峰國小","北峰國民小學","北嶺國小","北嶺國民小學","北平華德福實驗學校","北成國小","北成國民小學","北投區立農國民小學","北投國中","北投國小","北投國民中學","北投國民小學","北投幼兒園","北投托兒所","北投社區大學","北投社大","北政國中","北政國民中學","北教大","北教大實小","北斗國中","北斗國小","北斗國民中學","北斗國民小學","北斗家商","北斗鎮立托兒所","北斗高級家事商業職業學校","北新國中","北新國小","北新國民中學","北新國民小學","北昌國小","北昌國民小學","北林國小","北林國民小學","北梅國中","北梅國民中學","北港國中","北港國小","北港國民中學","北港國民小學","北港農工","北港鎮立托兒所","北港高中","北港高級中學","北港高級農工職業學校","北湖國小","北湖國民小學","北源國小","北源國民小學","北濱國小","北濱國民小學","北科","北科大","北科桃農","北科附工","北美國小","北美國民小學","北興國中","北興國民中學","北葉國小","北葉國民小學","北藝","北藝大","北護","北辰國小","北辰國民小學","北醫","北醫大","北門國中","北門國小","北門國民中學","北門國民小學","北門農工","北門高中","北門高級中學","北門高級農工職業學校","十信高中","十信高級中學","十全國小","十全國民小學","十分國小","十分國民小學","十字國小","十字國民小學","十興國小","十興國民小學","千秋國小","千秋國民小學","卑南國中","卑南國小","卑南國民中學","卑南國民小學","卑南鄉立托兒所","卓中","卓楓國小","卓楓國民小學","卓樂國小","卓樂國民小學","卓清國小","卓清國民小學","卓溪國小","卓溪國民小學","卓溪鄉立山國小","卓溪鄉立山國民小學","卓溪鄉立托兒所","卓環國小","卓環國民小學","卓蘭國小","卓蘭國民小學","卓蘭鎮立托兒所","卓蘭高級中等學校","協同高中","協同高級中學","協和國小","協和國民小學","協和工商","協和高級工商職業學校","協志工商","協志高級工商職業學校","協成國小","協成國民小學","協進國小","協進國民小學","南一中","南二中","南亞技術學院","南光國小","南光國民小學","南勢國小","南勢國民小學","南化國中","南化國小","南化國民中學","南化國民小學","南台技術學院","南台科技大學","南和國中","南和國小","南和國民中學","南和國民小學","南商","南埔國小","南埔國民小學","南大","南大附小","南女","南安國中","南安國小","南安國民中學","南安國民小學","南寧高中","南寧高級中學","南寮國小","南寮國民小學","南屏國小","南屏國民小學","南屯國小","南屯國民小學","南屯幼兒園","南屯托兒所","南山國小","南山國民小學","南崁國中","南崁國小","南崁國民中學","南崁國民小學","南崁幼兒園","南崁高中","南崁高級中等學校","南崗國中","南崗國民中學","南州國中","南州國小","南州國民中學","南州國民小學","南平中學","南庄國中","南庄國小","南庄國民中學","南庄國民小學","南庄鄉立托兒所","南應大","南成國小","南成國民小學","南投國中","南投國小","南投國民中學","南投國民小學","南投市立托兒所","南投特教","南投特殊教育學校","南投縣中寮鄉中寮國民小學","南投縣中寮鄉廣福國民小學","南投縣中寮鄉永和國民小學","南投縣中寮鄉永康國民小學","南投縣中寮鄉永樂國民小學","南投縣中寮鄉清水國民小學","南投縣中寮鄉爽文國民小學","南投縣中寮鄉立托兒所","南投縣中寮鄉至誠國民小學","南投縣仁愛鄉中正國民小學","南投縣仁愛鄉互助國民小學","南投縣仁愛鄉仁愛國民小學","南投縣仁愛鄉力行國民小學","南投縣仁愛鄉南豐國民小學","南投縣仁愛鄉合作國民小學","南投縣仁愛鄉廬山國民小學","南投縣仁愛鄉春陽國民小學","南投縣仁愛鄉法治國民小學","南投縣仁愛鄉清境國民小學","南投縣仁愛鄉發祥國民小學","南投縣仁愛鄉立托兒所","南投縣仁愛鄉紅葉國民小學","南投縣仁愛鄉萬豐國民小學","南投縣仁愛鄉親愛國民小學","南投縣仁愛鄉都達國民小學","南投縣信義鄉久美國民小學","南投縣信義鄉人和國民小學","南投縣信義鄉信義國民小學","南投縣信義鄉同富國民小學","南投縣信義鄉地利國民小學","南投縣信義鄉愛國國民小學","南投縣信義鄉新鄉國民小學","南投縣信義鄉東埔國民小學","南投縣信義鄉桐林國民小學","南投縣信義鄉潭南國民小學","南投縣信義鄉立托兒所","南投縣信義鄉羅娜國民小學","南投縣信義鄉豐丘國民小學","南投縣信義鄉隆華國民小學","南投縣信義鄉雙龍國民小學","南投縣南投市僑建國民小學","南投縣南投市光復國民小學","南投縣南投市光榮國民小學","南投縣南投市光華國民小學","南投縣南投市千秋國民小學","南投縣南投市南投國民小學","南投縣南投市嘉和國民小學","南投縣南投市平和國民小學","南投縣南投市康壽國民小學","南投縣南投市德興國民小學","南投縣南投市文山國民小學","南投縣南投市新豐國民小學","南投縣南投市漳和國民小學","南投縣南投市漳興國民小學","南投縣南投市營盤國民小學","南投縣南投市立托兒所","南投縣南投市西嶺國民小學","南投縣名間鄉中山國民小學","南投縣名間鄉僑興國民小學","南投縣名間鄉名崗國民小學","南投縣名間鄉名間國民小學","南投縣名間鄉弓鞋國民小學","南投縣名間鄉新民國民小學","南投縣名間鄉新街國民小學","南投縣名間鄉田豐國民小學","南投縣名間鄉立托兒所","南投縣國姓鄉乾峰國民小學","南投縣國姓鄉北山國民小學","南投縣國姓鄉北港國民小學","南投縣國姓鄉南港國民小學","南投縣國姓鄉國姓國民小學","南投縣國姓鄉港源國民小學","南投縣國姓鄉福龜國民小學","南投縣國姓鄉立托兒所","南投縣國姓鄉育樂國民小學","南投縣國姓鄉長流國民小學","南投縣國姓鄉長福國民小學","南投縣埔里鎮中峰國民小學","南投縣埔里鎮南光國民小學","南投縣埔里鎮史港國民小學","南投縣埔里鎮埔里國民小學","南投縣埔里鎮大成國民小學","南投縣埔里鎮太平國民小學","南投縣埔里鎮忠孝國民小學","南投縣埔里鎮愛蘭國民小學","南投縣埔里鎮桃源國民小學","南投縣埔里鎮水尾國民小學","南投縣埔里鎮溪南國民小學","南投縣埔里鎮立托兒所","南投縣埔里鎮育英國民小學","南投縣埔里鎮麒麟國民小學","南投縣水里鄉成城國民小學","南投縣水里鄉新興國民小學","南投縣水里鄉民和國民小學","南投縣水里鄉水里國民小學","南投縣水里鄉永興國民小學","南投縣水里鄉立托兒所","南投縣水里鄉郡坑國民小學","南投縣私立三育實驗完全中學","南投縣私立五育高級中學","南投縣私立同德家事商業職業學校","南投縣私立均頭國民中學","南投縣私立普台國民中小學","南投縣立三光國民中學","南投縣立中寮國民中學","南投縣立中興國民中學","南投縣立仁愛國民中學","南投縣立信義國民中學","南投縣立北山國民中學","南投縣立北梅國民中學","南投縣立南崗國民中學","南投縣立南投國民中學","南投縣立同富國民中學","南投縣立名間國民中學","南投縣立國姓國民中學","南投縣立埔里國民中學","南投縣立大成國民中學","南投縣立宏仁國民中學","南投縣立延和國民中學","南投縣立日新國民中學","南投縣立旭光高級中學","南投縣立明潭國民中學","南投縣立民和國民中學","南投縣立水里國民中學","南投縣立營北國民中學","南投縣立爽文國民中學","南投縣立瑞峰國民中學","南投縣立瑞竹國民中學","南投縣立社寮國民中學","南投縣立竹山國民中學","南投縣立草屯國民中學","南投縣立集集國民中學","南投縣立魚池國民中學","南投縣立鳳鳴國民中學","南投縣立鹿谷國民中學","南投縣竹山鎮中和國民小學","南投縣竹山鎮中州國民小學","南投縣竹山鎮前山國民小學","南投縣竹山鎮大鞍國民小學","南投縣竹山鎮延平國民小學","南投縣竹山鎮桶頭國民小學","南投縣竹山鎮瑞竹國民小學","南投縣竹山鎮社寮國民小學","南投縣竹山鎮秀林國民小學","南投縣竹山鎮立托兒所","南投縣竹山鎮竹山國民小學","南投縣竹山鎮過溪國民小學","南投縣竹山鎮雲林國民小學","南投縣竹山鎮鯉魚國民小學","南投縣草屯鎮中原國民小學","南投縣草屯鎮僑光國民小學","南投縣草屯鎮北投國民小學","南投縣草屯鎮土城國民小學","南投縣草屯鎮坪頂國民小學","南投縣草屯鎮富功國民小學","南投縣草屯鎮平林國民小學","南投縣草屯鎮敦和國民小學","南投縣草屯鎮新庄國民小學","南投縣草屯鎮炎峰國民小學","南投縣草屯鎮碧峰國民小學","南投縣草屯鎮立托兒所","南投縣草屯鎮草屯國民小學","南投縣草屯鎮虎山國民小學","南投縣草屯鎮雙冬國民小學","南投縣集集鎮和平國民小學","南投縣集集鎮永昌國民小學","南投縣集集鎮立托兒所","南投縣集集鎮隘寮國民小學","南投縣集集鎮集集國民小學","南投縣魚池鄉五城國民小學","南投縣魚池鄉共和國民小學","南投縣魚池鄉德化國民小學","南投縣魚池鄉新城國民小學","南投縣魚池鄉明潭國民小學","南投縣魚池鄉東光國民小學","南投縣魚池鄉立托兒所","南投縣魚池鄉頭社國民小學","南投縣魚池鄉魚池國民小學","南投縣鹿谷鄉內湖國民小學","南投縣鹿谷鄉初鄉國民小學","南投縣鹿谷鄉廣興國民小學","南投縣鹿谷鄉文昌國民小學","南投縣鹿谷鄉瑞田國民小學","南投縣鹿谷鄉秀峰國民小學","南投縣鹿谷鄉立托兒所","南投縣鹿谷鄉鳳凰國民小學","南投縣鹿谷鄉鹿谷國民小學","南投高中","南投高商","南投高級中學","南投高級商業職業學校","南新國中","南新國小","南新國民中學","南新國民小學","南梓實小","南梓實驗小學","南榮國中","南榮國小","南榮國民中學","南榮國民小學","南榮學院","南榮工商專科學校","南榮技術學院","南榮科大","南榮科技大學","南河國小","南河國民小學","南海實驗幼兒園","南海幼兒園","南海幼稚園","南港國小","南港國民小學","南港幼兒園","南港托兒所","南港社區大學","南港社大","南港高中","南港高工","南港高級中學","南港高級工業職業學校","南湖國中","南湖國小","南湖國民中學","南湖國民小學","南湖高中","南湖高級中學","南澳國小","南澳國民小學","南澳鄉立托兒所","南澳高中","南澳高級中學","南王國小","南王國民小學","南科國際實驗高級中學","南科大","南科實中","南美國小","南美國民小學","南臺技術學院","南臺科技大學","南興國中","南興國小","南興國民中學","南興國民小學","南英商工","南英高級商工職業學校","南華","南華國中","南華國小","南華國民中學","南華國民小學","南華大學","南華管理學院","南華高中","南華高級中學職業進修學校","南藝","南藝大","南護專","南豐國小","南豐國民小學","南郭國小","南郭國民小學","南鎮國小","南鎮國民小學","南門國中","南門國小","南門國民中學","南門國民小學","南開技術學院","南開科大","南開科技大學","南陽國小","南陽國民小學","南隆國中","南隆國民中學","南隘國小","南隘國民小學","南靖國小","南靖國民小學","博嘉實小","博嘉實驗國民小學","博屋瑪國小","博屋瑪國民小學","博愛國中","博愛國小","博愛國民中學","博愛國民小學","厚德國小","厚德國民小學","原斗中小學","原斗國民中小學","口埤實小","口埤實驗小學","口湖國中","口湖國小","口湖國民中學","口湖國民小學","口湖鄉立托兒所","口社國小","口社國民小學","古亭國中","古亭國小","古亭國民中學","古亭國民小學","古亭托兒所","古坑中小學","古坑國民中小學","古坑華德福實驗高中","古坑華德福實驗高級中學","古坑鄉立托兒所","古城國小","古城國民小學","古寧國小","古寧國民小學","古樓國小","古樓國民小學","古民國小","古民國民小學","古華國小","古華國民小學","古風國小","古風國民小學","台中一中","台中二中","台中健康暨管理學院","台中國小","台中國民小學","台中女中","台中女子高級中等學校","台中女高","台中家事商業高級中等學校","台中家商","台中實小","台中工業高級中等學校","台中市中區光復國民小學","台中市北區中華國民小學","台中市北區健行國民小學","台中市北區太平國民小學","台中市北區省三國民小學","台中市北區立人國民小學","台中市北區篤行國民小學","台中市北區賴厝國民小學","台中市北屯區仁愛國民小學","台中市北屯區仁美國民小學","台中市北屯區僑孝國民小學","台中市北屯區北屯國民小學","台中市北屯區四張犁國民小學","台中市北屯區四維國民小學","台中市北屯區大坑國民小學","台中市北屯區廍子國民小學","台中市北屯區建功國民小學","台中市北屯區文心國民小學","台中市北屯區文昌國民小學","台中市北屯區新興國民小學","台中市北屯區東光國民小學","台中市北屯區松竹國民小學","台中市北屯區軍功國民小學","台中市北屯區逢甲國民小學","台中市北屯區陳平國民小學","台中市南區信義國民小學","台中市南區和平國民小學","台中市南區國光國民小學","台中市南區樹義國民小學","台中市南屯區南屯國民小學","台中市南屯區大墩國民小學","台中市南屯區大新國民小學","台中市南屯區惠文國民小學","台中市南屯區文山國民小學","台中市南屯區春安國民小學","台中市南屯區東興國民小學","台中市南屯區永春國民小學","台中市南屯區鎮平國民小學","台中市南屯區黎明國民小學","台中市博屋瑪國民小學","台中市后里區七星國民小學","台中市后里區內埔國民小學","台中市后里區后里國民小學","台中市后里區月眉國民小學","台中市后里區泰安國民小學","台中市后里區育英國民小學","台中市和平區中坑國民小學","台中市和平區博愛國民小學","台中市和平區和平國民小學","台中市和平區平等國民小學","台中市和平區白冷國民小學","台中市和平區自由國民小學","台中市外埔區外埔國民小學","台中市外埔區安定國民小學","台中市外埔區水美國民小學","台中市外埔區鐵山國民小學","台中市外埔區馬鳴國民小學","台中市大安區三光國民小學","台中市大安區大安國民小學","台中市大安區永安國民小學","台中市大安區海墘國民小學","台中市大甲區大甲國民小學","台中市大甲區德化國民小學","台中市大甲區文昌國民小學","台中市大甲區文武國民小學","台中市大甲區日南國民小學","台中市大甲區東明國民小學","台中市大甲區東陽國民小學","台中市大甲區華龍國民小學","台中市大甲區西岐國民小學","台中市大甲區順天國民小學","台中市大肚區大忠國民小學","台中市大肚區大肚國民小學","台中市大肚區山陽國民小學","台中市大肚區永順國民小學","台中市大肚區瑞井國民小學","台中市大肚區瑞峰國民小學","台中市大肚區追分國民小學","台中市大里區健民國民小學","台中市大里區內新國民小學","台中市大里區塗城國民小學","台中市大里區大元國民小學","台中市大里區大里國民小學","台中市大里區崇光國民小學","台中市大里區永隆國民小學","台中市大里區瑞城國民小學","台中市大里區益民國民小學","台中市大里區立新國民小學","台中市大里區美群國民小學","台中市大里區草湖國民小學","台中市大雅區三和國民小學","台中市大雅區上楓國民小學","台中市大雅區六寶國民小學","台中市大雅區大明國民小學","台中市大雅區大雅國民小學","台中市大雅區文雅國民小學","台中市大雅區汝鎏國民小學","台中市大雅區陽明國民小學","台中市太平區中華國民小學","台中市太平區光隆國民小學","台中市太平區坪林國民小學","台中市太平區太平國民小學","台中市太平區宜欣國民小學","台中市太平區建平國民小學","台中市太平區新光國民小學","台中市太平區新平國民小學","台中市太平區新高國民小學","台中市太平區東平國民小學","台中市太平區東汴國民小學","台中市太平區車籠埔國民小學","台中市太平區長億國民小學","台中市太平區頭汴國民小學","台中市太平區黃竹國民小學","台中市新社區中和國民小學","台中市新社區協成國民小學","台中市新社區大南國民小學","台中市新社區大林國民小學","台中市新社區崑山國民小學","台中市新社區新社國民小學","台中市新社區東興國民小學","台中市新社區福民國民小學","台中市明德女子高級中學","台中市東勢區中山國民小學","台中市東勢區中科國民小學","台中市東勢區成功國民小學","台中市東勢區新成國民小學","台中市東勢區新盛國民小學","台中市東勢區明正國民小學","台中市東勢區東勢國民小學","台中市東勢區東新國民小學","台中市東勢區石城國民小學","台中市東勢區石角國民小學","台中市東區力行國民小學","台中市東區台中國民小學","台中市東區大智國民小學","台中市東區成功國民小學","台中市東區樂業國民小學","台中市東區進德國民小學","台中市梧棲區中正國民小學","台中市梧棲區中港國民小學","台中市梧棲區大德國民小學","台中市梧棲區梧南國民小學","台中市梧棲區梧棲國民小學","台中市梧棲區永寧國民小學","台中市沙鹿區公明國民小學","台中市沙鹿區公館國民小學","台中市沙鹿區北勢國民小學","台中市沙鹿區文光國民小學","台中市沙鹿區沙鹿國民小學","台中市沙鹿區竹林國民小學","台中市沙鹿區鹿峰國民小學","台中市清水區三田國民小學","台中市清水區吳厝國民小學","台中市清水區大楊國民小學","台中市清水區大秀國民小學","台中市清水區建國國民小學","台中市清水區東山國民小學","台中市清水區槺榔國民小學","台中市清水區清水國民小學","台中市清水區甲南國民小學","台中市清水區西寧國民小學","台中市清水區高美國民小學","台中市潭子區僑忠國民小學","台中市潭子區新興國民小學","台中市潭子區東寶國民小學","台中市潭子區潭子國民小學","台中市潭子區潭陽國民小學","台中市潭子區頭家國民小學","台中市烏日區九德國民小學","台中市烏日區五光國民小學","台中市烏日區僑仁國民小學","台中市烏日區喀哩國民小學","台中市烏日區旭光國民小學","台中市烏日區東園國民小學","台中市烏日區溪尾國民小學","台中市烏日區烏日國民小學","台中市石岡區土牛國民小學","台中市石岡區石岡國民小學","台中市神岡區圳堵國民小學","台中市神岡區岸裡國民小學","台中市神岡區社口國民小學","台中市神岡區神岡國民小學","台中市神岡區豐洲國民小學","台中市私立光華高級工業職業學校","台中市私立宜寧高級中學","台中市私立嶺東高級中學","台中市私立慎齋國民小學","台中市私立新民高級中學","台中市私立曉明女子高級中學","台中市私立衛道高級中學","台中市立三光國民中學","台中市立中山國民中學","台中市立中平國民中學","台中市立中港高級中學","台中市立五權國民中學","台中市立光復國民中小學","台中市立光德國民中學","台中市立光明國民中學","台中市立光榮國民中學","台中市立光正國民中學","台中市立公明國民中學","台中市立北勢國民中學","台中市立北新國民中學","台中市立南屯幼兒園","台中市立南屯托兒所","台中市立台中女子高級中等學校","台中市立台中家事商業高級中等學校","台中市立台中工業高級中等學校","台中市立台中特殊教育學校","台中市立台中第一高級中等學校","台中市立台中第二高級中等學校","台中市立后綜高級中學","台中市立后里國民中學","台中市立后里托兒所","台中市立向上國民中學","台中市立和平國民中學","台中市立和平托兒所","台中市立啟明學校","台中市立啟聰學校","台中市立善水國民中小學","台中市立四張犁國民中學","台中市立四箴國民中學","台中市立四育國民中學","台中市立外埔國民中學","台中市立外埔托兒所","台中市立大墩國民中學","台中市立大安國民中學","台中市立大安幼兒園","台中市立大安托兒所","台中市立大德國民中學","台中市立大業國民中學","台中市立大甲國民中學","台中市立大甲工業高級中等學校","台中市立大甲托兒所","台中市立大甲高級中等學校","台中市立大肚托兒所","台中市立大華國民中學","台中市立大道國民中學","台中市立大里托兒所","台中市立大里高級中學","台中市立大雅國民中學","台中市立大雅托兒所","台中市立太平國民中學","台中市立太平托兒所","台中市立安和國民中學","台中市立居仁國民中學","台中市立崇倫國民中學","台中市立崇德國民中學","台中市立忠明高級中學","台中市立惠文高級中學","台中市立成功國民中學","台中市立托兒所","台中市立文華高級中等學校","台中市立新光國民中學","台中市立新社托兒所","台中市立新社高級中學","台中市立日南國民中學","台中市立東勢國民中學","台中市立東勢工業高級中等學校","台中市立東勢托兒所","台中市立東山高級中學","台中市立東峰國民中學","台中市立東新國民中學","台中市立東華國民中學","台中市立梧棲國民中學","台中市立梧棲托兒所","台中市立梨山國民中小學","台中市立沙鹿國民中學","台中市立沙鹿工業高級中等學校","台中市立沙鹿托兒所","台中市立清水國民中學","台中市立清水托兒所","台中市立清水高級中等學校","台中市立清泉國民中學","台中市立清海國民中學","台中市立溪南國民中學","台中市立漢口國民中學","台中市立潭子國民中學","台中市立潭子幼兒園","台中市立潭子托兒所","台中市立潭秀國民中學","台中市立烏日國民中學","台中市立烏日托兒所","台中市立爽文國民中學","台中市立石岡國民中學","台中市立石岡托兒所","台中市立神圳國民中學","台中市立神岡國民中學","台中市立神岡托兒所","台中市立福科國民中學","台中市立立人國民中學","台中市立立新國民中學","台中市立育英國民中學","台中市立至善國民中學","台中市立萬和國民中學","台中市立西苑高級中學","台中市立豐南國民中學","台中市立豐原商業高級中等學校","台中市立豐原國民中學","台中市立豐原托兒所","台中市立豐原高級中等學校","台中市立豐東國民中學","台中市立豐陽國民中學","台中市立長億高級中學","台中市立雙十國民中學","台中市立霧峰國民中學","台中市立霧峰幼兒園","台中市立霧峰托兒所","台中市立霧峰農業工業高級中等學校","台中市立順天國民中學","台中市立鹿寮國民中學","台中市立黎明國民中學","台中市立龍井國民中學","台中市立龍井托兒所","台中市立龍津高級中等學校","台中市西區中正國民小學","台中市西區大勇國民小學","台中市西區大同國民小學","台中市西區忠信國民小學","台中市西區忠孝國民小學","台中市西區忠明國民小學","台中市西屯區上安國民小學","台中市西屯區上石國民小學","台中市西屯區何厝國民小學","台中市西屯區協和國民小學","台中市西屯區國安國民小學","台中市西屯區大仁國民小學","台中市西屯區大鵬國民小學","台中市西屯區惠來國民小學","台中市西屯區東海國民小學","台中市西屯區永安國民小學","台中市西屯區泰安國民小學","台中市西屯區西屯國民小學","台中市西屯區重慶國民小學","台中市西屯區長安國民小學","台中市豐原區南陽國民小學","台中市豐原區合作國民小學","台中市豐原區富春國民小學","台中市豐原區瑞穗國民小學","台中市豐原區福陽國民小學","台中市豐原區翁子國民小學","台中市豐原區葫蘆墩國民小學","台中市豐原區豐原國民小學","台中市豐原區豐村國民小學","台中市豐原區豐田國民小學","台中市霧峰區五福國民小學","台中市霧峰區僑榮國民小學","台中市霧峰區光正國民小學","台中市霧峰區吉峰國民小學","台中市霧峰區四德國民小學","台中市霧峰區峰谷國民小學","台中市霧峰區復興國民小學","台中市霧峰區桐林國民小學","台中市霧峰區萬豐國民小學","台中市霧峰區霧峰國民小學","台中市龍井區龍井國民小學","台中市龍井區龍山國民小學","台中市龍井區龍峰國民小學","台中市龍井區龍泉國民小學","台中市龍井區龍津國民小學","台中市龍井區龍海國民小學","台中市龍井區龍港國民小學","台中師範學院","台中技術學院","台中技術學院附設空中進修學院","台中技術學院附設進修學院","台中教育大學","台中教育大學附設實驗國民小學","台中特殊教育學校","台中科技大學","台中第一高級中等學校","台中第二高級中等學校","台中高工","台北南海實驗幼稚園","台北商業大學","台北商業技術學院","台北商業技術學院附設空中進修學院","台北城市科技大學","台北大學","台北市中山區中山國民小學","台北市中山區中正國民小學","台北市中山區五常國民小學","台北市中山區吉林國民小學","台北市中山區大佳國民小學","台北市中山區大直國民小學","台北市中山區懷生國民小學","台北市中山區永安國民小學","台北市中山區濱江國民小學","台北市中山區長安國民小學","台北市中山區長春國民小學","台北市中山社區大學","台北市中正區南門國民小學","台北市中正區忠孝國民小學","台北市中正區忠義國民小學","台北市中正區東門國民小學","台北市中正區河堤國民小學","台北市中正區螢橋國民小學","台北市中正社區大學","台北市信義區三興國民小學","台北市信義區信義國民小學","台北市信義區光復國民小學","台北市信義區博愛國民小學","台北市信義區吳興國民小學","台北市信義區永吉國民小學","台北市信義區永春國民小學","台北市信義區福德國民小學","台北市信義區興雅國民小學","台北市信義社區大學","台北市內湖區內湖國民小學","台北市內湖區南湖國民小學","台北市內湖區大湖國民小學","台北市內湖區康寧國民小學","台北市內湖區文湖國民小學","台北市內湖區新湖國民小學","台北市內湖區明湖國民小學","台北市內湖區東湖國民小學","台北市內湖區潭美國民小學","台北市內湖區碧湖國民小學","台北市內湖區西湖國民小學","台北市內湖區麗山國民小學","台北市內湖區麗湖國民小學","台北市內湖社區大學","台北市北投區北投國民小學","台北市北投區大屯國民小學","台北市北投區文化國民小學","台北市北投區文林國民小學","台北市北投區明德國民小學","台北市北投區桃源國民小學","台北市北投區清江國民小學","台北市北投區湖山國民小學","台北市北投區湖田國民小學","台北市北投區石牌國民小學","台北市北投區立農國民小學","台北市北投區義方國民小學","台北市北投區逸仙國民小學","台北市北投區關渡國民小學","台北市北投社區大學","台北市南港區修德國民小學","台北市南港區南港國民小學","台北市南港區成德國民小學","台北市南港區東新國民小學","台北市南港區玉成國民小學","台北市南港區胡適國民小學","台北市南港區舊莊國民小學","台北市南港社區大學","台北市博嘉實驗國民小學","台北市和平實驗國民小學","台北市國語實驗國民小學","台北市士林區三玉國民小學","台北市士林區劍潭國民小學","台北市士林區士東國民小學","台北市士林區士林國民小學","台北市士林區天母國民小學","台北市士林區富安國民小學","台北市士林區平等國民小學","台北市士林區文昌國民小學","台北市士林區溪山國民小學","台北市士林區百齡國民小學","台北市士林區社子國民小學","台北市士林區福林國民小學","台北市士林區芝山國民小學","台北市士林區葫蘆國民小學","台北市士林區蘭雅國民小學","台北市士林區陽明山國民小學","台北市士林區雙溪國民小學","台北市士林區雨聲國民小學","台北市士林區雨農國民小學","台北市士林社區大學","台北市大同區大同國民小學","台北市大同區大橋國民小學","台北市大同區大龍國民小學","台北市大同區太平國民小學","台北市大同區延平國民小學","台北市大同區日新國民小學","台北市大同區永樂國民小學","台北市大同區蓬萊國民小學","台北市大同區雙蓮國民小學","台北市大同社區大學","台北市大安區仁愛國民小學","台北市大安區公館國民小學","台北市大安區古亭國民小學","台北市大安區大安國民小學","台北市大安區幸安國民小學","台北市大安區建安國民小學","台北市大安區新生國民小學","台北市大安區金華國民小學","台北市大安區銘傳國民小學","台北市大安區龍安國民小學","台北市大安社區大學","台北市文山區力行國民小學","台北市文山區實踐國民小學","台北市文山區志清國民小學","台北市文山區指南國民小學","台北市文山區明道國民小學","台北市文山區景美國民小學","台北市文山區景興國民小學","台北市文山區木柵國民小學","台北市文山區武功國民小學","台北市文山區永建國民小學","台北市文山區溪口國民小學","台北市文山區興德國民小學","台北市文山區興華國民小學","台北市文山區興隆國民小學","台北市文山區萬福國民小學","台北市文山區萬興國民小學","台北市文山區萬芳國民小學","台北市文山區辛亥國民小學","台北市文山社區大學","台北市松山區三民國民小學","台北市松山區健康國民小學","台北市松山區敦化國民小學","台北市松山區松山國民小學","台北市松山區民族國民小學","台北市松山區民權國民小學","台北市松山區民生國民小學","台北市松山區西松國民小學","台北市松山社區大學","台北市泉源實驗國民小學","台北市私立中山小學","台北市私立中興高級中學","台北市私立光仁小學","台北市私立再興小學","台北市私立再興高級中學","台北市私立十信高級中學","台北市私立協和高級工商職業學校","台北市私立南華高級中學職業進修學校","台北市私立喬治高級工商職業學校","台北市私立大同高級中學","台北市私立大誠高級中學","台北市私立奎山國民中學","台北市私立奎山小學","台北市私立延平高級中學","台北市私立強恕高級中學","台北市私立復興實驗高級中學","台北市私立志仁高級中學職業進修學校","台北市私立志仁高級家事商業學校","台北市私立恕德高級家事商業職業學校","台北市私立惇敘高級工商職業學校","台北市私立文德女子高級中學","台北市私立新民小學","台北市私立方濟高級中學","台北市私立景文高級中學","台北市私立東山高級中學","台北市私立東方高級工商職業學校","台北市私立泰北高級中學","台北市私立滬江高級中學","台北市私立祐德高級中學","台北市私立稻江高級商業職業學校","台北市私立稻江高級護理家事職業學校","台北市私立立人國民中學","台北市私立立人小學","台北市私立立人高級中學","台北市私立育達高級商業家事職業學校","台北市私立華岡藝術學校","台北市私立華興小學","台北市私立華興高級中學","台北市私立薇閣小學","台北市私立薇閣高級中學","台北市私立衛理女子高級中學","台北市私立達人女子高級中學","台北市私立金甌女子高級中學","台北市私立開南高級商工職業學校","台北市私立開平餐飲職業學校","台北市私立靜修女子高級中學","台北市私立靜心國民中學","台北市私立靜心小學","台北市立三民國民中學","台北市立中山國民中學","台北市立中山女子高級中學","台北市立中山幼兒園","台北市立中山托兒所","台北市立中崙高級中學","台北市立中正國民中學","台北市立中正幼兒園","台北市立中正托兒所","台北市立中正高級中學","台北市立五常國民中學","台北市立仁愛國民中學","台北市立介壽國民中學","台北市立信義國民中學","台北市立信義幼兒園","台北市立信義托兒所","台北市立內湖國民中學","台北市立內湖幼兒園","台北市立內湖托兒所","台北市立內湖高級中學","台北市立內湖高級工業職業學校","台北市立北安國民中學","台北市立北投國民中學","台北市立北投幼兒園","台北市立北投托兒所","台北市立北政國民中學","台北市立南海實驗幼兒園","台北市立南港幼兒園","台北市立南港托兒所","台北市立南港高級中學","台北市立南港高級工業職業學校","台北市立南湖高級中學","台北市立南門國民中學","台北市立古亭國民中學","台北市立古亭托兒所","台北市立和平高級中學","台北市立啟明學校","台北市立啟智學校","台北市立啟聰學校","台北市立城中托兒所","台北市立士林國民中學","台北市立士林幼兒園","台北市立士林托兒所","台北市立士林高級商業職業學校","台北市立大同幼兒園","台北市立大同托兒所","台北市立大同高級中學","台北市立大學","台北市立大學附設實驗國民小學","台北市立大安國民中學","台北市立大安幼兒園","台北市立大安托兒所","台北市立大安高工附設高級工業職業進修學校","台北市立大安高級工業職業學校","台北市立大理高級中學","台北市立大直托兒所","台北市立大直高級中學","台北市立天母國民中學","台北市立實踐國民中學","台北市立師範學院","台北市立建國高級中學","台北市立建成國民中學","台北市立建成托兒所","台北市立弘道國民中學","台北市立復興高級中學","台北市立忠孝國民中學","台北市立懷生國民中學","台北市立成功托兒所","台北市立成功高級中學","台北市立成德國民中學","台北市立成淵高級中學","台北市立敦化國民中學","台北市立文山幼兒園","台北市立文山托兒所","台北市立文山特殊教育學校","台北市立新民國民中學","台北市立新興國民中學","台北市立明倫高級中學","台北市立明德國民中學","台北市立明湖國民中學","台北市立景美國民中學","台北市立景美女子高級中學","台北市立景興國民中學","台北市立木柵國民中學","台北市立木柵托兒所","台北市立木柵高級工業職業學校","台北市立東湖國民中學","台北市立松山幼兒園","台北市立松山托兒所","台北市立松山高級中學","台北市立松山高級商業家事職業學校","台北市立松山高級工農職業學校","台北市立格致國民中學","台北市立桃源國民中學","台北市立民族國民中學","台北市立民權國民中學","台北市立民生國民中學","台北市立民生托兒所","台北市立永吉國民中學","台北市立永春高級中學","台北市立濱江國民中學","台北市立瑠公國民中學","台北市立百齡高級中學","台北市立石牌國民中學","台北市立福安國民中學","台北市立第一女子高級中學","台北市立育成高級中學","台北市立育航幼兒園","台北市立育航幼稚園","台北市立自強托兒所","台北市立至善國民中學","台北市立興福國民中學","台北市立興雅國民中學","台北市立芳和實驗國民中學","台北市立華江高級中學","台北市立萬芳高級中學","台北市立萬華國民中學","台北市立萬華幼兒園","台北市立萬華托兒所","台北市立蘭州國民中學","台北市立蘭雅國民中學","台北市立螢橋國民中學","台北市立西松高級中學","台北市立西湖國民中學","台北市立誠正國民中學","台北市立重慶國民中學","台北市立金華國民中學","台北市立長安國民中學","台北市立關渡國民中學","台北市立陽明高級中學","台北市立雙園區南海幼稚園","台北市立雙園國民中學","台北市立雙園托兒所","台北市立體育學院","台北市立麗山國民中學","台北市立麗山高級中學","台北市立龍山國民中學","台北市立龍山托兒所","台北市立龍門國民中學","台北市職能發展學院","台北市萬華區大理國民小學","台北市萬華區新和國民小學","台北市萬華區東園國民小學","台北市萬華區福星國民小學","台北市萬華區老松國民小學","台北市萬華區華江國民小學","台北市萬華區萬大國民小學","台北市萬華區西園國民小學","台北市萬華區西門國民小學","台北市萬華區雙園國民小學","台北市萬華區龍山國民小學","台北市萬華社區大學","台北師範學院","台北技術學院","台北教育大學","台北教育大學附設實驗國民小學","台北海洋技術學院","台北科大附設進修學院","台北科技大學","台北科技大學附屬桃園農工高級中等學校","台北科技大學附設進修學院","台北藝術大學","台北護理健康大學","台北護理學院","台北農林專門學校","台北醫學大學","台北醫學院","台北高等農林學校","台南啟智學校","台南大學","台南大學附屬啟聰學校","台南大學附屬高級中學","台南大學附設實驗國民小學","台南女中","台南女子技術學院","台南女子高級中學","台南家齊高級中等學校","台南市七股區七股國民小學","台南市七股區三股國民小學","台南市七股區光復生態實驗小學","台南市七股區大文國民小學","台南市七股區建功國民小學","台南市七股區後港國民小學","台南市七股區樹林國民小學","台南市七股區竹橋國民小學","台南市七股區篤加國民小學","台南市七股區龍山國民小學","台南市下營區下營國民小學","台南市下營區中營國民小學","台南市下營區東興國民小學","台南市下營區甲中國民小學","台南市下營區賀建國民小學","台南市中區第一幼稚園","台南市中區第二幼稚園","台南市中西區協進國民小學","台南市中西區忠義國民小學","台南市中西區成功國民小學","台南市中西區永福國民小學","台南市中西區進學國民小學","台南市亞洲高級餐旅職業學校","台南市仁德區仁和國民小學","台南市仁德區仁德國民小學","台南市仁德區依仁國民小學","台南市仁德區大甲國民小學","台南市仁德區德南國民小學","台南市仁德區文賢國民小學","台南市仁德區虎山實驗小學","台南市仁德區長興國民小學","台南市佳里區仁愛國民小學","台南市佳里區佳興國民小學","台南市佳里區佳里國民小學","台南市佳里區信義國民小學","台南市佳里區塭內國民小學","台南市佳里區子龍國民小學","台南市佳里區延平國民小學","台南市佳里區通興國民小學","台南市六信高級中學","台南市六甲區六甲國民小學","台南市六甲區林鳳國民小學","台南市北區公園國民小學","台南市北區大光國民小學","台南市北區大港國民小學","台南市北區文元國民小學","台南市北區立人國民小學","台南市北區第三幼稚園","台南市北區第四幼稚園","台南市北區賢北國民小學","台南市北區開元國民小學","台南市北門區三慈國民小學","台南市北門區北門國民小學","台南市北門區文山國民小學","台南市北門區蚵寮國民小學","台南市北門區錦湖國民小學","台南市北門區雙春國民小學","台南市南化區北寮國民小學","台南市南化區南化國民小學","台南市南化區玉山國民小學","台南市南化區瑞峰國民小學","台南市南化區西埔國民小學","台南市南區喜樹國民小學","台南市南區志開實驗小學","台南市南區新興國民小學","台南市南區日新國民小學","台南市南區永華國民小學","台南市南區省躬國民小學","台南市南區龍崗國民小學","台南市善化區善化國民小學","台南市善化區善糖國民小學","台南市善化區大同國民小學","台南市善化區大成國民小學","台南市善化區小新國民小學","台南市善化區茄拔國民小學","台南市善化區陽明國民小學","台南市大內區二溪國民小學","台南市大內區大內國民小學","台南市天主教私立寶仁國民小學","台南市學甲區中洲國民小學","台南市學甲區學甲國民小學","台南市學甲區宅港國民小學","台南市學甲區東陽國民小學","台南市學甲區頂洲國民小學","台南市安南區南興國民小學","台南市安南區和順國民小學","台南市安南區土城國民小學","台南市安南區學東國民小學","台南市安南區安佃國民小學","台南市安南區安慶國民小學","台南市安南區安順國民小學","台南市安南區海佃國民小學","台南市安南區海東國民小學","台南市安南區鎮海國民小學","台南市安南區長安國民小學","台南市安南區青草國民小學","台南市安南區顯宮國民小學","台南市安定區南安國民小學","台南市安定區南興國民小學","台南市安定區安定國民小學","台南市安平區億載國民小學","台南市安平區安平國民小學","台南市安平區新南國民小學","台南市安平區石門國民小學","台南市安平區西門實驗小學","台南市官田區嘉南國民小學","台南市官田區官田國民小學","台南市官田區渡拔國民小學","台南市官田區隆田國民小學","台南市將軍區將軍國民小學","台南市將軍區漚汪國民小學","台南市將軍區苓和國民小學","台南市將軍區長平國民小學","台南市將軍區鯤鯓國民小學","台南市山上區山上國民小學","台南市左鎮區光榮國民小學","台南市左鎮區左鎮國民小學","台南市後壁區安溪國民小學","台南市後壁區後壁國民小學","台南市後壁區新嘉國民小學","台南市後壁區新東國民小學","台南市後壁區樹人國民小學","台南市後壁區永安國民小學","台南市後壁區菁寮國民小學","台南市新化區口埤實驗小學","台南市新化區大新國民小學","台南市新化區新化國民小學","台南市新化區正新國民小學","台南市新化區那拔國民小學","台南市新市區大社國民小學","台南市新市區新市國民小學","台南市新營區公誠國民小學","台南市新營區南梓實驗小學","台南市新營區土庫國民小學","台南市新營區新橋國民小學","台南市新營區新民國民小學","台南市新營區新泰國民小學","台南市新營區新營國民小學","台南市新營區新生國民小學","台南市新營區新興國民小學","台南市新營區新進國民小學","台南市東區勝利國民小學","台南市東區博愛國民小學","台南市東區大同國民小學","台南市東區崇學國民小學","台南市東區崇明國民小學","台南市東區復興國民小學","台南市東區德高國民小學","台南市東區東光國民小學","台南市東區第五幼稚園","台南市東區裕文國民小學","台南市東山區吉貝耍國民小學","台南市東山區東原國民小學","台南市東山區東山國民小學","台南市東山區聖賢國民小學","台南市東山區青山國民小學","台南市柳營區太康國民小學","台南市柳營區新山國民小學","台南市柳營區果毅國民小學","台南市柳營區柳營國民小學","台南市柳營區重溪國民小學","台南市楠西區楠西國民小學","台南市歸仁區保西國民小學","台南市歸仁區大潭國民小學","台南市歸仁區文化國民小學","台南市歸仁區歸仁國民小學","台南市歸仁區歸南國民小學","台南市歸仁區紅瓦厝國民小學","台南市永康區三村國民小學","台南市永康區五王國民小學","台南市永康區勝利國民小學","台南市永康區大橋國民小學","台南市永康區大灣國民小學","台南市永康區崑山國民小學","台南市永康區復興國民小學","台南市永康區永信國民小學","台南市永康區永康國民小學","台南市永康區西勢國民小學","台南市永康區龍潭國民小學","台南市玉井區層林國民小學","台南市玉井區玉井國民小學","台南市白河區仙草國民小學","台南市白河區內角國民小學","台南市白河區大竹國民小學","台南市白河區河東國民小學","台南市白河區玉豐國民小學","台南市白河區白河國民小學","台南市白河區竹門國民小學","台南市私立光華女子高級中學","台南市私立南英高級商工職業學校","台南市私立崑山高級中學","台南市私立建業高級中學","台南市私立德光女子高級中學","台南市私立慈幼高級工商職業學校","台南市私立慈濟國民小學","台南市私立慈濟高級中學","台南市私立昭明國民中學","台南市私立瀛海高級中學","台南市私立聖功女子高級中學","台南市私立長榮女子高級中學","台南市私立長榮高級中學","台南市立七股托兒所","台南市立下營國民中學","台南市立下營幼兒園","台南市立下營托兒所","台南市立中山國民中學","台南市立仁德國民中學","台南市立仁德幼兒園","台南市立仁德托兒所","台南市立仁德文賢國民中學","台南市立佳興國民中學","台南市立佳里國民中學","台南市立六甲國民中學","台南市立六甲幼兒園","台南市立六甲托兒所","台南市立北門國民中學","台南市立南化國民中學","台南市立南寧高級中學","台南市立南新國民中學","台南市立和順國民中學","台南市立善化國民中學","台南市立善化托兒所","台南市立土城高級中學","台南市立大內國民中學","台南市立大成國民中學","台南市立大橋國民中學","台南市立大灣高級中學","台南市立太子國民中學","台南市立學甲國民中學","台南市立學甲幼兒園","台南市立學甲托兒所","台南市立安南國民中學","台南市立安定國民中學","台南市立安定托兒所","台南市立安平國民中學","台南市立安順國民中學","台南市立官田國民中學","台南市立將軍國民中學","台南市立將軍幼兒園","台南市立將軍托兒所","台南市立小康幼兒園","台南市立小康托兒所","台南市立山上國民中學","台南市立崇明國民中學","台南市立左鎮國民中學","台南市立延平國民中學","台南市立建興國民中學","台南市立後壁國民中學","台南市立後港國民中學","台南市立後甲國民中學","台南市立復興國民中學","台南市立忠孝國民中學","台南市立成功國民中學","台南市立文賢國民中學","台南市立新化國民中學","台南市立新化幼兒園","台南市立新化托兒所","台南市立新市國民中學","台南市立新市幼兒園","台南市立新市托兒所","台南市立新東國民中學","台南市立新興國民中學","台南市立東原國民中學","台南市立東山國民中學","台南市立柳營國民中學","台南市立楠西國民中學","台南市立歸仁國民中學","台南市立歸仁幼兒園","台南市立歸仁托兒所","台南市立民德國民中學","台南市立永仁高級中學","台南市立永康國民中學","台南市立永康托兒所","台南市立沙崙國民中學","台南市立海佃國民中學","台南市立玉井國民中學","台南市立白河國民中學","台南市立白河托兒所","台南市立竹橋國民中學","台南市立第一幼兒園","台南市立第一幼稚園","台南市立第一托兒所","台南市立第三幼兒園","台南市立第三幼稚園","台南市立第三托兒所","台南市立第二幼兒園","台南市立第二托兒所","台南市立第五幼兒園","台南市立第五幼稚園","台南市立第五托兒所","台南市立第六幼兒園","台南市立第六托兒所","台南市立第四幼兒園","台南市立第四托兒所","台南市立菁寮國民中學","台南市立西港國民中學","台南市立金城國民中學","台南市立關廟國民中學","台南市立關廟幼兒園","台南市立關廟托兒所","台南市立鹽水國民中學","台南市立麻豆國民中學","台南市立麻豆幼兒園","台南市立麻豆托兒所","台南市立龍崎國民中學","台南市立龍崎托兒所","台南市西港區後營國民小學","台南市西港區成功國民小學","台南市西港區松林國民小學","台南市西港區港東國民小學","台南市西港區西港國民小學","台南市關廟區五甲國民小學","台南市關廟區保東國民小學","台南市關廟區崇和國民小學","台南市關廟區文和實驗小學","台南市關廟區新光國民小學","台南市關廟區深坑國民小學","台南市關廟區關廟國民小學","台南市鹽水區仁光國民小學","台南市鹽水區坔頭港國民小學","台南市鹽水區岸內國民小學","台南市鹽水區文昌國民小學","台南市鹽水區月津國民小學","台南市鹽水區歡雅國民小學","台南市鹽水區竹埔國民小學","台南市鹽水區鹽水國民小學","台南市麻豆區北勢國民小學","台南市麻豆區培文國民小學","台南市麻豆區大山國民小學","台南市麻豆區安業國民小學","台南市麻豆區文正國民小學","台南市麻豆區港尾國民小學","台南市麻豆區紀安國民小學","台南市麻豆區麻豆國民小學","台南市龍崎區龍崎國民小學","台南師範學院","台南應用科技大學","台南海事","台南第一高級中學","台南第二高級中學","台南藝術大學","台南藝術學院","台南護理專科學校","台南附中","台南高商","台南高工","台南高等工業學校","台南高級商業職業學校","台南高級工業職業學校","台南高級海事水產職業學校","台坂國小","台坂國民小學","台大","台東大學","台東大學附屬特殊教育學校","台東大學附屬體育高級中學","台東大學附設實驗國民小學","台東女中","台東女子高中","台東女子高級中學","台東女高","台東專科學校","台東市立托兒所","台東師範學院","台東幼稚園","台東縣卑南鄉初鹿國民小學","台東縣卑南鄉利嘉國民小學","台東縣卑南鄉大南國民小學","台東縣卑南鄉太平國民小學","台東縣卑南鄉富山國民小學","台東縣卑南鄉東成國民小學","台東縣卑南鄉溫泉國民小學","台東縣卑南鄉立托兒所","台東縣卑南鄉賓朗國民小學","台東縣台東市仁愛國民小學","台東縣台東市光明國民小學","台東縣台東市卑南國民小學","台東縣台東市南王國民小學","台東縣台東市台東幼稚園","台東縣台東市富岡國民小學","台東縣台東市寶桑國民小學","台東縣台東市岩灣國民小學","台東縣台東市康樂國民小學","台東縣台東市建和國民小學","台東縣台東市復興國民小學","台東縣台東市新園國民小學","台東縣台東市新生國民小學","台東縣台東市東海國民小學","台東縣台東市知本國民小學","台東縣台東市立托兒所","台東縣台東市豐年國民小學","台東縣台東市豐榮國民小學","台東縣台東市豐源國民小學","台東縣台東市豐田國民小學","台東縣台東市豐里國民小學","台東縣台東市馬蘭國民小學","台東縣大武鄉大武國民小學","台東縣大武鄉大鳥國民小學","台東縣大武鄉尚武國民小學","台東縣大武鄉立托兒所","台東縣太麻里鄉三和國民小學","台東縣太麻里鄉大溪國民小學","台東縣太麻里鄉大王國民小學","台東縣太麻里鄉立托兒所","台東縣太麻里鄉美和國民小學","台東縣太麻里鄉香蘭國民小學","台東縣延平鄉桃源國民小學","台東縣延平鄉武陵國民小學","台東縣延平鄉立托兒所","台東縣延平鄉紅葉國民小學","台東縣延平鄉鸞山國民小學","台東縣成功鎮三仙國民小學","台東縣成功鎮三民國民小學","台東縣成功鎮信義國民小學","台東縣成功鎮忠孝國民小學","台東縣成功鎮成功國民小學","台東縣成功鎮立托兒所","台東縣東河鄉北源國民小學","台東縣東河鄉東河國民小學","台東縣東河鄉泰源國民小學","台東縣東河鄉立托兒所","台東縣東河鄉興隆國民小學","台東縣東河鄉都蘭國民小學","台東縣池上鄉大坡國民小學","台東縣池上鄉福原國民小學","台東縣池上鄉立托兒所","台東縣池上鄉萬安國民小學","台東縣海端鄉初來國民小學","台東縣海端鄉加拿國民小學","台東縣海端鄉崁頂國民小學","台東縣海端鄉廣原國民小學","台東縣海端鄉海端國民小學","台東縣海端鄉立托兒所","台東縣海端鄉錦屏國民小學","台東縣海端鄉霧鹿國民小學","台東縣私立公東高級工業職業學校","台東縣私立育仁高級中學","台東縣立初鹿國民中學","台東縣立卑南國民中學","台東縣立大武國民中學","台東縣立大王國民中學","台東縣立寶桑國民中學","台東縣立托兒所","台東縣立新港國民中學","台東縣立新生國民中學","台東縣立新生幼兒園","台東縣立東海國民中學","台東縣立桃源國民中學","台東縣立池上國民中學","台東縣立泰源國民中學","台東縣立海端國民中學","台東縣立瑞源國民中學","台東縣立知本國民中學","台東縣立綠島國民中學","台東縣立蘭嶼高級中學","台東縣立豐田國民中學","台東縣立賓茂國民中學","台東縣立都蘭國民中學","台東縣立長濱國民中學","台東縣立關山國民中學","台東縣立鹿野國民中學","台東縣綠島鄉公館國民小學","台東縣綠島鄉立托兒所","台東縣綠島鄉綠島國民小學","台東縣蘭嶼鄉朗島國民小學","台東縣蘭嶼鄉東清國民小學","台東縣蘭嶼鄉椰油國民小學","台東縣蘭嶼鄉立托兒所","台東縣蘭嶼鄉蘭嶼國民小學","台東縣達仁鄉台坂國民小學","台東縣達仁鄉土坂國民小學","台東縣達仁鄉安朔國民小學","台東縣達仁鄉立托兒所","台東縣金峰鄉介達國民小學","台東縣金峰鄉嘉蘭國民小學","台東縣金峰鄉新興國民小學","台東縣金峰鄉立托兒所","台東縣金峰鄉賓茂國民小學","台東縣長濱鄉三間國民小學","台東縣長濱鄉寧埔國民小學","台東縣長濱鄉樟原國民小學","台東縣長濱鄉立托兒所","台東縣長濱鄉竹湖國民小學","台東縣長濱鄉長濱國民小學","台東縣關山鎮德高國民小學","台東縣關山鎮月眉國民小學","台東縣關山鎮立托兒所","台東縣關山鎮關山國民小學","台東縣關山鎮電光國民小學","台東縣鹿野鄉永安國民小學","台東縣鹿野鄉瑞源國民小學","台東縣鹿野鄉瑞豐國民小學","台東縣鹿野鄉立托兒所","台東縣鹿野鄉鹿野國民小學","台東縣鹿野鄉龍田國民小學","台東體中","台東高商","台東高級中學","台東高級商業職業學校","台灣大學","台灣工業技術學院","台灣師範大學","台灣師範大學附屬高級中學","台灣戲曲學院","台灣教育學院","台灣海洋大學","台灣海洋學院","台灣科技大學","台灣藝術大學","台灣藝術學院","台灣觀光學院","台灣警察專科學校","台灣體大","台灣體育學院","台灣體育運動大學","台科","台科大","台興國小","台興國民小學","台藝大","台西國中","台西國小","台西國民中學","台西國民小學","台西鄉立托兒所","台觀","台觀院","台體","史港國小","史港國民小學","右昌國中","右昌國小","右昌國民中學","右昌國民小學","司法官學院","合作國小","合作國民小學","合橫國小","合橫國民小學","合興國小","合興國民小學","吉安國中","吉安國小","吉安國民中學","吉安國民小學","吉安鄉立托兒所","吉峰國小","吉峰國民小學","吉慶國小","吉慶國民小學","吉東國小","吉東國民小學","吉林國小","吉林國民小學","吉洋國小","吉洋國民小學","吉貝國中","吉貝國小","吉貝國民中學","吉貝國民小學","吉貝耍國小","吉貝耍國民小學","同仁國小","同仁國民小學","同光國小","同光國民小學","同安國小","同安國民小學","同富國中","同富國小","同富國民中學","同富國民小學","同德國中","同德國小","同德國民中學","同德國民小學","同德家事商業職業學校","同德家商","同榮國小","同榮國民小學","同樂國小","同樂國民小學","同濟","同濟高級中學","名崗國小","名崗國民小學","名間國中","名間國小","名間國民中學","名間國民小學","名間鄉立托兒所","后厝國小","后厝國民小學","后庄國小","后庄國民小學","后綜高中","后綜高級中學","后里國中","后里國小","后里國民中學","后里國民小學","后里托兒所","向上國中","向上國民中學","君毅","君毅高級中學","吳厝國小","吳厝國民小學","吳江國小","吳江國民小學","吳沙國中","吳沙國民中學","吳興國小","吳興國民小學","吳鳳幼兒園","吳鳳幼稚園","吳鳳技術學院","吳鳳科大","吳鳳科技大學","和仁國小","和仁國民小學","和平國中","和平國小","和平國民中學","和平國民小學","和平實小","和平實驗國民小學","和平托兒所","和平高中","和平高級中學","和春技術學院","和東國小","和東國民小學","和睦國小","和睦國民小學","和美國小","和美國民小學","和美實校","和美實驗學校","和美鎮立托兒所","和美高級中學","和群國中","和群國民中學","和興國小","和興國民小學","和順國中","和順國小","和順國民中學","和順國民小學","和高","員山國中","員山國小","員山國民中學","員山國民小學","員山鄉立托兒所","員崠國小","員崠國民小學","員東國中","員東國小","員東國民中學","員東國民小學","員林國中","員林國小","員林國民中學","員林國民小學","員林家商","員林崇實高級工業職業學校","員林鎮立托兒所","員林高級中學","員林高級家事商業職業學校","員林高級農工職業學校","員樹林國小","員樹林國民小學","員農","員高","唐榮國小","唐榮國民小學","商達科大","啟文國小","啟文國民小學","啟新國中","啟新國民中學","啟明國小","啟明國民小學","啟明學校","啟智學校","啟聰學校","喀哩國小","喀哩國民小學","善化國中","善化國小","善化國民中學","善化國民小學","善化托兒所","善化高中","善化高級中學","善水中小學","善水國民中小學","善糖國小","善糖國民小學","喜樹國小","喜樹國民小學","喬治商職","喬治工商","喬治高級工商職業學校","嘉中","嘉北國小","嘉北國民小學","嘉南國小","嘉南國民小學","嘉南藥理學院","嘉南藥理科技大學","嘉南高中","嘉南高級中學","嘉和國小","嘉和國民小學","嘉商","嘉大附小","嘉女","嘉寶國小","嘉寶國民小學","嘉師","嘉新國中","嘉新國民中學","嘉義國中","嘉義國民中學","嘉義大學","嘉義大學附設實驗國民小學","嘉義女中","嘉義女子高級中學","嘉義家職","嘉義市世賢國民小學","嘉義市僑平國民小學","嘉義市北園國民小學","嘉義市博愛國民小學","嘉義市嘉北國民小學","嘉義市垂楊國民小學","嘉義市大同國民小學","嘉義市宣信國民小學","嘉義市崇文國民小學","嘉義市志航國民小學","嘉義市文雅國民小學","嘉義市林森國民小學","嘉義市民族國民小學","嘉義市港坪國民小學","嘉義市私立仁義高級中學","嘉義市私立嘉華高級中學","嘉義市私立大同高級商業職業學校","嘉義市私立宏仁女子高級中學","嘉義市私立東吳高級工業家事職業學校","嘉義市私立神州高級中學進修學校","嘉義市私立興華高級中學","嘉義市私立輔仁高級中學","嘉義市立仁高級中學","嘉義市立北園國民中學","嘉義市立北興國民中學","嘉義市立南興國民中學","嘉義市立吳鳳幼兒園","嘉義市立吳鳳幼稚園","嘉義市立嘉義國民中學","嘉義市立大業國民中學","嘉義市立幸福幼兒園","嘉義市立復國幼兒園","嘉義市立復國幼稚園","嘉義市立民生國民中學","嘉義市立玉山國民中學","嘉義市立蘭潭國民中學","嘉義市精忠國民小學","嘉義市育人國民小學","嘉義市興嘉國民小學","嘉義市興安國民小學","嘉義市蘭潭國民小學","嘉義師範學院","嘉義技術學院","嘉義特教","嘉義特殊教育學校","嘉義縣中埔鄉中埔國民小學","嘉義縣中埔鄉中山國民小學","嘉義縣中埔鄉同仁國民小學","嘉義縣中埔鄉和睦國民小學","嘉義縣中埔鄉和興國民小學","嘉義縣中埔鄉大有國民小學","嘉義縣中埔鄉沄水國民小學","嘉義縣中埔鄉灣潭國民小學","嘉義縣中埔鄉社口國民小學","嘉義縣中埔鄉立托兒所","嘉義縣中埔鄉頂六國民小學","嘉義縣六腳鄉六美國民小學","嘉義縣六腳鄉六腳國民小學","嘉義縣六腳鄉北美國民小學","嘉義縣六腳鄉更寮國民小學","嘉義縣六腳鄉灣內國民小學","嘉義縣六腳鄉立托兒所","嘉義縣六腳鄉蒜頭國民小學","嘉義縣嘉義市復國幼稚園","嘉義縣大林鎮三和國民小學","嘉義縣大林鎮中林國民小學","嘉義縣大林鎮大林國民小學","嘉義縣大林鎮平林國民小學","嘉義縣大林鎮排路國民小學","嘉義縣大林鎮社團國民小學","嘉義縣大林鎮立托兒所","嘉義縣太保市南新國民小學","嘉義縣太保市太保國民小學","嘉義縣太保市安東國民小學","嘉義縣太保市新埤國民小學","嘉義縣太保市立托兒所","嘉義縣布袋鎮好美國民小學","嘉義縣布袋鎮布新國民小學","嘉義縣布袋鎮布袋國民小學","嘉義縣布袋鎮新塭國民小學","嘉義縣布袋鎮新岑國民小學","嘉義縣布袋鎮景山國民小學","嘉義縣布袋鎮永安國民小學","嘉義縣布袋鎮立托兒所","嘉義縣布袋鎮貴林國民小學","嘉義縣布袋鎮過溝國民小學","嘉義縣新港鄉古民國民小學","嘉義縣新港鄉安和國民小學","嘉義縣新港鄉復興國民小學","嘉義縣新港鄉文昌國民小學","嘉義縣新港鄉新港國民小學","嘉義縣新港鄉月眉國民小學","嘉義縣新港鄉立托兒所","嘉義縣朴子市大同國民小學","嘉義縣朴子市大鄉國民小學","嘉義縣朴子市朴子國民小學","嘉義縣朴子市松梅國民小學","嘉義縣朴子市祥和國民小學","嘉義縣朴子市立托兒所","嘉義縣朴子市竹村國民小學","嘉義縣朴子市雙溪國民小學","嘉義縣東石鄉三江國民小學","嘉義縣東石鄉下楫國民小學","嘉義縣東石鄉塭港國民小學","嘉義縣東石鄉東石國民小學","嘉義縣東石鄉港墘國民小學","嘉義縣東石鄉立托兒所","嘉義縣東石鄉網寮國民小學","嘉義縣東石鄉龍崗國民小學","嘉義縣東石鄉龍港國民小學","嘉義縣梅山鄉仁和國民小學","嘉義縣梅山鄉大南國民小學","嘉義縣梅山鄉太和國民小學","嘉義縣梅山鄉太平國民小學","嘉義縣梅山鄉太興國民小學","嘉義縣梅山鄉梅北國民小學","嘉義縣梅山鄉梅圳國民小學","嘉義縣梅山鄉梅山國民小學","嘉義縣梅山鄉瑞峰國民小學","嘉義縣梅山鄉瑞里國民小學","嘉義縣梅山鄉立托兒所","嘉義縣民雄鄉三興國民小學","嘉義縣民雄鄉大崎國民小學","嘉義縣民雄鄉東榮國民小學","嘉義縣民雄鄉松山國民小學","嘉義縣民雄鄉民雄國民小學","嘉義縣民雄鄉福樂國民小學","嘉義縣民雄鄉秀林國民小學","嘉義縣民雄鄉立托兒所","嘉義縣民雄鄉興中國民小學","嘉義縣民雄鄉菁埔國民小學","嘉義縣水上鄉北回國民小學","嘉義縣水上鄉南靖國民小學","嘉義縣水上鄉大崙國民小學","嘉義縣水上鄉忠和國民小學","嘉義縣水上鄉成功國民小學","嘉義縣水上鄉柳林國民小學","嘉義縣水上鄉水上國民小學","嘉義縣水上鄉立托兒所","嘉義縣水上鄉義興國民小學","嘉義縣溪口鄉柳溝國民小學","嘉義縣溪口鄉柴林國民小學","嘉義縣溪口鄉溪口國民小學","嘉義縣溪口鄉立托兒所","嘉義縣溪口鄉美林國民小學","嘉義縣番路鄉內甕國民小學","嘉義縣番路鄉大湖國民小學","嘉義縣番路鄉民和國民小學","嘉義縣番路鄉立托兒所","嘉義縣番路鄉隙頂國民小學","嘉義縣番路鄉黎明國民小學","嘉義縣私立協同高級中學","嘉義縣私立協志高級工商職業學校","嘉義縣私立同濟高級中學","嘉義縣私立嘉南高級中學","嘉義縣私立弘德高級工商職業學校","嘉義縣私立明華高級中學","嘉義縣私立萬能高級工商職業學校","嘉義縣立中埔國民中學","嘉義縣立六嘉國民中學","嘉義縣立嘉新國民中學","嘉義縣立大吉國民中學","嘉義縣立大埔國民中小學","嘉義縣立大林國民中學","嘉義縣立太保國民中學","嘉義縣立布袋國民中學","嘉義縣立忠和國民中學","嘉義縣立新港國民中學","嘉義縣立昇平國民中學","嘉義縣立朴子國民中學","嘉義縣立東榮國民中學","嘉義縣立東石國民中學","嘉義縣立梅山國民中學","嘉義縣立民和國民中學","嘉義縣立民雄國民中學","嘉義縣立水上國民中學","嘉義縣立永慶高級中學","嘉義縣立溪口國民中學","嘉義縣立竹崎高級中學","嘉義縣立義竹國民中學","嘉義縣立過溝國民中學","嘉義縣立阿里山國民中小學","嘉義縣立鹿草國民中學","嘉義縣竹崎鄉中和國民小學","嘉義縣竹崎鄉中興國民小學","嘉義縣竹崎鄉光華國民小學","嘉義縣竹崎鄉內埔國民小學","嘉義縣竹崎鄉圓崇國民小學","嘉義縣竹崎鄉桃源國民小學","嘉義縣竹崎鄉沙坑國民小學","嘉義縣竹崎鄉立托兒所","嘉義縣竹崎鄉竹崎國民小學","嘉義縣竹崎鄉義仁國民小學","嘉義縣竹崎鄉鹿滿國民小學","嘉義縣竹崎鄉龍山國民小學","嘉義縣義竹鄉光榮國民小學","嘉義縣義竹鄉南興國民小學","嘉義縣義竹鄉和順國民小學","嘉義縣義竹鄉義竹國民小學","嘉義縣義竹鄉過路國民小學","嘉義縣豐山實驗教育學校","嘉義縣阿里山鄉來吉國民小學","嘉義縣阿里山鄉十字國民小學","嘉義縣阿里山鄉山美國民小學","嘉義縣阿里山鄉新美國民小學","嘉義縣阿里山鄉立托兒所","嘉義縣阿里山鄉茶山國民小學","嘉義縣阿里山鄉達邦國民小學","嘉義縣阿里山鄉香林國民小學","嘉義縣鹿草鄉下潭國民小學","嘉義縣鹿草鄉後塘國民小學","嘉義縣鹿草鄉碧潭國民小學","嘉義縣鹿草鄉竹園國民小學","嘉義縣鹿草鄉重寮國民小學","嘉義縣鹿草鄉鹿草國民小學","嘉義高商","嘉義高工","嘉義高級中學","嘉義高級商業職業學校","嘉義高級家事職業學校","嘉義高級工業職業學校","嘉興國中","嘉興國小","嘉興國民中學","嘉興國民小學","嘉華高中","嘉華高級中學","嘉藥","嘉蘭國小","嘉蘭國民小學","嘉誠國小","嘉誠國民小學","嘉里國小","嘉里國民小學","四季國小","四季國民小學","四張犁國中","四張犁國小","四張犁國民中學","四張犁國民小學","四德國小","四德國民小學","四林國小","四林國民小學","四湖國中","四湖國小","四湖國民中學","四湖國民小學","四湖鄉立托兒所","四箴國中","四箴國民中學","四結國小","四結國民小學","四維國小","四維國民小學","四維高中","四維高級中學","四育國中","四育國民中學","國光商工","國光國中","國光國小","國光國民中學","國光國民小學","國光高中","國光高級中學","國光高級商工職業學校","國姓國中","國姓國小","國姓國民中學","國姓國民小學","國姓鄉立托兒所","國安國小","國安國民小學","國家文官學院","國昌國中","國昌國民中學","國民中學","國泰國小","國泰國民小學","國福國小","國福國民小學","國立中央大學","國立中央大學醫學院","國立中央大學附屬中壢高級中學","國立中山大學","國立中山大學附屬國光高級中學","國立中正大學","國立中科實驗高級中學","國立中興大學","國立中興大學附屬台中高級農業職業學校","國立中興大學附屬臺中高級農業職業學校","國立中興大學附屬高級中學","國立中興高級中學","國立二林高級工商職業學校","國立交通大學","國立仁愛高級農業職業學校","國立佳冬高級農業職業學校","國立光復高級商工職業學校","國立內埔高級農工職業學校","國立勤益技術學院","國立勤益技術學院附設進修學院","國立勤益科技大學","國立勤益科技大學附設進修學院","國立北斗高級家事商業職業學校","國立北港高級中學","國立北港高級農工職業學校","國立北門高級中學","國立北門高級農工職業學校","國立卓蘭高級中等學校","國立南投特殊教育學校","國立南投高級中學","國立南投高級商業職業學校","國立南科國際實驗高級中學","國立台中師範學院","國立台中技術學院","國立台中技術學院附設空中進修學院","國立台中技術學院附設進修學院","國立台中教育大學","國立台中教育大學附設實驗國民小學","國立台中科技大學","國立台北商業大學","國立台北商業技術學院","國立台北商業技術學院附設空中進修學院","國立台北大學","國立台北師範學院","國立台北技術學院","國立台北教育大學","國立台北教育大學附設實驗國民小學","國立台北科技大學","國立台北科技大學附屬桃園農工高級中等學校","國立台北科技大學附設進修學院","國立台北藝術大學","國立台北護理健康大學","國立台北護理學院","國立台南啟智學校","國立台南大學","國立台南大學附屬啟聰學校","國立台南大學附屬高級中學","國立台南大學附設實驗國民小學","國立台南女子高級中學","國立台南家齊高級中等學校","國立台南師範學院","國立台南第一高級中學","國立台南第二高級中學","國立台南藝術大學","國立台南藝術學院","國立台南護理專科學校","國立台南高級商業職業學校","國立台南高級工業職業學校","國立台南高級海事水產職業學校","國立台東大學","國立台東大學附屬特殊教育學校","國立台東大學附屬體育高級中學","國立台東大學附設實驗國民小學","國立台東女子高級中學","國立台東專科學校","國立台東師範學院","國立台東高級中學","國立台東高級商業職業學校","國立台灣大學","國立台灣工業技術學院","國立台灣師範大學","國立台灣師範大學附屬高級中學","國立台灣戲曲學院","國立台灣教育學院","國立台灣海洋大學","國立台灣海洋學院","國立台灣科技大學","國立台灣藝術大學","國立台灣藝術學院","國立台灣體育學院","國立台灣體育運動大學","國立和美實驗學校","國立員林崇實高級工業職業學校","國立員林高級中學","國立員林高級家事商業職業學校","國立員林高級農工職業學校","國立善化高級中學","國立嘉義大學","國立嘉義大學附設實驗國民小學","國立嘉義女子高級中學","國立嘉義師範學院","國立嘉義技術學院","國立嘉義特殊教育學校","國立嘉義高級中學","國立嘉義高級商業職業學校","國立嘉義高級家事職業學校","國立嘉義高級工業職業學校","國立土庫高級商工職業學校","國立埔里高級工業職業學校","國立基隆女子高級中學","國立基隆特殊教育學校","國立基隆高級中學","國立基隆高級商工職業學校","國立基隆高級海事職業學校","國立大湖高級農工職業學校","國立宜蘭大學","國立宜蘭大學附設高級農工職業進修學校","國立宜蘭技術學院","國立宜蘭特殊教育學校","國立宜蘭高級中學","國立宜蘭高級商業職業學校","國立屏北高級中學","國立屏東商業技術學院","國立屏東大學","國立屏東大學附設實驗國民小學","國立屏東女子高級中學","國立屏東師範學院","國立屏東技術學院","國立屏東教育大學","國立屏東特殊教育學校","國立屏東科技大學","國立屏東高級中學","國立屏東高級工業職業學校","國立岡山高級中學","國立岡山高級農工職業學校","國立彰化女子高級中學","國立彰化師範大學","國立彰化師範大學附屬高級工業職業學校","國立彰化特殊教育學校","國立彰化高級中學","國立彰化高級商業職業學校","國立後壁高級中學","國立恆春高級工商職業學校","國立成功商業水產職業學校","國立成功大學","國立成功大學附設高級工業職業進修學校","國立政治大學","國立政治大學附屬高級中學","國立斗六高級中學","國立斗六高級家事商業職業學校","國立新化高級中學","國立新化高級工業職業學校","國立新港藝術高級中學","國立新營高級中學","國立新營高級工業職業學校","國立新竹女子高級中學","國立新竹師範學院","國立新竹教育大學","國立新竹特殊教育學校","國立新竹高級中學","國立新竹高級商業職業學校","國立新竹高級工業職業學校","國立新豐高級中學","國立旗山高級農工職業學校","國立旗美高級中學","國立暨南國際大學","國立暨南國際大學附屬高級中學","國立曾文高級家事商業職業學校","國立曾文高級農工職業學校","國立東港高級海事水產職業學校","國立東石高級中學","國立東華大學","國立東華大學附設實驗國民小學","國立民雄高級農工職業學校","國立水里高級商工職業學校","國立永靖高級工業職業學校","國立清華大學","國立清華大學附設實驗國民小學","國立溪湖高級中學","國立潮州高級中學","國立澎湖技術學院","國立澎湖科技大學","國立澎湖高級海事水產職業學校","國立玉井高級工商職業學校","國立玉里高級中學","國立白河高級商工職業學校","國立秀水高級工業職業學校","國立科學工業園區實驗高級中學","國立空中大學","國立竹北高級中學","國立竹南高級中學","國立竹山高級中學","國立竹東高級中學","國立羅東高級中學","國立羅東高級商業職業學校","國立羅東高級工業職業學校","國立聯合大學","國立聯合技術學院","國立臺中師範學院","國立臺中技術學院","國立臺中技術學院附設空中進修學院","國立臺中技術學院附設進修學院","國立臺中教育大學","國立臺中教育大學附設實驗國民小學","國立臺中科技大學","國立臺北商業大學","國立臺北商業技術學院","國立臺北商業技術學院附設空中進修學院","國立臺北大學","國立臺北師範學院","國立臺北技術學院","國立臺北教育大學","國立臺北教育大學附設實驗國民小學","國立臺北科技大學","國立臺北科技大學附屬桃園農工高級中等學校","國立臺北科技大學附設進修學院","國立臺北藝術大學","國立臺北護理健康大學","國立臺北護理學院","國立臺南啟智學校","國立臺南大學","國立臺南大學附屬啟聰學校","國立臺南大學附屬高級中學","國立臺南大學附設實驗國民小學","國立臺南女子高級中學","國立臺南家齊高級中等學校","國立臺南師範學院","國立臺南第一高級中學","國立臺南第二高級中學","國立臺南藝術大學","國立臺南藝術學院","國立臺南護理專科學校","國立臺南高級商業職業學校","國立臺南高級工業職業學校","國立臺南高級海事水產職業學校","國立臺東大學","國立臺東大學附屬特殊教育學校","國立臺東大學附屬體育高級中學","國立臺東大學附設實驗國民小學","國立臺東女子高級中學","國立臺東專科學校","國立臺東師範學院","國立臺東高級中學","國立臺東高級商業職業學校","國立臺灣大學","國立臺灣工業技術學院","國立臺灣師範大學","國立臺灣師範大學附屬高級中學","國立臺灣戲曲學院","國立臺灣教育學院","國立臺灣海洋大學","國立臺灣海洋學院","國立臺灣科技大學","國立臺灣藝術大學","國立臺灣藝術學院","國立臺灣體育學院","國立臺灣體育運動大學","國立花蓮女子高級中學","國立花蓮師範學院","國立花蓮特殊教育學校","國立花蓮高級中學","國立花蓮高級商業職業學校","國立花蓮高級工業職業學校","國立花蓮高級農業職業學校","國立苑裡高級中學","國立苗栗特殊教育學校","國立苗栗高級中學","國立苗栗高級商業職業學校","國立苗栗高級農工職業學校","國立草屯高級商工職業學校","國立華僑高級中等學校","國立華南高級商業職業學校","國立華商","國立藝術學院","國立蘇澳高級海事水產職業學校","國立蘭陽女子高級中學","國立虎尾技術學院","國立虎尾技術學院附設進修學院","國立虎尾科技大學","國立虎尾高級中學","國立虎尾高級農工職業學校","國立西螺高級農工職業學校","國立金門大學","國立金門技術學院","國立金門高級中學","國立金門高級農工職業學校","國立關山高級工商職業學校","國立關西高級中學","國立陽明大學","國立陽明醫學院","國立雲林技術學院","國立雲林特殊教育學校","國立雲林科技大學","國立頭城高級家事商業職業學校","國立馬公高級中學","國立馬祖高級中學","國立體育大學","國立體育學院","國立體育師範專科學校","國立高雄大學","國立高雄師範大學","國立高雄師範大學附屬高級中學","國立高雄師範學院","國立高雄應用科技大學附設進修學院","國立高雄技術學院","國立高雄海洋技術學院","國立高雄科學技術學院","國立高雄科學技術學院附設進修學院","國立高雄科技大學","國立高雄餐旅大學","國立高雄餐旅大學附屬餐旅高級中等學校","國立高雄餐旅學院","國立鳳山高級中學","國立鳳山高級商工職業學校","國立鳳新高級中學","國立鹿港高級中學","國聖國小","國聖國民小學","國華國中","國華國民中學","國語實小","國語實驗國民小學","國醫","國防大學","國防管理學校","國防管理學院","國防財經學校","國防財經學院","國防醫學院","國際商工","國際高級商工職業學校","國風國中","國風國民中學","圓富國中","圓富國民中學","圓崇國小","圓崇國民小學","圓潭國小","圓潭國民小學","土坂國小","土坂國民小學","土城國中","土城國小","土城國民中學","土城國民小學","土城幼兒園","土城托兒所","土城高中","土城高級中學","土庫商工","土庫國中","土庫國小","土庫國民中學","土庫國民小學","土庫鎮立托兒所","土庫高級商工職業學校","土牛國小","土牛國民小學","地利國小","地利國民小學","地磨兒國小","地磨兒國民小學","圳堵國小","圳堵國民小學","圳寮國小","圳寮國民小學","圳頭國小","圳頭國民小學","均頭國中","均頭國民中學","坔頭港國小","坔頭港國民小學","坪林國中","坪林國小","坪林國民中學","坪林國民小學","坪林幼兒園","坪林托兒所","坪頂國小","坪頂國民小學","垂楊國小","垂楊國民小學","城中國小","城中國民小學","城中托兒所","城市科大","城科","城科大","埔和國小","埔和國民小學","埔墘國小","埔墘國民小學","埔心國中","埔心國小","埔心國民中學","埔心國民小學","埔心鄉立托兒所","埔里國中","埔里國小","埔里國民中學","埔里國民小學","埔里鎮立托兒所","埔里高工","埔里高級工業職業學校","埔頂國小","埔頂國民小學","埔鹽國中","埔鹽國小","埔鹽國民中學","埔鹽國民小學","埔鹽鄉立托兒所","埤腳國小","埤腳國民小學","埤頭國中","埤頭國小","埤頭國民中學","埤頭國民小學","埤頭鄉立托兒所","培元高中","培元高級中學","培德工家","培德高級工業家事職業學校","培文國小","培文國民小學","培英國中","培英國小","培英國民中學","培英國民小學","基中","基隆商工","基隆女中","基隆女子高級中學","基隆市七堵區七堵國民小學","基隆市七堵區五堵國民小學","基隆市七堵區堵南國民小學","基隆市七堵區尚仁國民小學","基隆市七堵區建德幼稚園","基隆市七堵區復興國民小學","基隆市七堵區瑪陵國民小學","基隆市七堵區華興國民小學","基隆市七堵區長興國民小學","基隆市中山區中和國民小學","基隆市中山區中山國民小學","基隆市中山區中華國民小學","基隆市中山區仙洞國民小學","基隆市中山區德和國民小學","基隆市中山區港西國民小學","基隆市中正區中正國民小學","基隆市中正區八斗國民小學","基隆市中正區和平國民小學","基隆市中正區忠孝國民小學","基隆市中正區正濱國民小學","基隆市仁愛區仁愛國民小學","基隆市仁愛區南榮國民小學","基隆市仁愛區尚智國民小學","基隆市仁愛區成功國民小學","基隆市信義區中興國民小學","基隆市信義區信義國民小學","基隆市信義區月眉國民小學","基隆市信義區東信國民小學","基隆市信義區東光國民小學","基隆市信義區深澳國民小學","基隆市信義區深美國民小學","基隆市安樂區安樂國民小學","基隆市安樂區建德國民小學","基隆市安樂區武崙國民小學","基隆市安樂區西定國民小學","基隆市安樂區長樂國民小學","基隆市安樂區隆聖國民小學","基隆市暖暖區八堵國民小學","基隆市暖暖區暖暖國民小學","基隆市暖暖區暖江國民小學","基隆市暖暖區暖西國民小學","基隆市暖暖區碇內國民小學","基隆市私立二信高級中學","基隆市私立光隆高級家事商業職業學校","基隆市私立培德高級工業家事職業學校","基隆市私立聖心高級中學","基隆市立中山高級中學","基隆市立中正國民中學","基隆市立信義國民中學","基隆市立八斗高級中學","基隆市立南榮國民中學","基隆市立安心幼兒園","基隆市立安樂高級中學","基隆市立建德國民中學","基隆市立建德幼兒園","基隆市立建德幼稚園","基隆市立成功國民中學","基隆市立托兒所","基隆市立明德國民中學","基隆市立暖暖高級中學","基隆市立正濱國民中學","基隆市立武崙國民中學","基隆市立百福國民中學","基隆市立碇內國民中學","基隆市立過港幼兒園","基隆市立過港幼稚園","基隆市立銘傳國民中學","基隆海事","基隆特教","基隆特殊教育學校","基隆高級中學","基隆高級商工職業學校","基隆高級海事職業學校","堵南國小","堵南國民小學","塔樓國小","塔樓國民小學","塗城國小","塗城國民小學","塘岐國小","塘岐國民小學","塭內國小","塭內國民小學","塭子國小","塭子國民小學","塭港國小","塭港國民小學","墾丁國小","墾丁國民小學","壢中","士敏國小","士敏國民小學","士東國小","士東國民小學","士林國中","士林國小","士林國民中學","士林國民小學","士林幼兒園","士林托兒所","士林社區大學","士林高商","士林高級商業職業學校","士林高職","壯圍國中","壯圍國小","壯圍國民中學","壯圍國民小學","壯圍鄉立托兒所","壽天國小","壽天國民小學","壽山國中","壽山國小","壽山國民中學","壽山國民小學","壽山高中","壽山高級中等學校","壽豐國中","壽豐國小","壽豐國民中學","壽豐國民小學","壽豐鄉立托兒所","壽齡國小","壽齡國民小學","外交部外交及國際事務學院","外垵國小","外垵國民小學","外埔國中","外埔國小","外埔國民中學","外埔國民小學","外埔托兒所","外校學院","外社國小","外社國民小學","多年國小","多年國民小學","多納國小","多納國民小學","大仁","大仁國中","大仁國小","大仁國民中學","大仁國民小學","大仁技術學院","大仁科技大學","大佳國小","大佳國民小學","大倫國中","大倫國民中學","大元國小","大元國民小學","大光國小","大光國民小學","大內國中","大內國小","大內國民中學","大內國民小學","大勇國小","大勇國民小學","大南國小","大南國民小學","大吉國中","大吉國民中學","大同國中","大同國小","大同國民中學","大同國民小學","大同大學","大同工學院","大同工業專科學校","大同幼兒園","大同托兒所","大同技術學院","大同社區大學","大同社大","大同鄉立托兒所","大同高中","大同高商","大同高級中學","大同高級商業職業學校","大同高職","大嘉國小","大嘉國民小學","大園國中","大園國小","大園國民中學","大園國民小學","大園國際高級中等學校","大園幼兒園","大園高中","大坑國小","大坑國民小學","大坡國中","大坡國小","大坡國民中學","大坡國民小學","大坪國小","大坪國民小學","大城國中","大城國小","大城國民中學","大城國民小學","大城鄉立托兒所","大埔中小學","大埔國小","大埔國民中小學","大埔國民小學","大埤國中","大埤國小","大埤國民中學","大埤國民小學","大埤鄉立托兒所","大墩國中","大墩國小","大墩國民中學","大墩國民小學","大安國中","大安國小","大安國民中學","大安國民小學","大安幼兒園","大安托兒所","大安社區大學","大安社大","大安高工","大安高工附設高級工業職業進修學校","大安高級工業職業學校","大寮國中","大寮國小","大寮國民中學","大寮國民小學","大寮幼兒園","大屯國小","大屯國民小學","大山國小","大山國民小學","大崁國小","大崁國民小學","大崎國小","大崎國民小學","大崗國中","大崗國小","大崗國民中學","大崗國民小學","大崙國中","大崙國小","大崙國民中學","大崙國民小學","大平國小","大平國民小學","大庄國小","大庄國民小學","大德國中","大德國小","大德國民中學","大德國民小學","大德工商","大德工業商業職業學校","大忠國小","大忠國民小學","大慶商工","大慶高級商工職業學校","大成商工","大成國中","大成國小","大成國民中學","大成國民小學","大成幼稚園","大成高中","大成高級中學","大成高級商工職業學校","大文國小","大文國民小學","大新國小","大新國民小學","大明國小","大明國民小學","大智國小","大智國民小學","大有國中","大有國小","大有國民中學","大有國民小學","大村國中","大村國小","大村國民中學","大村國民小學","大村鄉立托兒所","大東國小","大東國民小學","大林國中","大林國小","大林國民中學","大林國民小學","大林鎮立托兒所","大楊國小","大楊國民小學","大業國中","大業國小","大業國民中學","大業國民小學","大榮國小","大榮國民小學","大榮高中","大榮高級中學","大樹國中","大樹國小","大樹國民中學","大樹國民小學","大橋國中","大橋國小","大橋國民中學","大橋國民小學","大武國中","大武國小","大武國民中學","大武國民小學","大武鄉立托兒所","大汕國小","大汕國民小學","大池國小","大池國民小學","大洲國中","大洲國小","大洲國民中學","大洲國民小學","大港國小","大港國民小學","大湖國中","大湖國小","大湖國民中學","大湖國民小學","大湖農工","大湖鄉立托兒所","大湖高級農工職業學校","大溪國中","大溪國小","大溪國民中學","大溪國民小學","大溪幼兒園","大溪高中","大溪高級中等學校","大漢技術學院","大潭國小","大潭國民小學","大灣國中","大灣國小","大灣國民中學","大灣國民小學","大灣高中","大灣高級中學","大王國中","大王國小","大王國民中學","大王國民小學","大理國小","大理國民小學","大理高中","大理高級中學","大甲國中","大甲國小","大甲國民中學","大甲國民小學","大甲工業高級中等學校","大甲托兒所","大甲高中","大甲高工","大甲高級中等學校","大直國小","大直國民小學","大直托兒所","大直高中","大直高級中學","大社國中","大社國小","大社國民中學","大社國民小學","大福國小","大福國民小學","大禹國小","大禹國民小學","大秀國小","大秀國民小學","大竹國中","大竹國小","大竹國民中學","大竹國民小學","大美國小","大美國民小學","大義國中","大義國民中學","大肚國小","大肚國民小學","大肚托兒所","大興國小","大興國民小學","大莊國小","大莊國民小學","大華國中","大華國小","大華國民中學","大華國民小學","大華技術學院","大葉","大葉大學","大葉工學院","大西國中","大西國小","大西國民中學","大西國民小學","大觀國中","大觀國小","大觀國民中學","大觀國民小學","大誠高中","大誠高級中學","大豐國小","大豐國民小學","大進國小","大進國民小學","大道國中","大道國民中學","大鄉國小","大鄉國民小學","大里區立新國民小學","大里國小","大里國民小學","大里托兒所","大里高中","大里高級中學","大隱國小","大隱國民小學","大雅國中","大雅國小","大雅國民中學","大雅國民小學","大雅托兒所","大鞍國小","大鞍國民小學","大鳥國小","大鳥國民小學","大鵬國小","大鵬國民小學","大龍國小","大龍國民小學","天主教私立寶仁國民小學","天南國小","天南國民小學","天母國中","天母國小","天母國民中學","天母國民小學","天生國小","天生國民小學","天盛國小","天盛國民小學","太保國中","太保國小","太保國民中學","太保國民小學","太保市立托兒所","太和國小","太和國民小學","太子國中","太子國民中學","太巴塱國小","太巴塱國民小學","太平國中","太平國小","太平國民中學","太平國民小學","太平托兒所","太康國小","太康國民小學","太昌國小","太昌國民小學","太興國小","太興國民小學","太麻里鄉立托兒所","奇美國小","奇美國民小學","奎山國中","奎山國民中學","奎山小學","奎輝國小","奎輝國民小學","好修國小","好修國民小學","好收國小","好收國民小學","好美國小","好美國民小學","姑山國小","姑山國民小學","媽厝國小","媽厝國民小學","子龍國小","子龍國民小學","孝威國小","孝威國民小學","學東國小","學東國民小學","學田國小","學田國民小學","學甲國中","學甲國小","學甲國民中學","學甲國民小學","學甲幼兒園","學甲托兒所","學進國小","學進國民小學","宅港國小","宅港國民小學","安佃國小","安佃國民小學","安南國中","安南國小","安南國民中學","安南國民小學","安和國中","安和國小","安和國民中學","安和國民小學","安坑國小","安坑國民小學","安定國中","安定國小","安定國民中學","安定國民小學","安定托兒所","安平國中","安平國小","安平國民中學","安平國民小學","安康高中","安康高級中學","安心幼兒園","安慶國小","安慶國民小學","安招國小","安招國民小學","安朔國小","安朔國民小學","安東國小","安東國民小學","安業國小","安業國民小學","安樂國小","安樂國民小學","安樂高中","安樂高級中學","安溪國中","安溪國小","安溪國民中學","安溪國民小學","安瀾國小","安瀾國民小學","安興國小","安興國民小學","安順國中","安順國小","安順國民中學","安順國民小學","宋屋國小","宋屋國民小學","宏仁國中","宏仁國小","宏仁國民中學","宏仁國民小學","宏仁女中","宏仁女子高級中學","宏崙國小","宏崙國民小學","官田國中","官田國小","官田國民中學","官田國民小學","宜中","宜大","宜大附農","宜寧高中","宜寧高級中學","宜昌國中","宜昌國小","宜昌國民中學","宜昌國民小學","宜梧國中","宜梧國民中學","宜欣國小","宜欣國民小學","宜蘭國中","宜蘭國小","宜蘭國民中學","宜蘭國民小學","宜蘭大學","宜蘭大學附設高級農工職業進修學校","宜蘭市立托兒所","宜蘭技術學院","宜蘭特教","宜蘭特殊教育學校","宜蘭縣三星鄉三星國民小學","宜蘭縣三星鄉大洲國民小學","宜蘭縣三星鄉大隱國民小學","宜蘭縣三星鄉憲明國民小學","宜蘭縣三星鄉立托兒所","宜蘭縣三星鄉萬富國民小學","宜蘭縣五結鄉中興國民小學","宜蘭縣五結鄉五結國民小學","宜蘭縣五結鄉利澤國民小學","宜蘭縣五結鄉孝威國民小學","宜蘭縣五結鄉學進國民小學","宜蘭縣五結鄉立托兒所","宜蘭縣冬山鄉冬山國民小學","宜蘭縣冬山鄉大進國民小學","宜蘭縣冬山鄉廣興國民小學","宜蘭縣冬山鄉東興國民小學","宜蘭縣冬山鄉柯林國民小學","宜蘭縣冬山鄉武淵國民小學","宜蘭縣冬山鄉清溝國民小學","宜蘭縣冬山鄉立托兒所","宜蘭縣冬山鄉順安國民小學","宜蘭縣南澳鄉南澳國民小學","宜蘭縣南澳鄉東澳國民小學","宜蘭縣南澳鄉武塔國民小學","宜蘭縣南澳鄉澳花國民小學","宜蘭縣南澳鄉碧候國民小學","宜蘭縣南澳鄉立托兒所","宜蘭縣南澳鄉金岳國民小學","宜蘭縣南澳鄉金洋國民小學","宜蘭縣員山鄉七賢國民小學","宜蘭縣員山鄉同樂國民小學","宜蘭縣員山鄉員山國民小學","宜蘭縣員山鄉大湖國民小學","宜蘭縣員山鄉深溝國民小學","宜蘭縣員山鄉湖山國民小學","宜蘭縣員山鄉立托兒所","宜蘭縣壯圍鄉公館國民小學","宜蘭縣壯圍鄉古亭國民小學","宜蘭縣壯圍鄉壯圍國民小學","宜蘭縣壯圍鄉大福國民小學","宜蘭縣壯圍鄉新南國民小學","宜蘭縣壯圍鄉立托兒所","宜蘭縣壯圍鄉過嶺國民小學","宜蘭縣大同鄉南山國民小學","宜蘭縣大同鄉四季國民小學","宜蘭縣大同鄉大同國民小學","宜蘭縣大同鄉寒溪國民小學","宜蘭縣大同鄉立托兒所","宜蘭縣宜蘭市中山國民小學","宜蘭縣宜蘭市光復國民小學","宜蘭縣宜蘭市凱旋國民小學","宜蘭縣宜蘭市力行國民小學","宜蘭縣宜蘭市南屏國民小學","宜蘭縣宜蘭市宜蘭國民小學","宜蘭縣宜蘭市新生國民小學","宜蘭縣宜蘭市立托兒所","宜蘭縣宜蘭市育才國民小學","宜蘭縣宜蘭市黎明國民小學","宜蘭縣礁溪鄉三民國民小學","宜蘭縣礁溪鄉四結國民小學","宜蘭縣礁溪鄉玉田國民小學","宜蘭縣礁溪鄉礁溪國民小學","宜蘭縣礁溪鄉立托兒所","宜蘭縣礁溪鄉龍潭國民小學","宜蘭縣私立中道高級中學","宜蘭縣私立慧燈高級中學","宜蘭縣立三星國民中學","宜蘭縣立中華國民中學","宜蘭縣立五結國民中學","宜蘭縣立人文國民中小學","宜蘭縣立內城國民中小學","宜蘭縣立冬山國民中學","宜蘭縣立凱旋國民中學","宜蘭縣立利澤國民中學","宜蘭縣立南安國民中學","宜蘭縣立南澳高級中學","宜蘭縣立吳沙國民中學","宜蘭縣立員山國民中學","宜蘭縣立國華國民中學","宜蘭縣立壯圍國民中學","宜蘭縣立大同國民中學","宜蘭縣立宜蘭國民中學","宜蘭縣立復興國民中學","宜蘭縣立慈心華德福教育實驗高級中等學校","宜蘭縣立文化國民中學","宜蘭縣立東光國民中學","宜蘭縣立礁溪國民中學","宜蘭縣立羅東國民中學","宜蘭縣立興中國民中學","宜蘭縣立蘇澳國民中學","宜蘭縣立順安國民中學","宜蘭縣立頭城國民中學","宜蘭縣羅東鎮公正國民小學","宜蘭縣羅東鎮北成國民小學","宜蘭縣羅東鎮成功國民小學","宜蘭縣羅東鎮立托兒所","宜蘭縣羅東鎮竹林國民小學","宜蘭縣羅東鎮羅東國民小學","宜蘭縣蘇澳鎮南安國民小學","宜蘭縣蘇澳鎮士敏國民小學","宜蘭縣蘇澳鎮岳明國民小學","宜蘭縣蘇澳鎮永樂國民小學","宜蘭縣蘇澳鎮立托兒所","宜蘭縣蘇澳鎮育英國民小學","宜蘭縣蘇澳鎮蓬萊國民小學","宜蘭縣蘇澳鎮蘇澳國民小學","宜蘭縣蘇澳鎮馬賽國民小學","宜蘭縣頭城鎮二城國民小學","宜蘭縣頭城鎮大溪國民小學","宜蘭縣頭城鎮大里國民小學","宜蘭縣頭城鎮梗枋國民小學","宜蘭縣頭城鎮立托兒所","宜蘭縣頭城鎮竹安國民小學","宜蘭縣頭城鎮頭城國民小學","宜蘭高商","宜蘭高級中學","宜蘭高級商業職業學校","客厝國小","客厝國民小學","客庄國小","客庄國民小學","宣信國小","宣信國民小學","家齊","富世國小","富世國民小學","富光國中","富光國民中學","富功國小","富功國民小學","富北國中","富北國民中學","富台國小","富台國民小學","富安國小","富安國民小學","富山國小","富山國民小學","富岡國中","富岡國小","富岡國民中學","富岡國民小學","富春國小","富春國民小學","富林國小","富林國民小學","富源國中","富源國小","富源國民中學","富源國民小學","富田國小","富田國民小學","富禮國中","富禮國民中學","富興國小","富興國民小學","富里國中","富里國小","富里國民中學","富里國民小學","富里鄉立托兒所","寒溪國小","寒溪國民小學","實踐國中","實踐國小","實踐國民中學","實踐國民小學","實踐大學","實踐設計管理學院","寧埔國小","寧埔國民小學","寶仁國小","寶來國中","寶來國小","寶來國民中學","寶來國民小學","寶山國中","寶山國小","寶山國民中學","寶山國民小學","寶山鄉立托兒所","寶桑國中","寶桑國小","寶桑國民中學","寶桑國民小學","寶石國小","寶石國民小學","寶隆國小","寶隆國民小學","將澳國中","將澳國民中學","將軍國中","將軍國小","將軍國民中學","將軍國民小學","將軍幼兒園","將軍托兒所","小坪國小","小坪國民小學","小康幼兒園","小康托兒所","小新國小","小新國民小學","小林國小","小林國民小學","小港國中","小港國小","小港國民中學","小港國民小學","小港高中","小港高級中學","尖中","尖山國小","尖山國民中學","尖山國民小學","尖石國中","尖石國小","尖石國民中學","尖石國民小學","尖石鄉立托兒所","尚仁國小","尚仁國民小學","尚德國小","尚德國民小學","尚智國小","尚智國民小學","尚武國小","尚武國民小學","居仁國中","居仁國民中學","屈尺國小","屈尺國民小學","屏中","屏北高中","屏北高級中學","屏大","屏大實小","屏女","屏山國小","屏山國民小學","屏師","屏教大","屏東商業技術學院","屏東大學","屏東大學附設實驗國民小學","屏東女中","屏東女子高級中學","屏東師範學院","屏東技術學院","屏東教育大學","屏東特教","屏東特殊教育學校","屏東科技大學","屏東縣三地門鄉口社國民小學","屏東縣三地門鄉地磨兒國民小學","屏東縣三地門鄉立托兒所","屏東縣三地門鄉賽嘉國民小學","屏東縣三地門鄉青山國民小學","屏東縣三地門鄉青葉國民小學","屏東縣九如鄉三多國民小學","屏東縣九如鄉九如國民小學","屏東縣九如鄉後庄國民小學","屏東縣九如鄉惠農國民小學","屏東縣佳冬鄉佳冬國民小學","屏東縣佳冬鄉塭子國民小學","屏東縣佳冬鄉大新國民小學","屏東縣佳冬鄉昌隆國民小學","屏東縣佳冬鄉玉光國民小學","屏東縣佳冬鄉立托兒所","屏東縣佳冬鄉羌園國民小學","屏東縣來義鄉來義國民小學","屏東縣來義鄉南和國民小學","屏東縣來義鄉古樓國民小學","屏東縣來義鄉文樂國民小學","屏東縣來義鄉望嘉國民小學","屏東縣來義鄉立托兒所","屏東縣內埔鄉僑智國民小學","屏東縣內埔鄉內埔國民小學","屏東縣內埔鄉富田國民小學","屏東縣內埔鄉崇文國民小學","屏東縣內埔鄉新生國民小學","屏東縣內埔鄉東勢國民小學","屏東縣內埔鄉東寧國民小學","屏東縣內埔鄉榮華國民小學","屏東縣內埔鄉泰安國民小學","屏東縣內埔鄉育英國民小學","屏東縣內埔鄉豐田國民小學","屏東縣內埔鄉隘寮國民小學","屏東縣內埔鄉黎明國民小學","屏東縣南州鄉南州國民小學","屏東縣南州鄉同安國民小學","屏東縣南州鄉溪北國民小學","屏東縣屏東市中正國民小學","屏東縣屏東市仁愛國民小學","屏東縣屏東市信義國民小學","屏東縣屏東市公館國民小學","屏東縣屏東市凌雲國民小學","屏東縣屏東市前進國民小學","屏東縣屏東市勝利國民小學","屏東縣屏東市和平國民小學","屏東縣屏東市唐榮國民小學","屏東縣屏東市大同國民小學","屏東縣屏東市崇蘭國民小學","屏東縣屏東市建國國民小學","屏東縣屏東市復興國民小學","屏東縣屏東市忠孝國民小學","屏東縣屏東市歸來國民小學","屏東縣屏東市民和國民小學","屏東縣屏東市民生國民小學","屏東縣屏東市海豐國民小學","屏東縣屏東市瑞光國民小學","屏東縣屏東市第一幼稚園","屏東縣屏東市鶴聲國民小學","屏東縣屏榮高級中學","屏東縣崁頂鄉力社國民小學","屏東縣崁頂鄉崁頂國民小學","屏東縣崁頂鄉港東國民小學","屏東縣崁頂鄉立托兒所","屏東縣恆春鎮僑勇國民小學","屏東縣恆春鎮墾丁國民小學","屏東縣恆春鎮大光國民小學","屏東縣恆春鎮大平國民小學","屏東縣恆春鎮山海國民小學","屏東縣恆春鎮恆春國民小學","屏東縣恆春鎮水泉國民小學","屏東縣新園鄉仙吉國民小學","屏東縣新園鄉新園國民小學","屏東縣新園鄉港西國民小學","屏東縣新園鄉烏龍國民小學","屏東縣新園鄉瓦磘國民小學","屏東縣新園鄉鹽洲國民小學","屏東縣新埤鄉大成國民小學","屏東縣新埤鄉新埤國民小學","屏東縣新埤鄉立托兒所","屏東縣新埤鄉萬隆國民小學","屏東縣新埤鄉餉潭國民小學","屏東縣春日鄉力里國民小學","屏東縣春日鄉古華國民小學","屏東縣春日鄉春日國民小學","屏東縣春日鄉立托兒所","屏東縣東港鎮以栗國民小學","屏東縣東港鎮大潭國民小學","屏東縣東港鎮東光國民小學","屏東縣東港鎮東港國民小學","屏東縣東港鎮東港幼稚園","屏東縣東港鎮東興國民小學","屏東縣東港鎮東隆國民小學","屏東縣東港鎮海濱國民小學","屏東縣枋寮鄉僑德國民小學","屏東縣枋寮鄉建興國民小學","屏東縣枋寮鄉東海國民小學","屏東縣枋寮鄉枋寮國民小學","屏東縣枋山鄉加祿國民小學","屏東縣枋山鄉楓港國民小學","屏東縣枋山鄉立托兒所","屏東縣林邊鄉仁和國民小學","屏東縣林邊鄉崎峰國民小學","屏東縣林邊鄉林邊國民小學","屏東縣林邊鄉水利國民小學","屏東縣林邊鄉立托兒所","屏東縣林邊鄉竹林國民小學","屏東縣泰武鄉武潭國民小學","屏東縣泰武鄉泰武國民小學","屏東縣泰武鄉立托兒所","屏東縣泰武鄉萬安國民小學","屏東縣滿州鄉永港國民小學","屏東縣滿州鄉滿州國民小學","屏東縣滿州鄉立托兒所","屏東縣滿州鄉長樂國民小學","屏東縣潮州鎮光春國民小學","屏東縣潮州鎮光華國民小學","屏東縣潮州鎮四林國民小學","屏東縣潮州鎮潮南國民小學","屏東縣潮州鎮潮和國民小學","屏東縣潮州鎮潮州國民小學","屏東縣潮州鎮潮昇國民小學","屏東縣潮州鎮潮東國民小學","屏東縣潮州鎮立托兒所","屏東縣牡丹鄉牡丹國民小學","屏東縣牡丹鄉石門國民小學","屏東縣牡丹鄉立托兒所","屏東縣牡丹鄉高士國民小學","屏東縣獅子鄉丹路國民小學","屏東縣獅子鄉內獅國民小學","屏東縣獅子鄉楓林國民小學","屏東縣獅子鄉立托兒所","屏東縣獅子鄉草埔國民小學","屏東縣琉球鄉全德國民小學","屏東縣琉球鄉天南國民小學","屏東縣琉球鄉琉球國民小學","屏東縣琉球鄉白沙國民小學","屏東縣琉球鄉立托兒所","屏東縣瑪家鄉佳義國民小學","屏東縣瑪家鄉北葉國民小學","屏東縣瑪家鄉立托兒所","屏東縣瑪家鄉長榮百合國民小學","屏東縣私立南榮國民中學","屏東縣私立志成高級商工職業學校","屏東縣私立慈惠高級護理助產職業學校","屏東縣私立新基高級中學","屏東縣私立日新高級工商職業學校","屏東縣私立明德高級中學","屏東縣私立民生高級家事商業職業學校","屏東縣私立美和高級中學","屏東縣私立華洲高級工業家事職業學校","屏東縣私立陸興高級中學","屏東縣立中正國民中學","屏東縣立九如國民中學","屏東縣立佳冬國民中學","屏東縣立來義高級中學","屏東縣立光春國民中學","屏東縣立內埔國民中學","屏東縣立公正國民中學","屏東縣立南州國民中學","屏東縣立大同高級中學","屏東縣立崇文國民中學","屏東縣立恆春國民中學","屏東縣立新園國民中學","屏東縣立新埤國民中學","屏東縣立明正國民中學","屏東縣立東新國民中學","屏東縣立東港高級中學","屏東縣立枋寮高級中學","屏東縣立林邊國民中學","屏東縣立泰武國民中學","屏東縣立滿州國民中學","屏東縣立潮州國民中學","屏東縣立牡丹國民中學","屏東縣立獅子國民中學","屏東縣立琉球國民中學","屏東縣立瑪家國民中學","屏東縣立竹田國民中學","屏東縣立至正國民中學","屏東縣立萬丹國民中學","屏東縣立萬巒國民中學","屏東縣立萬新國民中學","屏東縣立車城國民中學","屏東縣立里港國民中學","屏東縣立長治國民中學","屏東縣立高樹國民中學","屏東縣立高泰國民中學","屏東縣立鶴聲國民中學","屏東縣立鹽埔國民中學","屏東縣立麟洛國民中學","屏東縣竹田鄉大明國民小學","屏東縣竹田鄉立托兒所","屏東縣竹田鄉竹田國民小學","屏東縣竹田鄉西勢國民小學","屏東縣萬丹鄉四維國民小學","屏東縣萬丹鄉廣安國民小學","屏東縣萬丹鄉新庄國民小學","屏東縣萬丹鄉新興國民小學","屏東縣萬丹鄉社皮國民小學","屏東縣萬丹鄉興化國民小學","屏東縣萬丹鄉興華國民小學","屏東縣萬丹鄉萬丹國民小學","屏東縣萬巒鄉五溝國民小學","屏東縣萬巒鄉佳佐國民小學","屏東縣萬巒鄉立托兒所","屏東縣萬巒鄉萬巒國民小學","屏東縣萬巒鄉赤山國民小學","屏東縣車城鄉立托兒所","屏東縣車城鄉車城國民小學","屏東縣里港鄉三和國民小學","屏東縣里港鄉土庫國民小學","屏東縣里港鄉塔樓國民小學","屏東縣里港鄉玉田國民小學","屏東縣里港鄉立托兒所","屏東縣里港鄉載興國民小學","屏東縣里港鄉里港國民小學","屏東縣長治鄉德協國民小學","屏東縣長治鄉繁華國民小學","屏東縣長治鄉長興國民小學","屏東縣霧台鄉立托兒所","屏東縣霧台鄉霧台國民小學","屏東縣霧臺鄉立托兒所","屏東縣霧臺鄉霧臺國民小學","屏東縣高樹鄉南華國民小學","屏東縣高樹鄉廣興國民小學","屏東縣高樹鄉新南國民小學","屏東縣高樹鄉新豐國民小學","屏東縣高樹鄉泰山國民小學","屏東縣高樹鄉田子國民小學","屏東縣高樹鄉立托兒所","屏東縣高樹鄉舊寮國民小學","屏東縣高樹鄉高樹國民小學","屏東縣鹽埔鄉仕絨國民小學","屏東縣鹽埔鄉彭厝國民小學","屏東縣鹽埔鄉振興國民小學","屏東縣鹽埔鄉新圍國民小學","屏東縣鹽埔鄉立托兒所","屏東縣鹽埔鄉高朗國民小學","屏東縣鹽埔鄉鹽埔國民小學","屏東縣麟洛鄉立托兒所","屏東縣麟洛鄉麟洛國民小學","屏東高工","屏東高級中學","屏東高級工業職業學校","屏榮高中","屏榮高級中學","屏科大","層林國小","層林國民小學","屯山國小","屯山國民小學","山上國中","山上國小","山上國民中學","山上國民小學","山佳國小","山佳國民小學","山內國小","山內國民小學","山峰華德福教育實驗國民小學","山崎國小","山崎國民小學","山東國小","山東國民小學","山水國小","山水國民小學","山海國小","山海國民小學","山美國小","山美國民小學","山腳國中","山腳國小","山腳國民中學","山腳國民小學","山豐國小","山豐國民小學","山陽國小","山陽國民小學","山頂國小","山頂國民小學","岡山國中","岡山國小","岡山國民中學","岡山國民小學","岡山農工","岡山高中","岡山高級中學","岡山高級農工職業學校","岩灣國小","岩灣國民小學","岳明國小","岳明國民小學","岸內國小","岸內國民小學","岸裡國小","岸裡國民小學","峨眉國中","峨眉國小","峨眉國民中學","峨眉國民小學","峨眉鄉立托兒所","峰谷國小","峰谷國民小學","崁腳國小","崁腳國民小學","崁頂國小","崁頂國民小學","崁頂鄉立托兒所","崇仁醫護學校","崇仁醫護管理專科學校","崇倫國中","崇倫國民中學","崇先","崇先高級中學","崇光國小","崇光國民小學","崇右技術學院","崇右技院","崇和國小","崇和國民小學","崇學國小","崇學國民小學","崇實高工","崇德國中","崇德國小","崇德國民中學","崇德國民小學","崇文國中","崇文國小","崇文國民中學","崇文國民小學","崇明國中","崇明國小","崇明國民中學","崇明國民小學","崇林國中","崇林國民中學","崇蘭國小","崇蘭國民小學","崎峰國小","崎峰國民小學","崑山國小","崑山國民小學","崑山技術學院","崑山科技大學","崑山高中","崑山高級中學","崑科","崑科大","崙坪國小","崙坪國民小學","崙山國小","崙山國民小學","崙背國中","崙背國小","崙背國民中學","崙背國民小學","崙背鄉立托兒所","崙豐國小","崙豐國民小學","崙雅國小","崙雅國民小學","嵵裡國小","嵵裡國民小學","嶺口國小","嶺口國民小學","嶺東技術學院","嶺東科技大學","嶺東高中","嶺東高級中學","嶺科大","左營國中","左營國小","左營國民中學","左營國民小學","左營高中","左營高級中學","左鎮國中","左鎮國小","左鎮國民中學","左鎮國民小學","巨人高中","巨人高級中學","巴崚國小","巴崚國民小學","巴楠花部落小學","市北","市北大","市北附小","布新國小","布新國民小學","布袋國中","布袋國小","布袋國民中學","布袋國民小學","布袋鎮立托兒所","師大","師大附中","師範學院","平南國中","平南國民中學","平和國中","平和國小","平和國民中學","平和國民小學","平林國小","平林國民小學","平溪國中","平溪國小","平溪國民中學","平溪國民小學","平溪托兒所","平等國小","平等國民小學","平興國中","平興國小","平興國民中學","平興國民小學","平鎮國中","平鎮國民中學","平鎮幼兒園","平鎮高中","平鎮高級中等學校","幸安國小","幸安國民小學","幸福國中","幸福國小","幸福國民中學","幸福國民小學","幸福幼兒園","康壽國小","康壽國民小學","康寧國小","康寧國民小學","康寧護專","康寧醫護暨管理專科學校","康樂國小","康樂國民小學","廉使國小","廉使國民小學","廍子國小","廍子國民小學","廣原國小","廣原國民小學","廣安國小","廣安國民小學","廣福國小","廣福國民小學","廣興國小","廣興國民小學","廬山國小","廬山國民小學","延和國中","延和國民中學","延平國中","延平國小","延平國民中學","延平國民小學","延平鄉立托兒所","延平高中","延平高級中學","建中國小","建中國民小學","建功國小","建功國民小學","建功高中","建功高級中學","建台高中","建台高級中學","建和國小","建和國民小學","建國國中","建國國小","建國國民中學","建國國民小學","建國技術學院","建國高中","建國高級中學","建安國小","建安國民小學","建山國小","建山國民小學","建平國小","建平國民小學","建德國中","建德國小","建德國民中學","建德國民小學","建德幼兒園","建德幼稚園","建成國中","建成國民中學","建成托兒所","建新國小","建新國民小學","建業高中","建業高級中學","建臺高中","建臺高級中學","建興國中","建興國小","建興國民中學","建興國民小學","建華國中","建華國小","建華國民中學","建華國民小學","建陽國小","建陽國民小學","弓鞋國小","弓鞋國民小學","弘光","弘光技術學院","弘光科技大學","弘德工商","弘德高級工商職業學校","弘道國中","弘道國民中學","強恕高中","強恕高級中學","彌陀國中","彌陀國小","彌陀國民中學","彌陀國民小學","彭厝國小","彭厝國民小學","彭福國小","彭福國民小學","彰中","彰化女子高級中學","彰化市立托兒所","彰化師範大學","彰化師範大學附屬高級工業職業學校","彰化特殊教育學校","彰化縣二林鎮中正國民小學","彰化縣二林鎮中興國民小學","彰化縣二林鎮二林國民小學","彰化縣二林鎮廣興國民小學","彰化縣二林鎮新生國民小學","彰化縣二林鎮立托兒所","彰化縣二林鎮育德國民小學","彰化縣二林鎮興華國民小學","彰化縣二林鎮萬合國民小學","彰化縣二林鎮萬興國民小學","彰化縣二林鎮香田國民小學","彰化縣二水鄉二水國民小學","彰化縣二水鄉復興國民小學","彰化縣二水鄉源泉國民小學","彰化縣二水鄉立托兒所","彰化縣伸港鄉伸仁國民小學","彰化縣伸港鄉伸東國民小學","彰化縣伸港鄉大同國民小學","彰化縣伸港鄉新港國民小學","彰化縣伸港鄉立托兒所","彰化縣北斗鎮北斗國民小學","彰化縣北斗鎮大新國民小學","彰化縣北斗鎮立托兒所","彰化縣北斗鎮萬來國民小學","彰化縣北斗鎮螺陽國民小學","彰化縣北斗鎮螺青國民小學","彰化縣和美鎮和仁國民小學","彰化縣和美鎮和東國民小學","彰化縣和美鎮和美國民小學","彰化縣和美鎮培英國民小學","彰化縣和美鎮大嘉國民小學","彰化縣和美鎮大榮國民小學","彰化縣和美鎮新庄國民小學","彰化縣和美鎮立托兒所","彰化縣員林市僑信國民小學","彰化縣員林市員東國民小學","彰化縣員林市員林國民小學","彰化縣員林市明湖國民小學","彰化縣員林市東山國民小學","彰化縣員林市育英國民小學","彰化縣員林市青山國民小學","彰化縣員林市靜修國民小學","彰化縣員林市饒明國民小學","彰化縣員林鎮立托兒所","彰化縣埔心鄉埔心國民小學","彰化縣埔心鄉太平國民小學","彰化縣埔心鄉明聖國民小學","彰化縣埔心鄉梧鳳國民小學","彰化縣埔心鄉立托兒所","彰化縣埔心鄉羅厝國民小學","彰化縣埔心鄉舊館國民小學","彰化縣埔心鄉鳳霞國民小學","彰化縣埔鹽鄉南港國民小學","彰化縣埔鹽鄉埔鹽國民小學","彰化縣埔鹽鄉大園國民小學","彰化縣埔鹽鄉天盛國民小學","彰化縣埔鹽鄉好修國民小學","彰化縣埔鹽鄉新水國民小學","彰化縣埔鹽鄉永樂國民小學","彰化縣埔鹽鄉立托兒所","彰化縣埤頭鄉中和國民小學","彰化縣埤頭鄉合興國民小學","彰化縣埤頭鄉埤頭國民小學","彰化縣埤頭鄉大湖國民小學","彰化縣埤頭鄉立托兒所","彰化縣埤頭鄉芙朝國民小學","彰化縣埤頭鄉豐崙國民小學","彰化縣大城鄉大城國民小學","彰化縣大城鄉永光國民小學","彰化縣大城鄉潭墘國民小學","彰化縣大城鄉立托兒所","彰化縣大城鄉美豐國民小學","彰化縣大城鄉西港國民小學","彰化縣大城鄉頂庄國民小學","彰化縣大村鄉大村國民小學","彰化縣大村鄉大西國民小學","彰化縣大村鄉村上國民小學","彰化縣大村鄉村東國民小學","彰化縣大村鄉立托兒所","彰化縣彰化市三民國民小學","彰化縣彰化市中山國民小學","彰化縣彰化市南興國民小學","彰化縣彰化市南郭國民小學","彰化縣彰化市國聖國民小學","彰化縣彰化市大成國民小學","彰化縣彰化市大成幼稚園","彰化縣彰化市大竹國民小學","彰化縣彰化市平和國民小學","彰化縣彰化市忠孝國民小學","彰化縣彰化市快官國民小學","彰化縣彰化市東芳國民小學","彰化縣彰化市民生國民小學","彰化縣彰化市泰和國民小學","彰化縣彰化市石牌國民小學","彰化縣彰化市立托兒所","彰化縣彰化市聯興國民小學","彰化縣永靖鄉德興國民小學","彰化縣永靖鄉永興國民小學","彰化縣永靖鄉永靖國民小學","彰化縣永靖鄉福德國民小學","彰化縣永靖鄉福興國民小學","彰化縣永靖鄉立托兒所","彰化縣溪州鄉三條國民小學","彰化縣溪州鄉僑義國民小學","彰化縣溪州鄉南州國民小學","彰化縣溪州鄉圳寮國民小學","彰化縣溪州鄉大莊國民小學","彰化縣溪州鄉成功國民小學","彰化縣溪州鄉水尾國民小學","彰化縣溪州鄉溪州國民小學","彰化縣溪州鄉潮洋國民小學","彰化縣溪州鄉立托兒所","彰化縣溪湖鎮媽厝國民小學","彰化縣溪湖鎮東溪國民小學","彰化縣溪湖鎮湖北國民小學","彰化縣溪湖鎮湖南國民小學","彰化縣溪湖鎮湖東國民小學","彰化縣溪湖鎮湖西國民小學","彰化縣溪湖鎮溪湖國民小學","彰化縣溪湖鎮立托兒所","彰化縣田中鎮三潭國民小學","彰化縣田中鎮內安國民小學","彰化縣田中鎮大安國民小學","彰化縣田中鎮新民國民小學","彰化縣田中鎮明禮國民小學","彰化縣田中鎮東和國民小學","彰化縣田中鎮田中國民小學","彰化縣田中鎮立托兒所","彰化縣田尾鄉仁豐國民小學","彰化縣田尾鄉南鎮國民小學","彰化縣田尾鄉田尾國民小學","彰化縣田尾鄉立托兒所","彰化縣田尾鄉陸豐國民小學","彰化縣社頭鄉崙雅國民小學","彰化縣社頭鄉朝興國民小學","彰化縣社頭鄉橋頭國民小學","彰化縣社頭鄉清水國民小學","彰化縣社頭鄉湳雅國民小學","彰化縣社頭鄉社頭國民小學","彰化縣社頭鄉立托兒所","彰化縣社頭鄉舊社國民小學","彰化縣福興鄉大興國民小學","彰化縣福興鄉文昌國民小學","彰化縣福興鄉日新國民小學","彰化縣福興鄉永豐國民小學","彰化縣福興鄉立托兒所","彰化縣福興鄉管嶼國民小學","彰化縣福興鄉育新國民小學","彰化縣福興鄉西勢國民小學","彰化縣秀水鄉明正國民小學","彰化縣秀水鄉秀水國民小學","彰化縣秀水鄉立托兒所","彰化縣秀水鄉育民國民小學","彰化縣秀水鄉華龍國民小學","彰化縣秀水鄉陜西國民小學","彰化縣秀水鄉馬興國民小學","彰化縣私立培元高級中學","彰化縣私立大慶高級商工職業學校","彰化縣私立文興高級中學","彰化縣私立正德高級中學","彰化縣私立精誠高級中學","彰化縣私立達德高級商工職業學校","彰化縣立二林高級中學","彰化縣立二水國民中學","彰化縣立伸港國民中學","彰化縣立信義國民中小學","彰化縣立北斗國民中學","彰化縣立原斗國民中小學","彰化縣立和美高級中學","彰化縣立和群國民中學","彰化縣立員林國民中學","彰化縣立埔心國民中學","彰化縣立埔鹽國民中學","彰化縣立埤頭國民中學","彰化縣立大同國民中學","彰化縣立大城國民中學","彰化縣立大村國民中學","彰化縣立彰化藝術高級中學","彰化縣立彰安國民中學","彰化縣立彰德國民中學","彰化縣立彰泰國民中學","彰化縣立彰興國民中學","彰化縣立成功高級中學","彰化縣立明倫國民中學","彰化縣立永靖國民中學","彰化縣立溪州國民中學","彰化縣立溪湖國民中學","彰化縣立溪陽國民中學","彰化縣立田中高級中學","彰化縣立田尾國民中學","彰化縣立社頭國民中學","彰化縣立福興國民中學","彰化縣立秀水國民中學","彰化縣立竹塘國民中學","彰化縣立線西國民中學","彰化縣立芬園國民中學","彰化縣立花壇國民中學","彰化縣立芳苑國民中學","彰化縣立草湖國民中學","彰化縣立萬興國民中學","彰化縣立陽明國民中學","彰化縣立鹿江國際中小學","彰化縣立鹿港國民中學","彰化縣立鹿鳴國民中學","彰化縣竹塘鄉土庫國民小學","彰化縣竹塘鄉民靖國民小學","彰化縣竹塘鄉田頭國民小學","彰化縣竹塘鄉立托兒所","彰化縣竹塘鄉竹塘國民小學","彰化縣竹塘鄉長安國民小學","彰化縣線西鄉曉陽國民小學","彰化縣線西鄉立托兒所","彰化縣線西鄉線西國民小學","彰化縣芬園鄉同安國民小學","彰化縣芬園鄉富山國民小學","彰化縣芬園鄉寶山國民小學","彰化縣芬園鄉文德國民小學","彰化縣芬園鄉立托兒所","彰化縣芬園鄉芬園國民小學","彰化縣芬園鄉茄荖國民小學","彰化縣花壇鄉三春國民小學","彰化縣花壇鄉僑愛國民小學","彰化縣花壇鄉文祥國民小學","彰化縣花壇鄉白沙國民小學","彰化縣花壇鄉立托兒所","彰化縣花壇鄉花壇國民小學","彰化縣花壇鄉華南國民小學","彰化縣芳苑鄉建新國民小學","彰化縣芳苑鄉後寮國民小學","彰化縣芳苑鄉新寶國民小學","彰化縣芳苑鄉民權國民小學","彰化縣芳苑鄉漢寶國民小學","彰化縣芳苑鄉王功國民小學","彰化縣芳苑鄉立托兒所","彰化縣芳苑鄉育華國民小學","彰化縣芳苑鄉芳苑國民小學","彰化縣芳苑鄉草湖國民小學","彰化縣芳苑鄉路上國民小學","彰化縣鹿港鎮文開國民小學","彰化縣鹿港鎮新興國民小學","彰化縣鹿港鎮東興國民小學","彰化縣鹿港鎮洛津國民小學","彰化縣鹿港鎮海埔國民小學","彰化縣鹿港鎮立托兒所","彰化縣鹿港鎮草港國民小學","彰化縣鹿港鎮頂番國民小學","彰化縣鹿港鎮鹿東國民小學","彰化縣鹿港鎮鹿港國民小學","彰化藝中","彰化藝術高級中學","彰化藝高","彰化高商","彰化高級中學","彰化高級商業職業學校","彰女","彰安國中","彰安國民中學","彰師","彰師教特","彰師附工","彰德國中","彰德國民中學","彰泰國中","彰泰國民中學","彰興國中","彰興國民中學","彰藝","後勁國中","後勁國小","後勁國民中學","後勁國民小學","後埔國小","後埔國民小學","後塘國小","後塘國民小學","後壁國中","後壁國小","後壁國民中學","後壁國民小學","後壁高中","後壁高級中學","後寮國小","後寮國民小學","後庄國小","後庄國民小學","後港國中","後港國小","後港國民中學","後港國民小學","後營國小","後營國民小學","後甲國中","後甲國民中學","後紅國小","後紅國民小學","後龍國中","後龍國小","後龍國民中學","後龍國民小學","後龍鎮立托兒所","復國幼兒園","復國幼稚園","復安國小","復安國民小學","復旦國小","復旦國民小學","復興區幼兒園","復興區立幼兒園","復興國中","復興國小","復興國民中學","復興國民小學","復興實中","復興實驗高級中學","復興高中","復興高級中學","復華高中","復華高級中學","德光女中","德光女子高中","德光女子高級中學","德光女高","德化國小","德化國民小學","德協國小","德協國民小學","德南國小","德南國民小學","德和國小","德和國民小學","德明技術學院","德明科大","德明財經科技大學","德武國小","德武國民小學","德育技術學院","德興國小","德興國民小學","德霖","德霖技術學院","德音國小","德音國民小學","德高國小","德高國民小學","德龍國小","德龍國民小學","志仁家商","志仁高中職業進修學校","志仁高級中學職業進修學校","志仁高級家事商業學校","志學國小","志學國民小學","志成商工","志成高級商工職業學校","志清國中","志清國小","志清國民中學","志清國民小學","志航國小","志航國民小學","志開實小","志開實驗小學","忠信國小","忠信國民小學","忠信高中","忠信高級中學","忠和國中","忠和國小","忠和國民中學","忠和國民小學","忠孝國中","忠孝國小","忠孝國民中學","忠孝國民小學","忠山國小","忠山國民小學","忠明國小","忠明國民小學","忠明高中","忠明高級中學","忠福國小","忠福國民小學","忠義國小","忠義國民小學","忠貞國小","忠貞國民小學","快官國小","快官國民小學","快樂國小","快樂國民小學","思賢國小","思賢國民小學","恆商","恆春國中","恆春國小","恆春國民中學","恆春國民小學","恆春高級工商職業學校","恕德家商","恕德高級家事商業職業學校","惇敘工商","惇敘高級工商職業學校","惠來國小","惠來國民小學","惠文國小","惠文國民小學","惠文高中","惠文高級中學","惠農國小","惠農國民小學","愛國國小","愛國國民小學","愛群國小","愛群國民小學","愛蘭國小","愛蘭國民小學","慈幼工商","慈幼高級工商職業學校","慈心華德福教育實驗高級中等學校","慈惠醫專","慈惠醫護管理專科學校","慈惠高級護理助產職業學校","慈文國中","慈文國小","慈文國民中學","慈文國民小學","慈濟國小","慈濟國民小學","慈濟大學","慈濟大學實驗國民小學","慈濟大學附屬高級中學","慈濟大實小","慈濟實小","慈濟技術學院","慈濟醫學暨人文社會學院","慈濟醫學院","慈濟附中","慈濟高中","慈濟高級中學","慎齋國小","慎齋國民小學","慧燈高中","慧燈高級中學","憲明國小","憲明國民小學","懷生國中","懷生國小","懷生國民中學","懷生國民小學","成功商業水產職業學校","成功商水","成功國中","成功國小","成功國民中學","成功國民小學","成功大學","成功大學附設高級工業職業進修學校","成功托兒所","成功特殊教育學校","成功鎮立托兒所","成功高中","成功高級中學","成城國小","成城國民小學","成大","成大附工","成州國小","成州國民小學","成德國中","成德國小","成德國民中學","成德國民小學","成德高中","成德高級中學","成淵","成淵高級中學","成福國小","成福國民小學","成龍國小","成龍國民小學","戲曲學院","拯民國小","拯民國民小學","指南國小","指南國民小學","振興國小","振興國民小學","排路國小","排路國民小學","插角國小","插角國民小學","揚子高中","揚子高級中學","援中國小","援中國民小學","政大","政大附中","政工幹校","政工幹部學校","政治大學","政治大學附屬高級中學","敏惠醫專","敏惠醫護管理專科學校","敦化國中","敦化國小","敦化國民中學","敦化國民小學","敦和國小","敦和國民小學","敬恆中小學","敬恆國民中小學","文元國小","文元國民小學","文光國中","文光國小","文光國民中學","文光國民小學","文化國中","文化國小","文化國民中學","文化國民小學","文和實小","文和實驗小學","文大","文安國小","文安國民小學","文官學院","文山國中","文山國小","文山國民中學","文山國民小學","文山幼兒園","文山托兒所","文山特教","文山特殊教育學校","文山社區大學","文山社大","文山高中","文山高級中學","文峰國小","文峰國民小學","文府國中","文府國小","文府國民中學","文府國民小學","文德國小","文德國民小學","文德女中","文德女子高級中學","文心國小","文心國民小學","文昌國中","文昌國小","文昌國民中學","文昌國民小學","文林國中","文林國小","文林國民中學","文林國民小學","文樂國小","文樂國民小學","文欣國小","文欣國民小學","文正國小","文正國民小學","文武國小","文武國民小學","文湖國小","文湖國民小學","文澳國小","文澳國民小學","文生高中","文生高級中學","文祥國小","文祥國民小學","文聖國小","文聖國民小學","文興","文興國民小學","文興高中","文興高級中學","文苑國小","文苑國民小學","文英國中","文英國民中學","文華國小","文華國民小學","文華高中","文華高級中等學校","文藻","文藻外語學院","文蘭國小","文蘭國民小學","文賢","文賢國中","文賢國小","文賢國民中學","文賢國民小學","文開國小","文開國民小學","文雅國小","文雅國民小學","斗六國中","斗六國小","斗六國民中學","斗六國民小學","斗六家商","斗六市立托兒所","斗六高級中學","斗六高級家事商業職業學校","斗南國小","斗南國民小學","斗南鎮立幼兒園","斗南鎮立托兒所","斗南高中","斗南高級中學","斗煥國小","斗煥國民小學","斗高","新上國小","新上國民小學","新光國中","新光國小","新光國民中學","新光國民小學","新勢國小","新勢國民小學","新化國中","新化國小","新化國民中學","新化國民小學","新化幼兒園","新化托兒所","新化高中","新化高工","新化高級中學","新化高級工業職業學校","新北市三峽區三峽國民小學","新北市三峽區中園國民小學","新北市三峽區五寮國民小學","新北市三峽區介壽國民小學","新北市三峽區北大國民小學","新北市三峽區大埔國民小學","新北市三峽區大成國民小學","新北市三峽區安溪國民小學","新北市三峽區建安國民小學","新北市三峽區成福國民小學","新北市三峽區插角國民小學","新北市三峽區有木國民小學","新北市三峽區民義國民小學","新北市三峽區龍埔國民小學","新北市三芝區三芝國民小學","新北市三芝區橫山國民小學","新北市三芝區興華國民小學","新北市三重區三光國民小學","新北市三重區三重國民小學","新北市三重區二重國民小學","新北市三重區五華國民小學","新北市三重區修德國民小學","新北市三重區光榮國民小學","新北市三重區光興國民小學","新北市三重區厚德國民小學","新北市三重區正義國民小學","新北市三重區永福國民小學","新北市三重區碧華國民小學","新北市三重區興穀國民小學","新北市三重區重陽國民小學","新北市三重區集美國民小學","新北市中和區中和國民小學","新北市中和區光復國民小學","新北市中和區復興國民小學","新北市中和區景新國民小學","新北市中和區秀山國民小學","新北市中和區積穗國民小學","新北市中和區自強國民小學","新北市中和區興南國民小學","新北市中和區錦和國民小學","新北市五股區五股國民小學","新北市五股區德音國民小學","新北市五股區成州國民小學","新北市五股區更寮國民小學","新北市八里區八里國民小學","新北市八里區大崁國民小學","新北市八里區米倉國民小學","新北市八里區長坑國民小學","新北市土城區土城國民小學","新北市土城區安和國民小學","新北市土城區廣福國民小學","新北市土城區樂利國民小學","新北市土城區清水國民小學","新北市土城區頂埔國民小學","新北市坪林區坪林國民小學","新北市平溪區十分國民小學","新北市平溪區平溪國民小學","新北市平溪區菁桐國民小學","新北市新店區中正國民小學","新北市新店區北新國民小學","新北市新店區大豐國民小學","新北市新店區安坑國民小學","新北市新店區屈尺國民小學","新北市新店區新和國民小學","新北市新店區新店國民小學","新北市新店區直潭國民小學","新北市新店區雙城國民小學","新北市新店區雙峰國民小學","新北市新店區青潭國民小學","新北市新店區龜山國民小學","新北市新莊區中信國民小學","新北市新莊區中港國民小學","新北市新莊區丹鳳國民小學","新北市新莊區光華國民小學","新北市新莊區國泰國民小學","新北市新莊區思賢國民小學","新北市新莊區新泰國民小學","新北市新莊區新莊國民小學","新北市新莊區昌平國民小學","新北市新莊區昌隆國民小學","新北市新莊區榮富國民小學","新北市新莊區民安國民小學","新北市新莊區興化國民小學","新北市新莊區裕民國民小學","新北市新莊區豐年國民小學","新北市新莊區頭前國民小學","新北市板橋區中山國民小學","新北市板橋區信義國民小學","新北市板橋區國光國民小學","新北市板橋區埔墘國民小學","新北市板橋區大觀國民小學","新北市板橋區實踐國民小學","新北市板橋區後埔國民小學","新北市板橋區文德國民小學","新北市板橋區文聖國民小學","新北市板橋區新埔國民小學","新北市板橋區板橋國民小學","新北市板橋區江翠國民小學","新北市板橋區沙崙國民小學","新北市板橋區海山國民小學","新北市板橋區溪洲國民小學","新北市板橋區莒光國民小學","新北市板橋區重慶國民小學","新北市林口區南勢國民小學","新北市林口區嘉寶國民小學","新北市林口區新林國民小學","新北市林口區東湖國民小學","新北市林口區林口國民小學","新北市林口區瑞平國民小學","新北市林口區興福國民小學","新北市林口區頭湖國民小學","新北市林口區麗園國民小學","新北市林口區麗林國民小學","新北市樹林區三多國民小學","新北市樹林區大同國民小學","新北市樹林區山佳國民小學","新北市樹林區彭福國民小學","新北市樹林區文林國民小學","新北市樹林區柑園國民小學","新北市樹林區樹林國民小學","新北市樹林區武林國民小學","新北市樹林區育德國民小學","新北市樹林區育林國民小學","新北市永和區永和國民小學","新北市永和區永平國民小學","新北市永和區秀朗國民小學","新北市永和區網溪國民小學","新北市永和區頂溪國民小學","新北市汐止區保長國民小學","新北市汐止區北峰國民小學","新北市汐止區北港國民小學","新北市汐止區崇德國民小學","新北市汐止區東山國民小學","新北市汐止區樟樹國民小學","新北市汐止區汐止國民小學","新北市汐止區白雲國民小學","新北市汐止區秀峰國民小學","新北市汐止區金龍國民小學","新北市汐止區長安國民小學","新北市泰山區同榮國民小學","新北市泰山區明志國民小學","新北市泰山區泰山國民小學","新北市泰山區義學國民小學","新北市淡水區中泰國民小學","新北市淡水區坪頂國民小學","新北市淡水區天生國民小學","新北市淡水區屯山國民小學","新北市淡水區忠山國民小學","新北市淡水區文化國民小學","新北市淡水區新市國民小學","新北市淡水區新興國民小學","新北市淡水區水源國民小學","新北市淡水區淡水國民小學","新北市淡水區淡海國民小學","新北市淡水區竹圍國民小學","新北市淡水區育英國民小學","新北市淡水區興仁國民小學","新北市淡水區鄧公國民小學","新北市深坑區深坑國民小學","新北市烏來區福山國民小學","新北市瑞芳區九份國民小學","新北市瑞芳區吉慶國民小學","新北市瑞芳區濂洞國民小學","新北市瑞芳區猴硐國民小學","新北市瑞芳區瑞亭國民小學","新北市瑞芳區瑞柑國民小學","新北市瑞芳區瑞濱國民小學","新北市瑞芳區瑞芳國民小學","新北市瑞芳區瓜山國民小學","新北市瑞芳區義方國民小學","新北市瑞芳區鼻頭國民小學","新北市石碇區和平國民小學","新北市石碇區永定國民小學","新北市石碇區石碇國民小學","新北市石碇區雲海國民小學","新北市石門區乾華國民小學","新北市石門區石門國民小學","新北市石門區老梅國民小學","新北市立三和國民中學","新北市立三多國民中學","新北市立三峽國民中學","新北市立三峽幼兒園","新北市立三峽托兒所","新北市立三民高級中學","新北市立三芝國民中學","新北市立三芝幼兒園","新北市立三芝托兒所","新北市立三重幼兒園","新北市立三重托兒所","新北市立三重高級中學","新北市立三重高級商工職業學校","新北市立中和國民中學","新北市立中和幼兒園","新北市立中和托兒所","新北市立中和高級中學","新北市立中山國民中學","新北市立中平國民中學","新北市立中正國民中學","新北市立丹鳳高級中學","新北市立二重國民中學","新北市立五峰國民中學","新北市立五股國民中學","新北市立五股幼兒園","新北市立五股托兒所","新北市立佳林國民中學","新北市立光復高級中學","新北市立光榮國民中學","新北市立八里國民中學","新北市立八里幼兒園","新北市立八里托兒所","新北市立北大高級中學","新北市立土城國民中學","新北市立土城幼兒園","新北市立土城托兒所","新北市立坪林國民中學","新北市立坪林幼兒園","新北市立坪林托兒所","新北市立大觀國民中學","新北市立安康高級中學","新北市立安溪國民中學","新北市立尖山國民中學","新北市立崇林國民中學","新北市立平溪國民中學","新北市立平溪托兒所","新北市立忠孝國民中學","新北市立文山國民中學","新北市立新北特殊教育學校","新北市立新北高級中學","新北市立新北高級工業職業學校","新北市立新埔國民中學","新北市立新店幼兒園","新北市立新店托兒所","新北市立新店高級中學","新北市立新泰國民中學","新北市立新莊國民中學","新北市立新莊幼兒園","新北市立新莊托兒所","新北市立新莊高級中學","新北市立明德高級中學","新北市立明志國民中學","新北市立板橋國民中學","新北市立板橋幼兒園","新北市立板橋托兒所","新北市立板橋高級中學","新北市立林口國民中學","新北市立林口幼兒園","新北市立林口托兒所","新北市立林口高級中學","新北市立柑園國民中學","新北市立桃子腳國民中小學","新北市立樟樹國際實創高級中等學校","新北市立樹林幼兒園","新北市立樹林托兒所","新北市立樹林高級中學","新北市立欽賢國民中學","新北市立正德國民中學","新北市立永和國民中學","新北市立永和幼兒園","新北市立永和托兒所","新北市立永平高級中學","新北市立汐止國民中學","新北市立汐止托兒所","新北市立江翠國民中學","新北市立泰山國民中學","新北市立泰山幼兒園","新北市立泰山托兒所","新北市立泰山高級中學","新北市立海山高級中學","新北市立淡水國民中學","新北市立淡水幼兒園","新北市立淡水托兒所","新北市立淡水高級商工職業學校","新北市立深坑國民中學","新北市立深坑幼兒園","新北市立深坑托兒所","新北市立清水高級中學","新北市立溪崑國民中學","新北市立漳和國民中學","新北市立烏來國民中小學","新北市立烏來幼兒園","新北市立烏來托兒所","新北市立瑞芳國民中學","新北市立瑞芳幼兒園","新北市立瑞芳托兒所","新北市立瑞芳高級工業職業學校","新北市立石碇幼兒園","新北市立石碇托兒所","新北市立石碇高級中學","新北市立石門國民中學","新北市立石門幼兒園","新北市立石門托兒所","新北市立碧華國民中學","新北市立福和國民中學","新北市立福營國民中學","新北市立秀峰高級中學","新北市立積穗國民中學","新北市立竹圍高級中學","新北市立義學國民中學","新北市立育林國民中學","新北市立自強國民中學","新北市立萬里國民中學","新北市立萬里幼兒園","新北市立萬里托兒所","新北市立蘆洲國民中學","新北市立蘆洲幼兒園","新北市立蘆洲托兒所","新北市立豐珠國民中小學","新北市立貢寮國民中學","新北市立貢寮幼兒園","新北市立貢寮托兒所","新北市立達觀國民中小學","新北市立重慶國民中學","新北市立金山幼兒園","新北市立金山托兒所","新北市立金山高級中學","新北市立錦和高級中學","新北市立雙溪幼兒園","新北市立雙溪托兒所","新北市立雙溪高級中學","新北市立青山國民中小學","新北市立頭前國民中學","新北市立鳳鳴國民中學","新北市立鶯歌國民中學","新北市立鶯歌幼兒園","新北市立鶯歌托兒所","新北市立鶯歌高級工商職業學校","新北市立鷺江國民中學","新北市萬里區大坪國民小學","新北市萬里區大鵬國民小學","新北市萬里區崁腳國民小學","新北市萬里區萬里國民小學","新北市萬里區野柳國民小學","新北市蘆洲區仁愛國民小學","新北市蘆洲區忠義國民小學","新北市蘆洲區成功國民小學","新北市蘆洲區蘆洲國民小學","新北市蘆洲區鷺江國民小學","新北市貢寮區和美國民小學","新北市貢寮區澳底國民小學","新北市貢寮區福連國民小學","新北市貢寮區福隆國民小學","新北市貢寮區貢寮國民小學","新北市金山區三和國民小學","新北市金山區中角國民小學","新北市金山區金山國民小學","新北市金山區金美國民小學","新北市雙溪區上林國民小學","新北市雙溪區柑林國民小學","新北市雙溪區牡丹國民小學","新北市雙溪區雙溪國民小學","新北市鶯歌區中湖國民小學","新北市鶯歌區二橋國民小學","新北市鶯歌區建國國民小學","新北市鶯歌區昌福國民小學","新北市鶯歌區永吉國民小學","新北市鶯歌區鳳鳴國民小學","新北市鶯歌區鶯歌國民小學","新北特教","新北特殊教育學校","新北高中","新北高工","新北高級中學","新北高級工業職業學校","新南國小","新南國民小學","新和國小","新和國民小學","新嘉國小","新嘉國民小學","新圍國小","新圍國民小學","新園國中","新園國小","新園國民中學","新園國民小學","新坡國小","新坡國民小學","新城國中","新城國小","新城國民中學","新城國民小學","新城鄉立托兒所","新埔國中","新埔國小","新埔國民中學","新埔國民小學","新埔技術學院","新埔鎮立托兒所","新埤國中","新埤國小","新埤國民中學","新埤國民小學","新埤鄉立托兒所","新基高中","新基高級中學","新塭國小","新塭國民小學","新威國小","新威國民小學","新寶國小","新寶國民小學","新屋國小","新屋國民小學","新屋幼兒園","新屋高中","新屋高級中等學校","新山國小","新山國民小學","新岑國小","新岑國民小學","新市國中","新市國小","新市國民中學","新市國民小學","新市幼兒園","新市托兒所","新平國小","新平國民小學","新庄國小","新庄國民小學","新店國小","新店國民小學","新店幼兒園","新店托兒所","新店高中","新店高級中學","新成國小","新成國民小學","新明國中","新明國小","新明國民中學","新明國民小學","新星國小","新星國民小學","新東國中","新東國小","新東國民中學","新東國民小學","新林國小","新林國民小學","新榮國小","新榮國民小學","新樂國小","新樂國民小學","新橋國小","新橋國民小學","新民國中","新民國小","新民國民中學","新民國民小學","新民小學","新民高中","新民高級中學","新水國小","新水國民小學","新泰國中","新泰國小","新泰國民中學","新泰國民小學","新港中小學","新港國中","新港國小","新港國民中學","新港國民中小學","新港國民小學","新港藝術高級中學","新港藝高","新港鄉立托兒所","新湖國中","新湖國小","新湖國民中學","新湖國民小學","新營國小","新營國民小學","新營高中","新營高工","新營高級中學","新營高級工業職業學校","新生國中","新生國小","新生國民中學","新生國民小學","新生幼兒園","新生護專","新生醫護管理專科學校","新甲國小","新甲國民小學","新發國小","新發國民小學","新盛國小","新盛國民小學","新社國小","新社國民小學","新社托兒所","新社高中","新社高級中學","新科國中","新科國民中學","新竹國小","新竹國民小學","新竹女子高級中學","新竹市世界高級中學","新竹市北區北門國民小學","新竹市北區南寮國民小學","新竹市北區民富國民小學","新竹市北區舊社國民小學","新竹市北區西門國民小學","新竹市北區載熙國民小學","新竹市東區三民國民小學","新竹市東區建功國民小學","新竹市東區新竹國民小學","新竹市東區東園國民小學","新竹市東區東門國民小學","新竹市東區水源國民小學","新竹市東區科園國民小學","新竹市東區竹蓮國民小學","新竹市東區關埔國民小學","新竹市東區關東國民小學","新竹市東區陽光國民小學","新竹市東區青草湖國民小學","新竹市東區高峰國民小學","新竹市東區龍山國民小學","新竹市私立光復高級中學","新竹市私立曙光女子高級中學","新竹市私立磐石高級中學","新竹市私立義民高級中學","新竹市立三民國民中學","新竹市立光武國民中學","新竹市立光華國民中學","新竹市立內湖國民中學","新竹市立南華國民中學","新竹市立培英國民中學","新竹市立富禮國民中學","新竹市立幼稚園","新竹市立建功高級中學","新竹市立建華國民中學","新竹市立成德高級中學","新竹市立托兒所","新竹市立新科國民中學","新竹市立新竹幼兒園","新竹市立東區托兒所","新竹市立竹光國民中學","新竹市立育賢國民中學","新竹市立虎林國民中學","新竹市立香山區托兒所","新竹市立香山高級中學","新竹市華德福實驗學校","新竹市香山區內湖國民小學","新竹市香山區南隘國民小學","新竹市香山區大庄國民小學","新竹市香山區大湖國民小學","新竹市香山區朝山國民小學","新竹市香山區港南國民小學","新竹市香山區茄苳國民小學","新竹市香山區虎林國民小學","新竹市香山區頂埔國民小學","新竹市香山區香山國民小學","新竹師範學院","新竹幼兒園","新竹幼稚園","新竹教育大學","新竹特教","新竹特殊教育學校","新竹縣五峰鄉五峰國民小學","新竹縣五峰鄉桃山國民小學","新竹縣五峰鄉立托兒所","新竹縣五峰鄉花園國民小學","新竹縣仰德高級中學","新竹縣北埔鄉北埔國民小學","新竹縣北埔鄉大坪國民小學","新竹縣北埔鄉立托兒所","新竹縣寶山鄉寶山國民小學","新竹縣寶山鄉新城國民小學","新竹縣寶山鄉立托兒所","新竹縣寶山鄉雙溪國民小學","新竹縣尖石鄉嘉興國民小學","新竹縣尖石鄉尖石國民小學","新竹縣尖石鄉新光國民小學","新竹縣尖石鄉新樂國民小學","新竹縣尖石鄉梅花國民小學","新竹縣尖石鄉玉峰國民小學","新竹縣尖石鄉石磊國民小學","新竹縣尖石鄉秀巒國民小學","新竹縣尖石鄉立托兒所","新竹縣尖石鄉錦屏國民小學","新竹縣峨眉鄉富興國民小學","新竹縣峨眉鄉峨眉國民小學","新竹縣峨眉鄉立托兒所","新竹縣新埔鎮寶石國民小學","新竹縣新埔鎮文山國民小學","新竹縣新埔鎮新埔國民小學","新竹縣新埔鎮新星國民小學","新竹縣新埔鎮枋寮國民小學","新竹縣新埔鎮清水國民小學","新竹縣新埔鎮照東國民小學","新竹縣新埔鎮照門國民小學","新竹縣新埔鎮立托兒所","新竹縣新竹市新竹幼稚園","新竹縣新豐鄉埔和國民小學","新竹縣新豐鄉山崎國民小學","新竹縣新豐鄉新豐國民小學","新竹縣新豐鄉松林國民小學","新竹縣新豐鄉瑞興國民小學","新竹縣新豐鄉福興國民小學","新竹縣新豐鄉福龍國民小學","新竹縣新豐鄉立托兒所","新竹縣橫山鄉內灣國民小學","新竹縣橫山鄉大肚國民小學","新竹縣橫山鄉橫山國民小學","新竹縣橫山鄉沙坑國民小學","新竹縣橫山鄉田寮國民小學","新竹縣橫山鄉立托兒所","新竹縣湖口鄉中興國民小學","新竹縣湖口鄉信勢國民小學","新竹縣湖口鄉和興國民小學","新竹縣湖口鄉新湖國民小學","新竹縣湖口鄉湖口國民小學","新竹縣湖口鄉立托兒所","新竹縣湖口鄉華興國民小學","新竹縣湖口鄉長安國民小學","新竹縣私立內思高級工業職業學校","新竹縣私立忠信高級中學","新竹縣私立東泰高級中學","新竹縣立中正國民中學","新竹縣立二重國民中學","新竹縣立五峰國民中學","新竹縣立仁愛國民中學","新竹縣立六家高級中學","新竹縣立北埔國民中學","新竹縣立北平華德福實驗學校","新竹縣立博愛國民中學","新竹縣立員東國民中學","新竹縣立富光國民中學","新竹縣立寶山國民中學","新竹縣立尖石國民中學","新竹縣立峨眉國民中學","新竹縣立忠孝國民中學","新竹縣立成功國民中學","新竹縣立新埔國民中學","新竹縣立新湖國民中學","新竹縣立新豐國民中學","新竹縣立東興國民中學","新竹縣立橫山國民中學","新竹縣立湖口高級中學","新竹縣立照門國民中學","新竹縣立石光國民中學","新竹縣立竹北國民中學","新竹縣立竹東國民中學","新竹縣立竹東幼兒園","新竹縣立精華國民中學","新竹縣立自強國民中學","新竹縣立芎林國民中學","新竹縣立華山國民中學","新竹縣立關西國民中學","新竹縣立鳳岡國民中學","新竹縣竹北市中正國民小學","新竹縣竹北市光明國民小學","新竹縣竹北市六家國民小學","新竹縣竹北市十興國民小學","新竹縣竹北市博愛國民小學","新竹縣竹北市安興國民小學","新竹縣竹北市新港國民小學","新竹縣竹北市新社國民小學","新竹縣竹北市東海國民小學","新竹縣竹北市東興國民小學","新竹縣竹北市立托兒所","新竹縣竹北市竹仁國民小學","新竹縣竹北市竹北國民小學","新竹縣竹北市興隆國民小學","新竹縣竹北市豐田國民小學","新竹縣竹北市鳳岡國民小學","新竹縣竹北市麻園國民小學","新竹縣竹東鎮上館國民小學","新竹縣竹東鎮中山國民小學","新竹縣竹東鎮二重國民小學","新竹縣竹東鎮員崠國民小學","新竹縣竹東鎮大同國民小學","新竹縣竹東鎮瑞峰國民小學","新竹縣竹東鎮立托兒所","新竹縣竹東鎮竹中國民小學","新竹縣竹東鎮竹東國民小學","新竹縣竹東鎮竹東幼稚園","新竹縣竹東鎮陸豐國民小學","新竹縣芎林鄉五龍國民小學","新竹縣芎林鄉碧潭國民小學","新竹縣芎林鄉立托兒所","新竹縣芎林鄉芎林國民小學","新竹縣關西鎮南和國民小學","新竹縣關西鎮坪林國民小學","新竹縣關西鎮太平國民小學","新竹縣關西鎮東光國民小學","新竹縣關西鎮東安國民小學","新竹縣關西鎮玉山國民小學","新竹縣關西鎮石光國民小學","新竹縣關西鎮立托兒所","新竹縣關西鎮錦山國民小學","新竹縣關西鎮關西國民小學","新竹高商","新竹高工","新竹高級中學","新竹高級商業職業學校","新竹高級工業職業學校","新美國小","新美國民小學","新興國中","新興國小","新興國民中學","新興國民小學","新興高中","新興高級中學","新英國小","新英國民小學","新莊國中","新莊國小","新莊國民中學","新莊國民小學","新莊幼兒園","新莊托兒所","新莊高中","新莊高級中學","新藝高","新街國小","新街國民小學","新豐國中","新豐國小","新豐國民中學","新豐國民小學","新豐鄉立托兒所","新豐高中","新豐高級中學","新路國小","新路國民小學","新進國小","新進國民小學","新鄉國小","新鄉國民小學","新開國小","新開國民小學","新隆國小","新隆國民小學","新高國小","新高國民小學","方濟中學","方濟高級中學","旗尾國小","旗尾國民小學","旗山國中","旗山國小","旗山國民中學","旗山國民小學","旗山農工","旗山高級農工職業學校","旗津國中","旗津國小","旗津國民中學","旗津國民小學","旗美高中","旗美高級中學","日僑小學","日南國中","日南國小","日南國民中學","日南國民小學","日新國中","日新國小","日新國民中學","日新國民小學","日新工商","日新高級工商職業學校","旭光國小","旭光國民小學","旭光高中","旭光高級中學","昇平國中","昇平國民中學","昌平國小","昌平國民小學","昌福國小","昌福國民小學","昌隆國小","昌隆國民小學","明仁國中","明仁國民中學","明倫國中","明倫國小","明倫國民中學","明倫國民小學","明倫高中","明倫高級中學","明利國小","明利國民小學","明宗國小","明宗國民小學","明廉國小","明廉國民小學","明德國中","明德國小","明德國民中學","明德國民小學","明德女中","明德女子高中","明德女子高級中學","明德女高","明德高中","明德高級中學","明志國中","明志國小","明志國民中學","明志國民小學","明志技術學院","明志技術學院附設進修學院","明志科大","明志科技大學","明恥國小","明恥國民小學","明新技術學院","明新科大","明新科技大學","明正國中","明正國小","明正國民中學","明正國民小學","明湖國中","明湖國小","明湖國民中學","明湖國民小學","明潭國中","明潭國小","明潭國民中學","明潭國民小學","明禮國小","明禮國民小學","明義國中","明義國小","明義國民中學","明義國民小學","明聖國小","明聖國民小學","明華","明華國中","明華國民中學","明華高級中學","明誠高中","明誠高級中學","明道","明道國小","明道國民小學","明道大學","明道管理學院","明里國小","明里國民小學","明陽中學","春安國小","春安國民小學","春日國小","春日國民小學","春日鄉立托兒所","春陽國小","春陽國民小學","昭明國中","昭明國小","昭明國民中學","昭明國民小學","普仁國小","普仁國民小學","普台中小學","普台國民中小學","景山國小","景山國民小學","景文","景文技術學院","景文科技大學","景文高中","景文高級中學","景新國小","景新國民小學","景科大","景美國中","景美國小","景美國民中學","景美國民小學","景美女中","景美女子高級中學","景美女高","景義國小","景義國民小學","景興國中","景興國小","景興國民中學","景興國民小學","暖中","暖暖國小","暖暖國民小學","暖暖高級中學","暖江國小","暖江國民小學","暖西國小","暖西國民小學","暨南國際大學","暨南國際大學附屬高級中學","暨大","暨大附中","曉明女中","曉明女子高中","曉明女子高級中學","曉明女高","曉陽國小","曉陽國民小學","曙光女中","曙光女子高中","曙光女子高級中學","曙光女高","更寮國小","更寮國民小學","曹公國小","曹公國民小學","曾文商職","曾文家商","曾文農工","曾文高商","曾文高級家事商業職業學校","曾文高級農工職業學校","曾文高職","會稽國中","會稽國小","會稽國民中學","會稽國民小學","月津國小","月津國民小學","月眉國小","月眉國民小學","月美國小","月美國民小學","有木國小","有木國民小學","朗島國小","朗島國民小學","望嘉國小","望嘉國民小學","望安國中","望安國小","望安國民中學","望安國民小學","望安鄉立托兒所","朝山國小","朝山國民小學","朝興國小","朝興國民小學","朝陽國小","朝陽國民小學","朝陽技術學院","朝陽科大","朝陽科技大學","木柵國中","木柵國小","木柵國民中學","木柵國民小學","木柵托兒所","木柵高工","木柵高級工業職業學校","朴子國中","朴子國小","朴子國民中學","朴子國民小學","朴子市立托兒所","杉林國中","杉林國小","杉林國民中學","杉林國民小學","村上國小","村上國民小學","村東國小","村東國民小學","東中","東仁國中","東仁國民中學","東信國小","東信國民小學","東光國中","東光國小","東光國民中學","東光國民小學","東勢國中","東勢國小","東勢國民中學","東勢國民小學","東勢工業高級中等學校","東勢托兒所","東勢鄉立托兒所","東勢高工","東區托兒所","東南國中","東南國民中學","東南技術學院","東南科大","東南科技大學","東原國中","東原國小","東原國民中學","東原國民小學","東吳","東吳大學","東吳高級工業家事職業學校","東吳高職","東和國中","東和國小","東和國民中學","東和國民小學","東園國小","東園國民小學","東埔國小","東埔國民小學","東大","東大特教","東大附小","東大附體","東大體中","東女","東安國中","東安國小","東安國民中學","東安國民小學","東寧國小","東寧國民小學","東寶國小","東寶國民小學","東山國中","東山國小","東山國民中學","東山國民小學","東山高中","東山高級中學","東峰國中","東峰國民中學","東平國小","東平國民小學","東引中小學","東引國民中小學","東成國小","東成國民小學","東新國中","東新國小","東新國民中學","東新國民小學","東方工商","東方技術學院","東方高級工商職業學校","東明國中","東明國小","東明國民中學","東明國民小學","東榮國中","東榮國小","東榮國民中學","東榮國民小學","東汴國小","東汴國民小學","東河國小","東河國民小學","東河鄉立托兒所","東泰","東泰高級中學","東海國中","東海國小","東海國民中學","東海國民小學","東海大學","東海大學附屬高級中學","東海附中","東清國小","東清國民小學","東港國小","東港國民小學","東港幼稚園","東港海事","東港高中","東港高級中學","東港高級海事水產職業學校","東湖國中","東湖國小","東湖國民中學","東湖國民小學","東溪國小","東溪國民小學","東澳國小","東澳國民小學","東特","東石國中","東石國小","東石國民中學","東石國民小學","東石鄉立托兒所","東石高中","東石高級中學","東竹國小","東竹國民小學","東興國中","東興國小","東興國民中學","東興國民小學","東芳國小","東芳國民小學","東莒國小","東莒國民小學","東莞台商子弟學校","東莞學校","東華","東華國中","東華國民中學","東華大學","東華大學附設實驗國民小學","東華實小","東衛國小","東衛國民小學","東里國中","東里國小","東里國民中學","東里國民小學","東門國小","東門國民小學","東陽國小","東陽國民小學","東隆國小","東隆國民小學","松山商職","松山國小","松山國民小學","松山家商","松山工農","松山幼兒園","松山托兒所","松山社區大學","松山社大","松山高商","松山高級中學","松山高級商業家事職業學校","松山高級工農職業學校","松山高職","松林國小","松林國民小學","松梅國小","松梅國民小學","松浦國小","松浦國民小學","松竹國小","松竹國民小學","松高","板橋國中","板橋國小","板橋國民中學","板橋國民小學","板橋幼兒園","板橋托兒所","板橋高中","板橋高級中學","枋中","枋寮國小","枋寮國民小學","枋寮高級中學","枋山鄉立托兒所","林中國小","林中國民小學","林內國中","林內國小","林內國民中學","林內國民小學","林內鄉立托兒所","林厝國小","林厝國民小學","林口國中","林口國小","林口國民中學","林口國民小學","林口幼兒園","林口托兒所","林口高中","林口高級中學","林園國小","林園國民小學","林園高中","林園高級中學","林森國小","林森國民小學","林榮國小","林榮國民小學","林邊國中","林邊國小","林邊國民中學","林邊國民小學","林邊鄉立托兒所","林頭國小","林頭國民小學","林鳳國小","林鳳國民小學","果毅國小","果毅國民小學","柏村國小","柏村國民小學","柑園國中","柑園國小","柑園國民中學","柑園國民小學","柑林國小","柑林國民小學","柯林國小","柯林國民小學","柳林國小","柳林國民小學","柳溝國小","柳溝國民小學","柳營國中","柳營國小","柳營國民中學","柳營國民小學","柴林國小","柴林國民小學","栗林國小","栗林國民小學","格致國中","格致國民中學","桂林國小","桂林國民小學","桃園國中","桃園國小","桃園國民中學","桃園國民小學","桃園市中壢區中原國民小學","桃園市中壢區中壢國民小學","桃園市中壢區中平國民小學","桃園市中壢區中正國民小學","桃園市中壢區信義國民小學","桃園市中壢區元生國民小學","桃園市中壢區內壢國民小學","桃園市中壢區內定國民小學","桃園市中壢區大崙國民小學","桃園市中壢區富台國民小學","桃園市中壢區山東國民小學","桃園市中壢區忠福國民小學","桃園市中壢區新明國民小學","桃園市中壢區新街國民小學","桃園市中壢區普仁國民小學","桃園市中壢區林森國民小學","桃園市中壢區自立國民小學","桃園市中壢區興仁國民小學","桃園市中壢區興國國民小學","桃園市中壢區芭里國民小學","桃園市中壢區華勛國民小學","桃園市中壢區青埔國民小學","桃園市中壢區龍岡國民小學","桃園市八德區八德國民小學","桃園市八德區大勇國民小學","桃園市八德區大安國民小學","桃園市八德區大忠國民小學","桃園市八德區大成國民小學","桃園市八德區廣興國民小學","桃園市八德區瑞豐國民小學","桃園市八德區茄苳國民小學","桃園市八德區霄裡國民小學","桃園市大園區五權國民小學","桃園市大園區內海國民小學","桃園市大園區后厝國民小學","桃園市大園區圳頭國民小學","桃園市大園區埔心國民小學","桃園市大園區大園國民小學","桃園市大園區沙崙國民小學","桃園市大園區溪海國民小學","桃園市大園區潮音國民小學","桃園市大園區竹圍國民小學","桃園市大園區菓林國民小學","桃園市大園區陳康國民小學","桃園市大溪區中興國民小學","桃園市大溪區仁和國民小學","桃園市大溪區仁善國民小學","桃園市大溪區僑愛國民小學","桃園市大溪區內柵國民小學","桃園市大溪區南興國民小學","桃園市大溪區員樹林國民小學","桃園市大溪區大溪國民小學","桃園市大溪區永福國民小學","桃園市大溪區瑞祥國民小學","桃園市大溪區田心國民小學","桃園市大溪區百吉國民小學","桃園市大溪區福安國民小學","桃園市大溪區美華國民小學","桃園市平鎮區北勢國民小學","桃園市平鎮區南勢國民小學","桃園市平鎮區宋屋國民小學","桃園市平鎮區山豐國民小學","桃園市平鎮區平興國民小學","桃園市平鎮區復旦國民小學","桃園市平鎮區忠貞國民小學","桃園市平鎮區文化國民小學","桃園市平鎮區新勢國民小學","桃園市平鎮區新榮國民小學","桃園市平鎮區東勢國民小學","桃園市平鎮區東安國民小學","桃園市平鎮區祥安國民小學","桃園市平鎮區義興國民小學","桃園市復興區三光國民小學","桃園市復興區三民國民小學","桃園市復興區介壽國民小學","桃園市復興區光華國民小學","桃園市復興區奎輝國民小學","桃園市復興區巴崚國民小學","桃園市復興區立幼兒園","桃園市復興區羅浮國民小學","桃園市復興區義盛國民小學","桃園市復興區長興國民小學","桃園市復興區霞雲國民小學","桃園市復興區高義國民小學","桃園市新屋區北湖國民小學","桃園市新屋區啟文國民小學","桃園市新屋區埔頂國民小學","桃園市新屋區大坡國民小學","桃園市新屋區新屋國民小學","桃園市新屋區東明國民小學","桃園市新屋區永安國民小學","桃園市新屋區社子國民小學","桃園市新屋區笨港國民小學","桃園市新屋區蚵間國民小學","桃園市新屋區頭洲國民小學","桃園市桃園區中埔國民小學","桃園市桃園區中山國民小學","桃園市桃園區北門國民小學","桃園市桃園區南門國民小學","桃園市桃園區同安國民小學","桃園市桃園區同德國民小學","桃園市桃園區大有國民小學","桃園市桃園區大業國民小學","桃園市桃園區建國國民小學","桃園市桃園區建德國民小學","桃園市桃園區快樂國民小學","桃園市桃園區慈文國民小學","桃園市桃園區成功國民小學","桃園市桃園區文山國民小學","桃園市桃園區新埔國民小學","桃園市桃園區會稽國民小學","桃園市桃園區東門國民小學","桃園市桃園區桃園國民小學","桃園市桃園區永順國民小學","桃園市桃園區莊敬國民小學","桃園市桃園區西門國民小學","桃園市桃園區青溪國民小學","桃園市桃園區龍山國民小學","桃園市楊梅區上湖國民小學","桃園市楊梅區上田國民小學","桃園市楊梅區四維國民小學","桃園市楊梅區大同國民小學","桃園市楊梅區富岡國民小學","桃園市楊梅區楊心國民小學","桃園市楊梅區楊明國民小學","桃園市楊梅區楊梅國民小學","桃園市楊梅區水美國民小學","桃園市楊梅區瑞原國民小學","桃園市楊梅區瑞埔國民小學","桃園市楊梅區瑞塘國民小學","桃園市楊梅區瑞梅國民小學","桃園市楊梅區高榮國民小學","桃園市立中壢商業高級中等學校","桃園市立中壢國民中學","桃園市立中壢家事商業高級中等學校","桃園市立中壢幼兒園","桃園市立中壢高鐵幼兒園","桃園市立中興國民中學","桃園市立仁和國民中學","桃園市立仁美國民中學","桃園市立介壽國民中學","桃園市立光明國民中學","桃園市立內壢國民中學","桃園市立內壢高級中等學校","桃園市立八德國民中學","桃園市立八德幼兒園","桃園市立凌雲國民中學","桃園市立南崁國民中學","桃園市立南崁幼兒園","桃園市立南崁高級中等學校","桃園市立同德國民中學","桃園市立壽山高級中等學校","桃園市立大園國民中學","桃園市立大園國際高級中等學校","桃園市立大園幼兒園","桃園市立大坡國民中學","桃園市立大崗國民中學","桃園市立大崙國民中學","桃園市立大成國民中學","桃園市立大有國民中學","桃園市立大溪國民中學","桃園市立大溪幼兒園","桃園市立大溪高級中等學校","桃園市立大竹國民中學","桃園市立富岡國民中學","桃園市立山腳國民中學","桃園市立平南國民中學","桃園市立平興國民中學","桃園市立平鎮國民中學","桃園市立平鎮幼兒園","桃園市立平鎮高級中等學校","桃園市立幸福國民中學","桃園市立建國國民中學","桃園市立慈文國民中學","桃園市立文昌國民中學","桃園市立新屋幼兒園","桃園市立新屋高級中等學校","桃園市立新明國民中學","桃園市立會稽國民中學","桃園市立東安國民中學","桃園市立東興國民中學","桃園市立桃園國民中學","桃園市立桃園幼兒園","桃園市立桃園特殊教育學校","桃園市立桃園高級中等學校","桃園市立楊光國民中小學","桃園市立楊明國民中學","桃園市立楊梅國民中學","桃園市立楊梅幼兒園","桃園市立楊梅高級中等學校","桃園市立武漢國民中學","桃園市立武陵高級中等學校","桃園市立永安國民中學","桃園市立永豐高級中等學校","桃園市立瑞原國民中學","桃園市立瑞坪國民中學","桃園市立石門國民中學","桃園市立福豐國民中學","桃園市立竹圍國民中學","桃園市立經國國民中學","桃園市立自強國民中學","桃園市立興南國民中學","桃園市立草漯國民中學","桃園市立蘆竹幼兒園","桃園市立觀音國民中學","桃園市立觀音幼兒園","桃園市立觀音高級中等學校","桃園市立迴龍國民中小學","桃園市立過嶺國民中學","桃園市立陽明高級中等學校","桃園市立青埔國民中學","桃園市立青溪國民中學","桃園市立龍岡國民中學","桃園市立龍潭國民中學","桃園市立龍潭幼兒園","桃園市立龍潭高級中等學校","桃園市立龍興國民中學","桃園市立龜山國民中學","桃園市立龜山幼兒園","桃園市蘆竹區光明國民小學","桃園市蘆竹區公埔國民小學","桃園市蘆竹區南崁國民小學","桃園市蘆竹區外社國民小學","桃園市蘆竹區大竹國民小學","桃園市蘆竹區大華國民小學","桃園市蘆竹區山腳國民小學","桃園市蘆竹區新興國民小學","桃園市蘆竹區新莊國民小學","桃園市蘆竹區海湖國民小學","桃園市蘆竹區蘆竹國民小學","桃園市蘆竹區錦興國民小學","桃園市蘆竹區頂社國民小學","桃園市蘆竹區龍安國民小學","桃園市觀音區上大國民小學","桃園市觀音區保生國民小學","桃園市觀音區大潭國民小學","桃園市觀音區富林國民小學","桃園市觀音區崙坪國民小學","桃園市觀音區新坡國民小學","桃園市觀音區樹林國民小學","桃園市觀音區育仁國民小學","桃園市觀音區草漯國民小學","桃園市觀音區觀音國民小學","桃園市龍潭區三和國民小學","桃園市龍潭區三坑國民小學","桃園市龍潭區德龍國民小學","桃園市龍潭區武漢國民小學","桃園市龍潭區潛龍國民小學","桃園市龍潭區石門國民小學","桃園市龍潭區雙龍國民小學","桃園市龍潭區高原國民小學","桃園市龍潭區龍星國民小學","桃園市龍潭區龍源國民小學","桃園市龍潭區龍潭國民小學","桃園市龜山區南美國民小學","桃園市龜山區壽山國民小學","桃園市龜山區大坑國民小學","桃園市龜山區大埔國民小學","桃園市龜山區大崗國民小學","桃園市龜山區大湖國民小學","桃園市龜山區山頂國民小學","桃園市龜山區幸福國民小學","桃園市龜山區文欣國民小學","桃園市龜山區文華國民小學","桃園市龜山區新路國民小學","桃園市龜山區楓樹國民小學","桃園市龜山區樂善國民小學","桃園市龜山區福源國民小學","桃園市龜山區自強國民小學","桃園市龜山區長庚國民小學","桃園市龜山區龍壽國民小學","桃園市龜山區龜山國民小學","桃園幼兒園","桃園特教","桃園特殊教育學校","桃園農工","桃園高中","桃園高級中等學校","桃子腳中小學","桃子腳國民中小學","桃山國小","桃山國民小學","桃源國中","桃源國小","桃源國民中學","桃源國民小學","桐林國小","桐林國民小學","桶頭國小","桶頭國民小學","梅北國小","梅北國民小學","梅園國小","梅園國民小學","梅圳國小","梅圳國民小學","梅山國中","梅山國小","梅山國民中學","梅山國民小學","梅山鄉立托兒所","梅林國小","梅林國民小學","梅花國小","梅花國民小學","梓官國中","梓官國小","梓官國民中學","梓官國民小學","梗枋國小","梗枋國民小學","梧南國小","梧南國民小學","梧棲國中","梧棲國小","梧棲國民中學","梧棲國民小學","梧棲托兒所","梧鳳國小","梧鳳國民小學","梨山中小學","梨山國民中小學","棋山國小","棋山國民小學","椰油國小","椰油國民小學","楊光中小學","楊光國民中小學","楊心國小","楊心國民小學","楊明國中","楊明國小","楊明國民中學","楊明國民小學","楊梅國中","楊梅國小","楊梅國民中學","楊梅國民小學","楊梅幼兒園","楊梅高中","楊梅高級中等學校","楓林國小","楓林國民小學","楓樹國小","楓樹國民小學","楓港國小","楓港國民小學","楠梓國中","楠梓國小","楠梓國民中學","楠梓國民小學","楠梓特殊學校","楠梓高中","楠梓高級中學","楠西國中","楠西國小","楠西國民中學","楠西國民小學","楠陽國小","楠陽國民小學","榮富國小","榮富國民小學","榮華國小","榮華國民小學","槺榔國小","槺榔國民小學","樂利國小","樂利國民小學","樂合國小","樂合國民小學","樂善國小","樂善國民小學","樂業國小","樂業國民小學","樂群國小","樂群國民小學","樟中","樟原國小","樟原國民小學","樟山國小","樟山國民小學","樟樹國小","樟樹國民小學","樟樹國際實創高級中等學校","樟湖","樟湖生態國民中小學","樹中","樹人國小","樹人國民小學","樹人醫護管理專科學校","樹德家商","樹德技術學院","樹德科技大學","樹德高級家事商業職業學校","樹林國小","樹林國民小學","樹林幼兒園","樹林托兒所","樹林高級中學","樹科大","樹義國小","樹義國民小學","橋頭國中","橋頭國小","橋頭國民中學","橋頭國民小學","橫山國中","橫山國小","橫山國民中學","橫山國民小學","橫山鄉立托兒所","欽賢國中","欽賢國民中學","歡雅國小","歡雅國民小學","正修技術學院","正修科大","正修科技大學","正德國中","正德國民中學","正德高中","正德高級中學","正心","正心高級中學","正新國小","正新國民小學","正濱國中","正濱國小","正濱國民中學","正濱國民小學","正義國小","正義國民小學","正興國中","正興國小","正興國民中學","正興國民小學","武功國小","武功國民小學","武塔國小","武塔國民小學","武崙國中","武崙國小","武崙國民中學","武崙國民小學","武林國小","武林國民小學","武榮國小","武榮國民小學","武淵國小","武淵國民小學","武漢國中","武漢國小","武漢國民中學","武漢國民小學","武潭國小","武潭國民小學","武陵國小","武陵國民小學","武陵高中","武陵高級中等學校","歸仁國中","歸仁國小","歸仁國民中學","歸仁國民小學","歸仁幼兒園","歸仁托兒所","歸來國小","歸來國民小學","歸南國小","歸南國民小學","民和國中","民和國小","民和國民中學","民和國民小學","民安國小","民安國民小學","民富國小","民富國民小學","民德國中","民德國民中學","民族國中","民族國小","民族國民中學","民族國民小學","民權國中","民權國小","民權國民中學","民權國民小學","民生商職","民生國中","民生國小","民生國民中學","民生國民小學","民生家商","民生托兒所","民生高商","民生高級家事商業職業學校","民生高職","民義國小","民義國民小學","民雄國中","民雄國小","民雄國民中學","民雄國民小學","民雄農工","民雄鄉立托兒所","民雄高級農工職業學校","民靖國小","民靖國民小學","水上國中","水上國小","水上國民中學","水上國民小學","水上鄉立托兒所","水利國小","水利國民小學","水寮國小","水寮國民小學","水尾國小","水尾國民小學","水林國中","水林國民中學","水林鄉立托兒所","水泉國小","水泉國民小學","水源國小","水源國民小學","水燦林國小","水燦林國民小學","水璉國小","水璉國民小學","水碓國小","水碓國民小學","水美國小","水美國民小學","水里商工","水里國中","水里國小","水里國民中學","水里國民小學","水里鄉立托兒所","水里高級商工職業學校","永仁高中","永仁高級中學","永信國小","永信國民小學","永光國小","永光國民小學","永吉國中","永吉國小","永吉國民中學","永吉國民小學","永和國中","永和國小","永和國民中學","永和國民小學","永和幼兒園","永和托兒所","永安國中","永安國小","永安國民中學","永安國民小學","永定國小","永定國民小學","永寧國小","永寧國民小學","永平","永平國小","永平國民小學","永平高級中學","永年高中","永年高級中學","永康國中","永康國小","永康國民中學","永康國民小學","永康托兒所","永建國小","永建國民小學","永慶高中","永慶高級中學","永昌國小","永昌國民小學","永春國小","永春國民小學","永春高中","永春高級中學","永樂國小","永樂國民小學","永清國小","永清國民小學","永港國小","永港國民小學","永福國小","永福國民小學","永興國小","永興國民小學","永芳國小","永芳國民小學","永華國小","永華國民小學","永豐國小","永豐國民小學","永豐高中","永豐高級中等學校","永貞國小","永貞國民小學","永達技術學院","永隆國小","永隆國民小學","永靖國中","永靖國小","永靖國民中學","永靖國民小學","永靖鄉立托兒所","永靖高工","永靖高級工業職業學校","永順國小","永順國民小學","汐止國中","汐止國小","汐止國民中學","汐止國民小學","汐止托兒所","汕尾國小","汕尾國民小學","汝鎏國小","汝鎏國民小學","江翠國中","江翠國小","江翠國民中學","江翠國民小學","池上國中","池上國民中學","池上鄉立托兒所","池東國小","池東國民小學","汶水國小","汶水國民小學","沄水國小","沄水國民小學","沙坑國小","沙坑國民小學","沙崙國中","沙崙國小","沙崙國民中學","沙崙國民小學","沙港國小","沙港國民小學","沙鹿國中","沙鹿國小","沙鹿國民中學","沙鹿國民小學","沙鹿工業高級中等學校","沙鹿托兒所","沙鹿高工","河堤國小","河堤國民小學","河東國小","河東國民小學","河濱國小","河濱國民小學","油廠國小","油廠國民小學","油車國小","油車國民小學","泉州國小","泉州國民小學","泉源實小","泉源實驗國民小學","法務部司法官學院","法官學院","法治國小","法治國民小學","法鼓佛教研修學院","泰中","泰北高中","泰北高級中學","泰和國小","泰和國民小學","泰安中小學","泰安國小","泰安國民中小學","泰安國民小學","泰安鄉立托兒所","泰山國中","泰山國小","泰山國民中學","泰山國民小學","泰山幼兒園","泰山托兒所","泰山高級中學","泰武國中","泰武國小","泰武國民中學","泰武國民小學","泰武鄉立托兒所","泰源國中","泰源國小","泰源國民中學","泰源國民小學","泰興國小","泰興國民小學","洛津國小","洛津國民小學","海佃國中","海佃國小","海佃國民中學","海佃國民小學","海口國小","海口國民小學","海埔國小","海埔國民小學","海墘國小","海墘國民小學","海大","海寶國小","海寶國民小學","海山國小","海山國民小學","海山高中","海山高級中學","海星高中","海星高級中學","海東國小","海東國民小學","海洋大學","海湖國小","海湖國民小學","海濱國小","海濱國民小學","海科大","海端國中","海端國小","海端國民中學","海端國民小學","海端鄉立托兒所","海豐國小","海豐國民小學","海軍工程學院","海軍技術學校","海軍機械學校","海軍聯合參謀大學","海軍軍官學校","海軍陸戰隊學校","海軍電雷學校","海青工商","海青高級工商職業學校","淡大","淡水商工","淡水國中","淡水國小","淡水國民中學","淡水國民小學","淡水工商管理學院","淡水幼兒園","淡水托兒所","淡水高級商工職業學校","淡江大學","淡江文理學院","淡江英語專科學校","淡海國小","淡海國民小學","深坑國中","深坑國小","深坑國民中學","深坑國民小學","深坑幼兒園","深坑托兒所","深水國小","深水國民小學","深溝國小","深溝國民小學","深澳國小","深澳國民小學","深美國小","深美國民小學","淵明","淵明國民中學","清中","清境國小","清境國民小學","清大","清大附小","清安國小","清安國民小學","清水國中","清水國小","清水國民中學","清水國民小學","清水托兒所","清水高中","清水高級中學","清水高級中等學校","清江國小","清江國民小學","清泉國中","清泉國民中學","清海國中","清海國民中學","清溝國小","清溝國民小學","清華大學","清華大學附設實驗國民小學","清雲技術學院","清雲科大","清雲科大附設專科進修學校","清雲科大附設進修學院","清雲科技大學","清雲科技大學附設專科進修學校","清雲科技大學附設進修學院","渡拔國小","渡拔國民小學","港南國小","港南國民小學","港口國小","港口國民小學","港和國小","港和國民小學","港坪國小","港坪國民小學","港埔國小","港埔國民小學","港墘國小","港墘國民小學","港尾國小","港尾國民小學","港東國小","港東國民小學","港源國小","港源國民小學","港西國小","港西國民小學","湖內國中","湖內國民中學","湖內幼兒園","湖北國小","湖北國民小學","湖南國小","湖南國民小學","湖口國小","湖口國民小學","湖口鄉立托兒所","湖口高中","湖口高級中學","湖埔國小","湖埔國民小學","湖山國小","湖山國民小學","湖東國小","湖東國民小學","湖田國小","湖田國民小學","湖西國中","湖西國小","湖西國民中學","湖西國民小學","湖西鄉立托兒所","湳雅國小","湳雅國民小學","源城國小","源城國民小學","源泉國小","源泉國民小學","溝坪國小","溝坪國民小學","溝壩國小","溝壩國民小學","溪北國小","溪北國民小學","溪南國中","溪南國小","溪南國民中學","溪南國民小學","溪口國中","溪口國小","溪口國民中學","溪口國民小學","溪口鄉立托兒所","溪埔國中","溪埔國小","溪埔國民中學","溪埔國民小學","溪寮國小","溪寮國民小學","溪尾國小","溪尾國民小學","溪山國小","溪山國民小學","溪崑國中","溪崑國民中學","溪州國中","溪州國小","溪州國民中學","溪州國民小學","溪州鄉立托兒所","溪洲國小","溪洲國民小學","溪海國小","溪海國民小學","溪湖國中","溪湖國小","溪湖國民中學","溪湖國民小學","溪湖鎮立托兒所","溪湖高中","溪湖高級中學","溪陽國中","溪陽國民中學","溫泉國小","溫泉國民小學","滬江高中","滬江高級中學","滿州國中","滿州國小","滿州國民中學","滿州國民小學","滿州鄉立托兒所","漚汪國小","漚汪國民小學","漢口國中","漢口國民中學","漢寶國小","漢寶國民小學","漢民國小","漢民國民小學","漳和","漳和國小","漳和國民中學","漳和國民小學","漳興國小","漳興國民小學","潛龍國小","潛龍國民小學","潭南國小","潭南國民小學","潭墘國小","潭墘國民小學","潭子國中","潭子國小","潭子國民中學","潭子國民小學","潭子幼兒園","潭子托兒所","潭秀國中","潭秀國民中學","潭美國小","潭美國民小學","潭陽國小","潭陽國民小學","潮南國小","潮南國民小學","潮厝華德福教育實小","潮厝華德福教育實驗國民小學","潮和國小","潮和國民小學","潮寮國中","潮寮國小","潮寮國民中學","潮寮國民小學","潮州國中","潮州國小","潮州國民中學","潮州國民小學","潮州鎮立托兒所","潮州高中","潮州高級中學","潮昇國小","潮昇國民小學","潮東國小","潮東國民小學","潮洋國小","潮洋國民小學","潮音國小","潮音國民小學","澎南國中","澎南國民中學","澎湖技術學院","澎湖海事","澎湖科技大學","澎湖縣七美鄉七美國民小學","澎湖縣七美鄉立托兒所","澎湖縣七美鄉雙湖國民小學","澎湖縣望安鄉將軍國民小學","澎湖縣望安鄉望安國民小學","澎湖縣望安鄉立托兒所","澎湖縣望安鄉花嶼國民小學","澎湖縣湖西鄉成功國民小學","澎湖縣湖西鄉沙港國民小學","澎湖縣湖西鄉湖西國民小學","澎湖縣湖西鄉立托兒所","澎湖縣湖西鄉西溪國民小學","澎湖縣湖西鄉隘門國民小學","澎湖縣湖西鄉龍門國民小學","澎湖縣白沙鄉中屯國民小學","澎湖縣白沙鄉吉貝國民小學","澎湖縣白沙鄉後寮國民小學","澎湖縣白沙鄉立托兒所","澎湖縣白沙鄉講美國民小學","澎湖縣白沙鄉赤崁國民小學","澎湖縣白沙鄉鳥嶼國民小學","澎湖縣立七美國民中學","澎湖縣立中正國民中學","澎湖縣立吉貝國民中學","澎湖縣立將澳國民中學","澎湖縣立志清國民中學","澎湖縣立文光國民中學","澎湖縣立望安國民中學","澎湖縣立湖西國民中學","澎湖縣立澎南國民中學","澎湖縣立白沙國民中學","澎湖縣立西嶼國民中學","澎湖縣立鎮海國民中學","澎湖縣立馬公國民中學","澎湖縣立鳥嶼國民中學","澎湖縣西嶼鄉內垵國民小學","澎湖縣西嶼鄉合橫國民小學","澎湖縣西嶼鄉外垵國民小學","澎湖縣西嶼鄉大池國民小學","澎湖縣西嶼鄉池東國民小學","澎湖縣西嶼鄉立托兒所","澎湖縣西嶼鄉竹灣國民小學","澎湖縣馬公市中山國民小學","澎湖縣馬公市中正國民小學","澎湖縣馬公市中興國民小學","澎湖縣馬公市五德國民小學","澎湖縣馬公市山水國民小學","澎湖縣馬公市嵵裡國民小學","澎湖縣馬公市文光國民小學","澎湖縣馬公市文澳國民小學","澎湖縣馬公市東衛國民小學","澎湖縣馬公市石泉國民小學","澎湖縣馬公市立托兒所","澎湖縣馬公市興仁國民小學","澎湖縣馬公市虎井國民小學","澎湖縣馬公市風櫃國民小學","澎湖縣馬公市馬公國民小學","澎湖高級海事水產職業學校","澎科","澎科大","澳底國小","澳底國民小學","澳花國小","澳花國民小學","濂洞國小","濂洞國民小學","濱江國中","濱江國小","濱江國民中學","濱江國民小學","瀛海","瀛海高級中學","灣內國小","灣內國民小學","灣潭國小","灣潭國民小學","炎峰國小","炎峰國民小學","烈嶼國中","烈嶼國民中學","烏來中小學","烏來國民中小學","烏來幼兒園","烏來托兒所","烏日國中","烏日國小","烏日國民中學","烏日國民小學","烏日托兒所","烏林國小","烏林國民小學","烏眉國中","烏眉國小","烏眉國民中學","烏眉國民小學","烏龍國小","烏龍國民小學","照南國中","照南國小","照南國民中學","照南國民小學","照東國小","照東國民小學","照門國中","照門國小","照門國民中學","照門國民小學","燕中","燕巢國小","燕巢國民中學","燕巢國民小學","營北國中","營北國民中學","營盤國小","營盤國民小學","爽文國中","爽文國小","爽文國民中學","爽文國民小學","牡丹國中","牡丹國小","牡丹國民中學","牡丹國民小學","牡丹鄉立托兒所","猴硐國小","猴硐國民小學","獅子國中","獅子國民中學","獅子鄉立托兒所","獅湖國小","獅湖國民小學","獅潭國中","獅潭國小","獅潭國民中學","獅潭國民小學","獅甲國中","獅甲國小","獅甲國民中學","獅甲國民小學","獸醫學校","玄奘人文社會學院","玉井國中","玉井國小","玉井國民中學","玉井國民小學","玉井工商","玉井高級工商職業學校","玉光國小","玉光國民小學","玉山國中","玉山國小","玉山國民中學","玉山國民小學","玉峰國小","玉峰國民小學","玉成國小","玉成國民小學","玉東國中","玉東國民中學","玉田國小","玉田國民小學","玉豐國小","玉豐國民小學","玉里國中","玉里國小","玉里國民中學","玉里國民小學","玉里高中","玉里高級中學","王公國小","王公國民小學","王功國小","王功國民小學","琉球國中","琉球國小","琉球國民中學","琉球國民小學","琉球鄉立托兒所","瑞井國小","瑞井國民小學","瑞亭國小","瑞亭國民小學","瑞光國小","瑞光國民小學","瑞北國小","瑞北國民小學","瑞原國中","瑞原國小","瑞原國民中學","瑞原國民小學","瑞坪國中","瑞坪國民中學","瑞城國小","瑞城國民小學","瑞埔國小","瑞埔國民小學","瑞塘國小","瑞塘國民小學","瑞峰國中","瑞峰國小","瑞峰國民中學","瑞峰國民小學","瑞平國小","瑞平國民小學","瑞柑國小","瑞柑國民小學","瑞梅國小","瑞梅國民小學","瑞源國中","瑞源國小","瑞源國民中學","瑞源國民小學","瑞濱國小","瑞濱國民小學","瑞田國小","瑞田國民小學","瑞祥國小","瑞祥國民小學","瑞祥高中","瑞祥高級中學","瑞穗國中","瑞穗國小","瑞穗國民中學","瑞穗國民小學","瑞穗鄉立托兒所","瑞竹國中","瑞竹國小","瑞竹國民中學","瑞竹國民小學","瑞美國小","瑞美國民小學","瑞興國小","瑞興國民小學","瑞芳國中","瑞芳國小","瑞芳國民中學","瑞芳國民小學","瑞芳幼兒園","瑞芳托兒所","瑞芳高工","瑞芳高級工業職業學校","瑞豐國中","瑞豐國小","瑞豐國民中學","瑞豐國民小學","瑞里國小","瑞里國民小學","瑠公國中","瑠公國民中學","瑪家國中","瑪家國民中學","瑪家鄉立托兒所","瑪陵國小","瑪陵國民小學","環球技術學院","環球科技大學","環科","環科大","瓜山國小","瓜山國民小學","瓦磘國小","瓦磘國民小學","田中國小","田中國民小學","田中鎮立托兒所","田中高中","田中高級中學","田子國小","田子國民小學","田寮國中","田寮國小","田寮國民中學","田寮國民小學","田尾國中","田尾國小","田尾國民中學","田尾國民小學","田尾鄉立托兒所","田心國小","田心國民小學","田美國小","田美國民小學","田豐國小","田豐國民小學","田頭國小","田頭國民小學","甲中國小","甲中國民小學","甲仙國中","甲仙國小","甲仙國民中學","甲仙國民小學","甲南國小","甲南國民小學","甲圍國小","甲圍國民小學","番路鄉立托兒所","登發國小","登發國民小學","發祥國小","發祥國民小學","白冷國小","白冷國民小學","白沙國中","白沙國小","白沙國民中學","白沙國民小學","白沙鄉立托兒所","白河商工","白河國中","白河國小","白河國民中學","白河國民小學","白河托兒所","白河高級商工職業學校","白雲國小","白雲國民小學","百吉國小","百吉國民小學","百福國中","百福國民中學","百齡國小","百齡國民小學","百齡高中","百齡高級中學","益新工商","益新高級工商職業學校","益民國小","益民國民小學","直潭國小","直潭國民小學","省三國小","省三國民小學","省躬國小","省躬國民小學","真理大學","知本國中","知本國小","知本國民中學","知本國民小學","石光國中","石光國小","石光國民中學","石光國民小學","石城國小","石城國民小學","石岡國中","石岡國小","石岡國民中學","石岡國民小學","石岡托兒所","石榴國中","石榴國小","石榴國民中學","石榴國民小學","石泉國小","石泉國民小學","石牌國中","石牌國小","石牌國民中學","石牌國民小學","石碇國小","石碇國民小學","石碇幼兒園","石碇托兒所","石碇高中","石碇高級中學","石磊國小","石磊國民小學","石角國小","石角國民小學","石門國中","石門國小","石門國民中學","石門國民小學","石門幼兒園","石門托兒所","石龜國小","石龜國民小學","砂崙國小","砂崙國民小學","碇內國中","碇內國小","碇內國民中學","碇內國民小學","碧候國小","碧候國民小學","碧峰國小","碧峰國民小學","碧湖國小","碧湖國民小學","碧潭國小","碧潭國民小學","碧華國中","碧華國小","碧華國民中學","碧華國民小學","磐石高中","磐石高級中學","礁溪國中","礁溪國小","礁溪國民中學","礁溪國民小學","礁溪鄉立托兒所","社口國小","社口國民小學","社團國小","社團國民小學","社子國小","社子國民小學","社寮國中","社寮國小","社寮國民中學","社寮國民小學","社皮國小","社皮國民小學","社頭國中","社頭國小","社頭國民中學","社頭國民小學","社頭鄉立托兒所","祐德高中","祐德高級中學","神圳國中","神圳國民中學","神岡國中","神岡國小","神岡國民中學","神岡國民小學","神岡托兒所","神州補校","神州高中","神州高級中學進修學校","祥和國小","祥和國民小學","祥安國小","祥安國民小學","福原國小","福原國民小學","福和國中","福和國民中學","福基國小","福基國民小學","福安國中","福安國小","福安國民中學","福安國民小學","福山國中","福山國小","福山國民中學","福山國民小學","福康國小","福康國民小學","福德國小","福德國民小學","福星國小","福星國民小學","福智高中","福智高級中學","福東國小","福東國民小學","福林國小","福林國民小學","福樂國小","福樂國民小學","福民國小","福民國民小學","福源國小","福源國民小學","福營國中","福營國民中學","福科國中","福科國民中學","福興國中","福興國小","福興國民中學","福興國民小學","福興武術中小學","福興武術國民中小學","福興鄉立托兒所","福誠國小","福誠國民小學","福誠高中","福誠高級中學","福豐國中","福豐國民中學","福連國小","福連國民小學","福陽國小","福陽國民小學","福隆國小","福隆國民小學","福龍國小","福龍國民小學","福龜國小","福龜國民小學","秀中","秀山國小","秀山國民小學","秀峰國小","秀峰國民小學","秀峰高級中學","秀巒國小","秀巒國民小學","秀工","秀朗國小","秀朗國民小學","秀林國中","秀林國小","秀林國民中學","秀林國民小學","秀林鄉立托兒所","秀水國中","秀水國小","秀水國民中學","秀水國民小學","秀水鄉立托兒所","秀水高級工業職業學校","秀潭國小","秀潭國民小學","私立世界新聞傳播學院","私立中原大學","私立中原理工學院","私立中台醫護技術學院","私立中國文化大學","私立中國文化學院","私立中國醫藥學院","私立中山牙醫專科學校","私立中山醫學專科學校","私立中山醫學院","私立中山醫學院專科學校","私立中臺醫護技術學院","私立中華工學院","私立元智工學院","私立南台技術學院","私立南榮工商專科學校","私立南臺技術學院","私立南華管理學院","私立台北醫學院","私立嘉南藥理學院","私立大同工學院","私立大同工業專科學校","私立大華技術學院","私立大葉工學院","私立實踐設計管理學院","私立崑山技術學院","私立慈惠醫護管理專科學校","私立慈濟醫學暨人文社會學院","私立慈濟醫學院","私立明新技術學院","私立景文技術學院","私立朝陽技術學院","私立東吳大學","私立東海大學附屬高級中學","私立樹人醫護管理專科學校","私立樹德技術學院","私立淡水工商管理學院","私立淡江大學","私立淡江文理學院","私立淡江英語專科學校","私立玄奘人文社會學院","私立臺北醫學院","私立華梵人文科技學院","私立親民工商專科學校","私立輔仁大學","私立逢甲學院","私立銘傳管理學院","私立長庚醫學院暨工程學院","私立長榮管理學院","私立靜宜大學","私立靜宜女子文理學院","私立靜宜女子英語專科學校","私立高苑技術學院","私立高雄工學院","私立高雄醫學院","私立高鳳技術學院","科園國小","科園國民小學","科園實中","科學工業園區實驗高級中學","稻江","稻江商職","稻江科技暨管理學院","稻江護家","稻江高級商業職業學校","稻江高級護理家事職業學校","稻香國小","稻香國民小學","積中","積穗國小","積穗國民中學","積穗國民小學","空中大學","空大","空軍聯合參謀大學","空軍航空技術學院","空軍軍官學校","空軍防空學校","立人國中","立人國小","立人國民中學","立人小學","立人高中","立人高級中學","立仁高中","立仁高級中學","立德國中","立德國民中學","立德大學","立德管理學院","立志高中","立志高級中學","立新國中","立新國小","立新國民中學","立農國小","竹中","竹中國小","竹中國民小學","竹仁國小","竹仁國民小學","竹光國中","竹光國民中學","竹北國中","竹北國小","竹北國民中學","竹北國民小學","竹北市立托兒所","竹北高中","竹北高級中學","竹南國中","竹南國小","竹南國民中學","竹南國民小學","竹南鎮立托兒所","竹南高中","竹南高級中學","竹圍國中","竹圍國小","竹圍國民中學","竹圍國民小學","竹圍高中","竹圍高級中學","竹園國小","竹園國民小學","竹埔國小","竹埔國民小學","竹塘國中","竹塘國小","竹塘國民中學","竹塘國民小學","竹塘鄉立托兒所","竹女","竹安國小","竹安國民小學","竹山國中","竹山國小","竹山國民中學","竹山國民小學","竹山鎮立托兒所","竹山高中","竹山高級中學","竹崎國小","竹崎國民小學","竹崎鄉立托兒所","竹崎高中","竹崎高級中學","竹師","竹後國小","竹後國民小學","竹教","竹村國小","竹村國民小學","竹東國中","竹東國小","竹東國民中學","竹東國民小學","竹東幼兒園","竹東幼稚園","竹東鎮立托兒所","竹東高中","竹東高級中學","竹林國小","竹林國民小學","竹橋國中","竹橋國小","竹橋國民中學","竹橋國民小學","竹湖國小","竹湖國民小學","竹滬國小","竹滬國民小學","竹灣國小","竹灣國民小學","竹特","竹田國中","竹田國小","竹田國民中學","竹田國民小學","竹田鄉立托兒所","竹科實中","竹興國小","竹興國民小學","竹蓮國小","竹蓮國民小學","竹門國小","竹門國民小學","笨港國小","笨港國民小學","第一女子高級中學","第三幼稚園","第五幼稚園","第四幼稚園","管嶼國小","管嶼國民小學","篤加國小","篤加國民小學","篤行國小","篤行國民小學","米倉國小","米倉國民小學","精忠國小","精忠國民小學","精華國中","精華國民中學","精誠高中","精誠高級中學","紀安國小","紀安國民小學","紅毛港國小","紅毛港國民小學","紅瓦厝國小","紅瓦厝國民小學","紅葉國小","紅葉國民小學","經國國中","經國國民中學","經國學院","經國管理暨健康學院","綠島國中","綠島國小","綠島國民中學","綠島國民小學","綠島鄉立托兒所","維多利亞國小","維多利亞國民小學","維新國小","維新國民小學","維真國中","維真國民中學","網寮國小","網寮國民小學","網溪國小","網溪國民小學","線西國中","線西國小","線西國民中學","線西國民小學","線西鄉立托兒所","繁華國小","繁華國民小學","羅厝國小","羅厝國民小學","羅娜國小","羅娜國民小學","羅東國中","羅東國小","羅東國民中學","羅東國民小學","羅東鎮立托兒所","羅東高商","羅東高工","羅東高級中學","羅東高級商業職業學校","羅東高級工業職業學校","羅浮國小","羅浮國民小學","羅高","羌園國小","羌園國民小學","美和國小","美和國民小學","美和技術學院","美和科大","美和科技大學","美和高中","美和高級中學","美國學校","美崙國中","美崙國民中學","美林國小","美林國民小學","美濃國中","美濃國小","美濃國民中學","美濃國民小學","美群國小","美群國民小學","美華國小","美華國民小學","美豐國小","美豐國民小學","義仁國小","義仁國民小學","義學國中","義學國小","義學國民中學","義學國民小學","義守","義守大學","義峰高中","義峰高級中學","義方國小","義方國民小學","義民高中","義民高級中學","義盛國小","義盛國民小學","義竹國中","義竹國小","義竹國民中學","義竹國民小學","義興國小","義興國民小學","義賢國小","義賢國民小學","翁園國小","翁園國民小學","翁子國小","翁子國民小學","翠屏","翠屏國民中小學","老松國小","老松國民小學","老梅國小","老梅國民小學","耕莘健康管理專科學校","耕莘專校","聖功","聖功女中","聖功女子高中","聖功女子高級中學","聖功女高","聖心高中","聖心高級中學","聖母護專","聖母醫護管理專科學校","聖約翰","聖約翰技術學院","聖約翰科技大學","聖賢國小","聖賢國民小學","聯勤測量學校","聯勤財務學校","聯合大學","聯合技術學院","聯大","聯美國小","聯美國民小學","聯興國小","聯興國民小學","職能發展學院","育人國小","育人國民小學","育仁國小","育仁國民小學","育仁高中","育仁高級中學","育德國小","育德國民小學","育成高中","育成高級中學","育才國小","育才國民小學","育新國小","育新國民小學","育林","育林國小","育林國民中學","育林國民小學","育樂國小","育樂國民小學","育民國小","育民國民小學","育民工家","育民高級工業家事職業學校","育航幼兒園","育航幼稚園","育英國中","育英國小","育英國民中學","育英國民小學","育英醫專","育英醫護管理專科學校","育華國小","育華國民小學","育賢國中","育賢國民中學","育達商業技術學院","育達商業科技大學","育達商職","育達家商","育達高商","育達高級商業家事職業學校","育達高職","胡適國小","胡適國民小學","自強國中","自強國小","自強國民中學","自強國民小學","自強托兒所","自由國小","自由國民小學","自立國小","自立國民小學","至善國中","至善國民中學","至正國中","至正國民中學","至誠國小","至誠國民小學","致民國中","致民國民中學","致理技術學院","致遠管理學院","臺中一中","臺中二中","臺中健康暨管理學院","臺中國小","臺中國民小學","臺中女中","臺中女子高級中等學校","臺中女高","臺中家事商業高級中等學校","臺中家商","臺中實小","臺中工業高級中等學校","臺中市中區光復國民小學","臺中市北區中華國民小學","臺中市北區健行國民小學","臺中市北區太平國民小學","臺中市北區省三國民小學","臺中市北區立人國民小學","臺中市北區篤行國民小學","臺中市北區賴厝國民小學","臺中市北屯區仁愛國民小學","臺中市北屯區仁美國民小學","臺中市北屯區僑孝國民小學","臺中市北屯區北屯國民小學","臺中市北屯區四張犁國民小學","臺中市北屯區四維國民小學","臺中市北屯區大坑國民小學","臺中市北屯區廍子國民小學","臺中市北屯區建功國民小學","臺中市北屯區文心國民小學","臺中市北屯區文昌國民小學","臺中市北屯區新興國民小學","臺中市北屯區東光國民小學","臺中市北屯區松竹國民小學","臺中市北屯區軍功國民小學","臺中市北屯區逢甲國民小學","臺中市北屯區陳平國民小學","臺中市南區信義國民小學","臺中市南區和平國民小學","臺中市南區國光國民小學","臺中市南區樹義國民小學","臺中市南屯區南屯國民小學","臺中市南屯區大墩國民小學","臺中市南屯區大新國民小學","臺中市南屯區惠文國民小學","臺中市南屯區文山國民小學","臺中市南屯區春安國民小學","臺中市南屯區東興國民小學","臺中市南屯區永春國民小學","臺中市南屯區鎮平國民小學","臺中市南屯區黎明國民小學","臺中市博屋瑪國民小學","臺中市后里區七星國民小學","臺中市后里區內埔國民小學","臺中市后里區后里國民小學","臺中市后里區月眉國民小學","臺中市后里區泰安國民小學","臺中市后里區育英國民小學","臺中市和平區中坑國民小學","臺中市和平區博愛國民小學","臺中市和平區和平國民小學","臺中市和平區平等國民小學","臺中市和平區白冷國民小學","臺中市和平區自由國民小學","臺中市外埔區外埔國民小學","臺中市外埔區安定國民小學","臺中市外埔區水美國民小學","臺中市外埔區鐵山國民小學","臺中市外埔區馬鳴國民小學","臺中市大安區三光國民小學","臺中市大安區大安國民小學","臺中市大安區永安國民小學","臺中市大安區海墘國民小學","臺中市大甲區大甲國民小學","臺中市大甲區德化國民小學","臺中市大甲區文昌國民小學","臺中市大甲區文武國民小學","臺中市大甲區日南國民小學","臺中市大甲區東明國民小學","臺中市大甲區東陽國民小學","臺中市大甲區華龍國民小學","臺中市大甲區西岐國民小學","臺中市大甲區順天國民小學","臺中市大肚區大忠國民小學","臺中市大肚區大肚國民小學","臺中市大肚區山陽國民小學","臺中市大肚區永順國民小學","臺中市大肚區瑞井國民小學","臺中市大肚區瑞峰國民小學","臺中市大肚區追分國民小學","臺中市大里區健民國民小學","臺中市大里區內新國民小學","臺中市大里區塗城國民小學","臺中市大里區大元國民小學","臺中市大里區大里國民小學","臺中市大里區崇光國民小學","臺中市大里區永隆國民小學","臺中市大里區瑞城國民小學","臺中市大里區益民國民小學","臺中市大里區立新國民小學","臺中市大里區美群國民小學","臺中市大里區草湖國民小學","臺中市大雅區三和國民小學","臺中市大雅區上楓國民小學","臺中市大雅區六寶國民小學","臺中市大雅區大明國民小學","臺中市大雅區大雅國民小學","臺中市大雅區文雅國民小學","臺中市大雅區汝鎏國民小學","臺中市大雅區陽明國民小學","臺中市太平區中華國民小學","臺中市太平區光隆國民小學","臺中市太平區坪林國民小學","臺中市太平區太平國民小學","臺中市太平區宜欣國民小學","臺中市太平區建平國民小學","臺中市太平區新光國民小學","臺中市太平區新平國民小學","臺中市太平區新高國民小學","臺中市太平區東平國民小學","臺中市太平區東汴國民小學","臺中市太平區車籠埔國民小學","臺中市太平區長億國民小學","臺中市太平區頭汴國民小學","臺中市太平區黃竹國民小學","臺中市新社區中和國民小學","臺中市新社區協成國民小學","臺中市新社區大南國民小學","臺中市新社區大林國民小學","臺中市新社區崑山國民小學","臺中市新社區新社國民小學","臺中市新社區東興國民小學","臺中市新社區福民國民小學","臺中市明德女子高級中學","臺中市東勢區中山國民小學","臺中市東勢區中科國民小學","臺中市東勢區成功國民小學","臺中市東勢區新成國民小學","臺中市東勢區新盛國民小學","臺中市東勢區明正國民小學","臺中市東勢區東勢國民小學","臺中市東勢區東新國民小學","臺中市東勢區石城國民小學","臺中市東勢區石角國民小學","臺中市東區力行國民小學","臺中市東區大智國民小學","臺中市東區成功國民小學","臺中市東區樂業國民小學","臺中市東區臺中國民小學","臺中市東區進德國民小學","臺中市梧棲區中正國民小學","臺中市梧棲區中港國民小學","臺中市梧棲區大德國民小學","臺中市梧棲區梧南國民小學","臺中市梧棲區梧棲國民小學","臺中市梧棲區永寧國民小學","臺中市沙鹿區公明國民小學","臺中市沙鹿區公館國民小學","臺中市沙鹿區北勢國民小學","臺中市沙鹿區文光國民小學","臺中市沙鹿區沙鹿國民小學","臺中市沙鹿區竹林國民小學","臺中市沙鹿區鹿峰國民小學","臺中市清水區三田國民小學","臺中市清水區吳厝國民小學","臺中市清水區大楊國民小學","臺中市清水區大秀國民小學","臺中市清水區建國國民小學","臺中市清水區東山國民小學","臺中市清水區槺榔國民小學","臺中市清水區清水國民小學","臺中市清水區甲南國民小學","臺中市清水區西寧國民小學","臺中市清水區高美國民小學","臺中市潭子區僑忠國民小學","臺中市潭子區新興國民小學","臺中市潭子區東寶國民小學","臺中市潭子區潭子國民小學","臺中市潭子區潭陽國民小學","臺中市潭子區頭家國民小學","臺中市烏日區九德國民小學","臺中市烏日區五光國民小學","臺中市烏日區僑仁國民小學","臺中市烏日區喀哩國民小學","臺中市烏日區旭光國民小學","臺中市烏日區東園國民小學","臺中市烏日區溪尾國民小學","臺中市烏日區烏日國民小學","臺中市石岡區土牛國民小學","臺中市石岡區石岡國民小學","臺中市神岡區圳堵國民小學","臺中市神岡區岸裡國民小學","臺中市神岡區社口國民小學","臺中市神岡區神岡國民小學","臺中市神岡區豐洲國民小學","臺中市私立光華高級工業職業學校","臺中市私立宜寧高級中學","臺中市私立嶺東高級中學","臺中市私立慎齋國民小學","臺中市私立新民高級中學","臺中市私立曉明女子高級中學","臺中市私立衛道高級中學","臺中市立三光國民中學","臺中市立中山國民中學","臺中市立中平國民中學","臺中市立中港高級中學","臺中市立五權國民中學","臺中市立光復國民中小學","臺中市立光德國民中學","臺中市立光明國民中學","臺中市立光榮國民中學","臺中市立光正國民中學","臺中市立公明國民中學","臺中市立北勢國民中學","臺中市立北新國民中學","臺中市立南屯幼兒園","臺中市立南屯托兒所","臺中市立后綜高級中學","臺中市立后里國民中學","臺中市立后里托兒所","臺中市立向上國民中學","臺中市立和平國民中學","臺中市立和平托兒所","臺中市立啟明學校","臺中市立啟聰學校","臺中市立善水國民中小學","臺中市立四張犁國民中學","臺中市立四箴國民中學","臺中市立四育國民中學","臺中市立外埔國民中學","臺中市立外埔托兒所","臺中市立大墩國民中學","臺中市立大安國民中學","臺中市立大安幼兒園","臺中市立大安托兒所","臺中市立大德國民中學","臺中市立大業國民中學","臺中市立大甲國民中學","臺中市立大甲工業高級中等學校","臺中市立大甲托兒所","臺中市立大甲高級中等學校","臺中市立大肚托兒所","臺中市立大華國民中學","臺中市立大道國民中學","臺中市立大里托兒所","臺中市立大里高級中學","臺中市立大雅國民中學","臺中市立大雅托兒所","臺中市立太平國民中學","臺中市立太平托兒所","臺中市立安和國民中學","臺中市立居仁國民中學","臺中市立崇倫國民中學","臺中市立崇德國民中學","臺中市立忠明高級中學","臺中市立惠文高級中學","臺中市立成功國民中學","臺中市立托兒所","臺中市立文華高級中等學校","臺中市立新光國民中學","臺中市立新社托兒所","臺中市立新社高級中學","臺中市立日南國民中學","臺中市立東勢國民中學","臺中市立東勢工業高級中等學校","臺中市立東勢托兒所","臺中市立東山高級中學","臺中市立東峰國民中學","臺中市立東新國民中學","臺中市立東華國民中學","臺中市立梧棲國民中學","臺中市立梧棲托兒所","臺中市立梨山國民中小學","臺中市立沙鹿國民中學","臺中市立沙鹿工業高級中等學校","臺中市立沙鹿托兒所","臺中市立清水國民中學","臺中市立清水托兒所","臺中市立清水高級中等學校","臺中市立清泉國民中學","臺中市立清海國民中學","臺中市立溪南國民中學","臺中市立漢口國民中學","臺中市立潭子國民中學","臺中市立潭子幼兒園","臺中市立潭子托兒所","臺中市立潭秀國民中學","臺中市立烏日國民中學","臺中市立烏日托兒所","臺中市立爽文國民中學","臺中市立石岡國民中學","臺中市立石岡托兒所","臺中市立神圳國民中學","臺中市立神岡國民中學","臺中市立神岡托兒所","臺中市立福科國民中學","臺中市立立人國民中學","臺中市立立新國民中學","臺中市立育英國民中學","臺中市立至善國民中學","臺中市立臺中女子高級中等學校","臺中市立臺中家事商業高級中等學校","臺中市立臺中工業高級中等學校","臺中市立臺中特殊教育學校","臺中市立臺中第一高級中等學校","臺中市立臺中第二高級中等學校","臺中市立萬和國民中學","臺中市立西苑高級中學","臺中市立豐南國民中學","臺中市立豐原商業高級中等學校","臺中市立豐原國民中學","臺中市立豐原托兒所","臺中市立豐原高級中等學校","臺中市立豐東國民中學","臺中市立豐陽國民中學","臺中市立長億高級中學","臺中市立雙十國民中學","臺中市立霧峰國民中學","臺中市立霧峰幼兒園","臺中市立霧峰托兒所","臺中市立霧峰農業工業高級中等學校","臺中市立順天國民中學","臺中市立鹿寮國民中學","臺中市立黎明國民中學","臺中市立龍井國民中學","臺中市立龍井托兒所","臺中市立龍津高級中等學校","臺中市西區中正國民小學","臺中市西區大勇國民小學","臺中市西區大同國民小學","臺中市西區忠信國民小學","臺中市西區忠孝國民小學","臺中市西區忠明國民小學","臺中市西屯區上安國民小學","臺中市西屯區上石國民小學","臺中市西屯區何厝國民小學","臺中市西屯區協和國民小學","臺中市西屯區國安國民小學","臺中市西屯區大仁國民小學","臺中市西屯區大鵬國民小學","臺中市西屯區惠來國民小學","臺中市西屯區東海國民小學","臺中市西屯區永安國民小學","臺中市西屯區泰安國民小學","臺中市西屯區西屯國民小學","臺中市西屯區重慶國民小學","臺中市西屯區長安國民小學","臺中市豐原區南陽國民小學","臺中市豐原區合作國民小學","臺中市豐原區富春國民小學","臺中市豐原區瑞穗國民小學","臺中市豐原區福陽國民小學","臺中市豐原區翁子國民小學","臺中市豐原區葫蘆墩國民小學","臺中市豐原區豐原國民小學","臺中市豐原區豐村國民小學","臺中市豐原區豐田國民小學","臺中市霧峰區五福國民小學","臺中市霧峰區僑榮國民小學","臺中市霧峰區光正國民小學","臺中市霧峰區吉峰國民小學","臺中市霧峰區四德國民小學","臺中市霧峰區峰谷國民小學","臺中市霧峰區復興國民小學","臺中市霧峰區桐林國民小學","臺中市霧峰區萬豐國民小學","臺中市霧峰區霧峰國民小學","臺中市龍井區龍井國民小學","臺中市龍井區龍山國民小學","臺中市龍井區龍峰國民小學","臺中市龍井區龍泉國民小學","臺中市龍井區龍津國民小學","臺中市龍井區龍海國民小學","臺中市龍井區龍港國民小學","臺中師範學院","臺中技術學院","臺中技術學院附設空中進修學院","臺中技術學院附設進修學院","臺中教育大學","臺中教育大學附設實驗國民小學","臺中特殊教育學校","臺中科技大學","臺中第一高級中等學校","臺中第二高級中等學校","臺中高工","臺北南海實驗幼稚園","臺北商業大學","臺北商業技術學院","臺北商業技術學院附設空中進修學院","臺北城市科技大學","臺北大學","臺北市中山區中山國民小學","臺北市中山區中正國民小學","臺北市中山區五常國民小學","臺北市中山區吉林國民小學","臺北市中山區大佳國民小學","臺北市中山區大直國民小學","臺北市中山區懷生國民小學","臺北市中山區永安國民小學","臺北市中山區濱江國民小學","臺北市中山區長安國民小學","臺北市中山區長春國民小學","臺北市中山社區大學","臺北市中正區南門國民小學","臺北市中正區忠孝國民小學","臺北市中正區忠義國民小學","臺北市中正區東門國民小學","臺北市中正區河堤國民小學","臺北市中正區螢橋國民小學","臺北市中正社區大學","臺北市信義區三興國民小學","臺北市信義區信義國民小學","臺北市信義區光復國民小學","臺北市信義區博愛國民小學","臺北市信義區吳興國民小學","臺北市信義區永吉國民小學","臺北市信義區永春國民小學","臺北市信義區福德國民小學","臺北市信義區興雅國民小學","臺北市信義社區大學","臺北市內湖區內湖國民小學","臺北市內湖區南湖國民小學","臺北市內湖區大湖國民小學","臺北市內湖區康寧國民小學","臺北市內湖區文湖國民小學","臺北市內湖區新湖國民小學","臺北市內湖區明湖國民小學","臺北市內湖區東湖國民小學","臺北市內湖區潭美國民小學","臺北市內湖區碧湖國民小學","臺北市內湖區西湖國民小學","臺北市內湖區麗山國民小學","臺北市內湖區麗湖國民小學","臺北市內湖社區大學","臺北市北投區北投國民小學","臺北市北投區大屯國民小學","臺北市北投區文化國民小學","臺北市北投區文林國民小學","臺北市北投區明德國民小學","臺北市北投區桃源國民小學","臺北市北投區清江國民小學","臺北市北投區湖山國民小學","臺北市北投區湖田國民小學","臺北市北投區石牌國民小學","臺北市北投區立農國民小學","臺北市北投區義方國民小學","臺北市北投區逸仙國民小學","臺北市北投區關渡國民小學","臺北市北投社區大學","臺北市南港區修德國民小學","臺北市南港區南港國民小學","臺北市南港區成德國民小學","臺北市南港區東新國民小學","臺北市南港區玉成國民小學","臺北市南港區胡適國民小學","臺北市南港區舊莊國民小學","臺北市南港社區大學","臺北市博嘉實驗國民小學","臺北市和平實驗國民小學","臺北市國語實驗國民小學","臺北市士林區三玉國民小學","臺北市士林區劍潭國民小學","臺北市士林區士東國民小學","臺北市士林區士林國民小學","臺北市士林區天母國民小學","臺北市士林區富安國民小學","臺北市士林區平等國民小學","臺北市士林區文昌國民小學","臺北市士林區溪山國民小學","臺北市士林區百齡國民小學","臺北市士林區社子國民小學","臺北市士林區福林國民小學","臺北市士林區芝山國民小學","臺北市士林區葫蘆國民小學","臺北市士林區蘭雅國民小學","臺北市士林區陽明山國民小學","臺北市士林區雙溪國民小學","臺北市士林區雨聲國民小學","臺北市士林區雨農國民小學","臺北市士林社區大學","臺北市大同區大同國民小學","臺北市大同區大橋國民小學","臺北市大同區大龍國民小學","臺北市大同區太平國民小學","臺北市大同區延平國民小學","臺北市大同區日新國民小學","臺北市大同區永樂國民小學","臺北市大同區蓬萊國民小學","臺北市大同區雙蓮國民小學","臺北市大同社區大學","臺北市大安區仁愛國民小學","臺北市大安區公館國民小學","臺北市大安區古亭國民小學","臺北市大安區大安國民小學","臺北市大安區幸安國民小學","臺北市大安區建安國民小學","臺北市大安區新生國民小學","臺北市大安區金華國民小學","臺北市大安區銘傳國民小學","臺北市大安區龍安國民小學","臺北市大安社區大學","臺北市文山區力行國民小學","臺北市文山區實踐國民小學","臺北市文山區志清國民小學","臺北市文山區指南國民小學","臺北市文山區明道國民小學","臺北市文山區景美國民小學","臺北市文山區景興國民小學","臺北市文山區木柵國民小學","臺北市文山區武功國民小學","臺北市文山區永建國民小學","臺北市文山區溪口國民小學","臺北市文山區興德國民小學","臺北市文山區興華國民小學","臺北市文山區興隆國民小學","臺北市文山區萬福國民小學","臺北市文山區萬興國民小學","臺北市文山區萬芳國民小學","臺北市文山區辛亥國民小學","臺北市文山社區大學","臺北市松山區三民國民小學","臺北市松山區健康國民小學","臺北市松山區敦化國民小學","臺北市松山區松山國民小學","臺北市松山區民族國民小學","臺北市松山區民權國民小學","臺北市松山區民生國民小學","臺北市松山區西松國民小學","臺北市松山社區大學","臺北市泉源實驗國民小學","臺北市私立中山小學","臺北市私立中興高級中學","臺北市私立光仁小學","臺北市私立再興小學","臺北市私立再興高級中學","臺北市私立十信高級中學","臺北市私立協和高級工商職業學校","臺北市私立南華高級中學職業進修學校","臺北市私立喬治高級工商職業學校","臺北市私立大同高級中學","臺北市私立大誠高級中學","臺北市私立奎山國民中學","臺北市私立奎山小學","臺北市私立延平高級中學","臺北市私立強恕高級中學","臺北市私立復興實驗高級中學","臺北市私立志仁高級中學職業進修學校","臺北市私立志仁高級家事商業學校","臺北市私立恕德高級家事商業職業學校","臺北市私立惇敘高級工商職業學校","臺北市私立文德女子高級中學","臺北市私立新民小學","臺北市私立方濟高級中學","臺北市私立景文高級中學","臺北市私立東山高級中學","臺北市私立東方高級工商職業學校","臺北市私立泰北高級中學","臺北市私立滬江高級中學","臺北市私立祐德高級中學","臺北市私立稻江高級商業職業學校","臺北市私立稻江高級護理家事職業學校","臺北市私立立人國民中學","臺北市私立立人小學","臺北市私立立人高級中學","臺北市私立育達高級商業家事職業學校","臺北市私立華岡藝術學校","臺北市私立華興小學","臺北市私立華興高級中學","臺北市私立薇閣小學","臺北市私立薇閣高級中學","臺北市私立衛理女子高級中學","臺北市私立達人女子高級中學","臺北市私立金甌女子高級中學","臺北市私立開南高級商工職業學校","臺北市私立開平餐飲職業學校","臺北市私立靜修女子高級中學","臺北市私立靜心國民中學","臺北市私立靜心小學","臺北市立三民國民中學","臺北市立中山國民中學","臺北市立中山女子高級中學","臺北市立中山幼兒園","臺北市立中山托兒所","臺北市立中崙高級中學","臺北市立中正國民中學","臺北市立中正幼兒園","臺北市立中正托兒所","臺北市立中正高級中學","臺北市立五常國民中學","臺北市立仁愛國民中學","臺北市立介壽國民中學","臺北市立信義國民中學","臺北市立信義幼兒園","臺北市立信義托兒所","臺北市立內湖國民中學","臺北市立內湖幼兒園","臺北市立內湖托兒所","臺北市立內湖高級中學","臺北市立內湖高級工業職業學校","臺北市立北安國民中學","臺北市立北投國民中學","臺北市立北投幼兒園","臺北市立北投托兒所","臺北市立北政國民中學","臺北市立南海實驗幼兒園","臺北市立南港幼兒園","臺北市立南港托兒所","臺北市立南港高級中學","臺北市立南港高級工業職業學校","臺北市立南湖高級中學","臺北市立南門國民中學","臺北市立古亭國民中學","臺北市立古亭托兒所","臺北市立和平高級中學","臺北市立啟明學校","臺北市立啟智學校","臺北市立啟聰學校","臺北市立城中托兒所","臺北市立士林國民中學","臺北市立士林幼兒園","臺北市立士林托兒所","臺北市立士林高級商業職業學校","臺北市立大同幼兒園","臺北市立大同托兒所","臺北市立大同高級中學","臺北市立大學","臺北市立大學附設實驗國民小學","臺北市立大安國民中學","臺北市立大安幼兒園","臺北市立大安托兒所","臺北市立大安高工附設高級工業職業進修學校","臺北市立大安高級工業職業學校","臺北市立大理高級中學","臺北市立大直托兒所","臺北市立大直高級中學","臺北市立天母國民中學","臺北市立實踐國民中學","臺北市立師範學院","臺北市立建國高級中學","臺北市立建成國民中學","臺北市立建成托兒所","臺北市立弘道國民中學","臺北市立復興高級中學","臺北市立忠孝國民中學","臺北市立懷生國民中學","臺北市立成功托兒所","臺北市立成功高級中學","臺北市立成德國民中學","臺北市立成淵高級中學","臺北市立敦化國民中學","臺北市立文山幼兒園","臺北市立文山托兒所","臺北市立文山特殊教育學校","臺北市立新民國民中學","臺北市立新興國民中學","臺北市立明倫高級中學","臺北市立明德國民中學","臺北市立明湖國民中學","臺北市立景美國民中學","臺北市立景美女子高級中學","臺北市立景興國民中學","臺北市立木柵國民中學","臺北市立木柵托兒所","臺北市立木柵高級工業職業學校","臺北市立東湖國民中學","臺北市立松山幼兒園","臺北市立松山托兒所","臺北市立松山高級中學","臺北市立松山高級商業家事職業學校","臺北市立松山高級工農職業學校","臺北市立格致國民中學","臺北市立桃源國民中學","臺北市立民族國民中學","臺北市立民權國民中學","臺北市立民生國民中學","臺北市立民生托兒所","臺北市立永吉國民中學","臺北市立永春高級中學","臺北市立濱江國民中學","臺北市立瑠公國民中學","臺北市立百齡高級中學","臺北市立石牌國民中學","臺北市立福安國民中學","臺北市立第一女子高級中學","臺北市立育成高級中學","臺北市立育航幼兒園","臺北市立育航幼稚園","臺北市立自強托兒所","臺北市立至善國民中學","臺北市立興福國民中學","臺北市立興雅國民中學","臺北市立芳和實驗國民中學","臺北市立華江高級中學","臺北市立萬芳高級中學","臺北市立萬華國民中學","臺北市立萬華幼兒園","臺北市立萬華托兒所","臺北市立蘭州國民中學","臺北市立蘭雅國民中學","臺北市立螢橋國民中學","臺北市立西松高級中學","臺北市立西湖國民中學","臺北市立誠正國民中學","臺北市立重慶國民中學","臺北市立金華國民中學","臺北市立長安國民中學","臺北市立關渡國民中學","臺北市立陽明高級中學","臺北市立雙園區南海幼稚園","臺北市立雙園國民中學","臺北市立雙園托兒所","臺北市立體育學院","臺北市立麗山國民中學","臺北市立麗山高級中學","臺北市立龍山國民中學","臺北市立龍山托兒所","臺北市立龍門國民中學","臺北市職能發展學院","臺北市萬華區大理國民小學","臺北市萬華區新和國民小學","臺北市萬華區東園國民小學","臺北市萬華區福星國民小學","臺北市萬華區老松國民小學","臺北市萬華區華江國民小學","臺北市萬華區萬大國民小學","臺北市萬華區西園國民小學","臺北市萬華區西門國民小學","臺北市萬華區雙園國民小學","臺北市萬華區龍山國民小學","臺北市萬華社區大學","臺北師範學院","臺北技術學院","臺北教育大學","臺北教育大學附設實驗國民小學","臺北科大附設進修學院","臺北科技大學","臺北科技大學附屬桃園農工高級中等學校","臺北科技大學附設進修學院","臺北藝術大學","臺北護理健康大學","臺北護理學院","臺北農林專門學校","臺北醫學大學","臺北醫學院","臺北高等農林學校","臺南啟智學校","臺南大學","臺南大學附屬啟聰學校","臺南大學附屬高級中學","臺南大學附設實驗國民小學","臺南女中","臺南女子技術學院","臺南女子高級中學","臺南家齊高級中等學校","臺南市七股區七股國民小學","臺南市七股區三股國民小學","臺南市七股區光復生態實驗小學","臺南市七股區大文國民小學","臺南市七股區建功國民小學","臺南市七股區後港國民小學","臺南市七股區樹林國民小學","臺南市七股區竹橋國民小學","臺南市七股區篤加國民小學","臺南市七股區龍山國民小學","臺南市下營區下營國民小學","臺南市下營區中營國民小學","臺南市下營區東興國民小學","臺南市下營區甲中國民小學","臺南市下營區賀建國民小學","臺南市中區第一幼稚園","臺南市中區第二幼稚園","臺南市中西區協進國民小學","臺南市中西區忠義國民小學","臺南市中西區成功國民小學","臺南市中西區永福國民小學","臺南市中西區進學國民小學","臺南市亞洲高級餐旅職業學校","臺南市仁德區仁和國民小學","臺南市仁德區仁德國民小學","臺南市仁德區依仁國民小學","臺南市仁德區大甲國民小學","臺南市仁德區德南國民小學","臺南市仁德區文賢國民小學","臺南市仁德區虎山實驗小學","臺南市仁德區長興國民小學","臺南市佳里區仁愛國民小學","臺南市佳里區佳興國民小學","臺南市佳里區佳里國民小學","臺南市佳里區信義國民小學","臺南市佳里區塭內國民小學","臺南市佳里區子龍國民小學","臺南市佳里區延平國民小學","臺南市佳里區通興國民小學","臺南市六信高級中學","臺南市六甲區六甲國民小學","臺南市六甲區林鳳國民小學","臺南市北區公園國民小學","臺南市北區大光國民小學","臺南市北區大港國民小學","臺南市北區文元國民小學","臺南市北區立人國民小學","臺南市北區第三幼稚園","臺南市北區第四幼稚園","臺南市北區賢北國民小學","臺南市北區開元國民小學","臺南市北門區三慈國民小學","臺南市北門區北門國民小學","臺南市北門區文山國民小學","臺南市北門區蚵寮國民小學","臺南市北門區錦湖國民小學","臺南市北門區雙春國民小學","臺南市南化區北寮國民小學","臺南市南化區南化國民小學","臺南市南化區玉山國民小學","臺南市南化區瑞峰國民小學","臺南市南化區西埔國民小學","臺南市南區喜樹國民小學","臺南市南區志開實驗小學","臺南市南區新興國民小學","臺南市南區日新國民小學","臺南市南區永華國民小學","臺南市南區省躬國民小學","臺南市南區龍崗國民小學","臺南市善化區善化國民小學","臺南市善化區善糖國民小學","臺南市善化區大同國民小學","臺南市善化區大成國民小學","臺南市善化區小新國民小學","臺南市善化區茄拔國民小學","臺南市善化區陽明國民小學","臺南市大內區二溪國民小學","臺南市大內區大內國民小學","臺南市天主教私立寶仁國民小學","臺南市學甲區中洲國民小學","臺南市學甲區學甲國民小學","臺南市學甲區宅港國民小學","臺南市學甲區東陽國民小學","臺南市學甲區頂洲國民小學","臺南市安南區南興國民小學","臺南市安南區和順國民小學","臺南市安南區土城國民小學","臺南市安南區學東國民小學","臺南市安南區安佃國民小學","臺南市安南區安慶國民小學","臺南市安南區安順國民小學","臺南市安南區海佃國民小學","臺南市安南區海東國民小學","臺南市安南區鎮海國民小學","臺南市安南區長安國民小學","臺南市安南區青草國民小學","臺南市安南區顯宮國民小學","臺南市安定區南安國民小學","臺南市安定區南興國民小學","臺南市安定區安定國民小學","臺南市安平區億載國民小學","臺南市安平區安平國民小學","臺南市安平區新南國民小學","臺南市安平區石門國民小學","臺南市安平區西門實驗小學","臺南市官田區嘉南國民小學","臺南市官田區官田國民小學","臺南市官田區渡拔國民小學","臺南市官田區隆田國民小學","臺南市將軍區將軍國民小學","臺南市將軍區漚汪國民小學","臺南市將軍區苓和國民小學","臺南市將軍區長平國民小學","臺南市將軍區鯤鯓國民小學","臺南市山上區山上國民小學","臺南市左鎮區光榮國民小學","臺南市左鎮區左鎮國民小學","臺南市後壁區安溪國民小學","臺南市後壁區後壁國民小學","臺南市後壁區新嘉國民小學","臺南市後壁區新東國民小學","臺南市後壁區樹人國民小學","臺南市後壁區永安國民小學","臺南市後壁區菁寮國民小學","臺南市新化區口埤實驗小學","臺南市新化區大新國民小學","臺南市新化區新化國民小學","臺南市新化區正新國民小學","臺南市新化區那拔國民小學","臺南市新市區大社國民小學","臺南市新市區新市國民小學","臺南市新營區公誠國民小學","臺南市新營區南梓實驗小學","臺南市新營區土庫國民小學","臺南市新營區新橋國民小學","臺南市新營區新民國民小學","臺南市新營區新泰國民小學","臺南市新營區新營國民小學","臺南市新營區新生國民小學","臺南市新營區新興國民小學","臺南市新營區新進國民小學","臺南市東區勝利國民小學","臺南市東區博愛國民小學","臺南市東區大同國民小學","臺南市東區崇學國民小學","臺南市東區崇明國民小學","臺南市東區復興國民小學","臺南市東區德高國民小學","臺南市東區東光國民小學","臺南市東區第五幼稚園","臺南市東區裕文國民小學","臺南市東山區吉貝耍國民小學","臺南市東山區東原國民小學","臺南市東山區東山國民小學","臺南市東山區聖賢國民小學","臺南市東山區青山國民小學","臺南市柳營區太康國民小學","臺南市柳營區新山國民小學","臺南市柳營區果毅國民小學","臺南市柳營區柳營國民小學","臺南市柳營區重溪國民小學","臺南市楠西區楠西國民小學","臺南市歸仁區保西國民小學","臺南市歸仁區大潭國民小學","臺南市歸仁區文化國民小學","臺南市歸仁區歸仁國民小學","臺南市歸仁區歸南國民小學","臺南市歸仁區紅瓦厝國民小學","臺南市永康區三村國民小學","臺南市永康區五王國民小學","臺南市永康區勝利國民小學","臺南市永康區大橋國民小學","臺南市永康區大灣國民小學","臺南市永康區崑山國民小學","臺南市永康區復興國民小學","臺南市永康區永信國民小學","臺南市永康區永康國民小學","臺南市永康區西勢國民小學","臺南市永康區龍潭國民小學","臺南市玉井區層林國民小學","臺南市玉井區玉井國民小學","臺南市白河區仙草國民小學","臺南市白河區內角國民小學","臺南市白河區大竹國民小學","臺南市白河區河東國民小學","臺南市白河區玉豐國民小學","臺南市白河區白河國民小學","臺南市白河區竹門國民小學","臺南市私立光華女子高級中學","臺南市私立南英高級商工職業學校","臺南市私立崑山高級中學","臺南市私立建業高級中學","臺南市私立德光女子高級中學","臺南市私立慈幼高級工商職業學校","臺南市私立慈濟國民小學","臺南市私立慈濟高級中學","臺南市私立昭明國民中學","臺南市私立瀛海高級中學","臺南市私立聖功女子高級中學","臺南市私立長榮女子高級中學","臺南市私立長榮高級中學","臺南市立七股托兒所","臺南市立下營國民中學","臺南市立下營幼兒園","臺南市立下營托兒所","臺南市立中山國民中學","臺南市立仁德國民中學","臺南市立仁德幼兒園","臺南市立仁德托兒所","臺南市立仁德文賢國民中學","臺南市立佳興國民中學","臺南市立佳里國民中學","臺南市立六甲國民中學","臺南市立六甲幼兒園","臺南市立六甲托兒所","臺南市立北門國民中學","臺南市立南化國民中學","臺南市立南寧高級中學","臺南市立南新國民中學","臺南市立和順國民中學","臺南市立善化國民中學","臺南市立善化托兒所","臺南市立土城高級中學","臺南市立大內國民中學","臺南市立大成國民中學","臺南市立大橋國民中學","臺南市立大灣高級中學","臺南市立太子國民中學","臺南市立學甲國民中學","臺南市立學甲幼兒園","臺南市立學甲托兒所","臺南市立安南國民中學","臺南市立安定國民中學","臺南市立安定托兒所","臺南市立安平國民中學","臺南市立安順國民中學","臺南市立官田國民中學","臺南市立將軍國民中學","臺南市立將軍幼兒園","臺南市立將軍托兒所","臺南市立小康幼兒園","臺南市立小康托兒所","臺南市立山上國民中學","臺南市立崇明國民中學","臺南市立左鎮國民中學","臺南市立延平國民中學","臺南市立建興國民中學","臺南市立後壁國民中學","臺南市立後港國民中學","臺南市立後甲國民中學","臺南市立復興國民中學","臺南市立忠孝國民中學","臺南市立成功國民中學","臺南市立文賢國民中學","臺南市立新化國民中學","臺南市立新化幼兒園","臺南市立新化托兒所","臺南市立新市國民中學","臺南市立新市幼兒園","臺南市立新市托兒所","臺南市立新東國民中學","臺南市立新興國民中學","臺南市立東原國民中學","臺南市立東山國民中學","臺南市立柳營國民中學","臺南市立楠西國民中學","臺南市立歸仁國民中學","臺南市立歸仁幼兒園","臺南市立歸仁托兒所","臺南市立民德國民中學","臺南市立永仁高級中學","臺南市立永康國民中學","臺南市立永康托兒所","臺南市立沙崙國民中學","臺南市立海佃國民中學","臺南市立玉井國民中學","臺南市立白河國民中學","臺南市立白河托兒所","臺南市立竹橋國民中學","臺南市立第一幼兒園","臺南市立第一幼稚園","臺南市立第一托兒所","臺南市立第三幼兒園","臺南市立第三幼稚園","臺南市立第三托兒所","臺南市立第二幼兒園","臺南市立第二托兒所","臺南市立第五幼兒園","臺南市立第五幼稚園","臺南市立第五托兒所","臺南市立第六幼兒園","臺南市立第六托兒所","臺南市立第四幼兒園","臺南市立第四托兒所","臺南市立菁寮國民中學","臺南市立西港國民中學","臺南市立金城國民中學","臺南市立關廟國民中學","臺南市立關廟幼兒園","臺南市立關廟托兒所","臺南市立鹽水國民中學","臺南市立麻豆國民中學","臺南市立麻豆幼兒園","臺南市立麻豆托兒所","臺南市立龍崎國民中學","臺南市立龍崎托兒所","臺南市西港區後營國民小學","臺南市西港區成功國民小學","臺南市西港區松林國民小學","臺南市西港區港東國民小學","臺南市西港區西港國民小學","臺南市關廟區五甲國民小學","臺南市關廟區保東國民小學","臺南市關廟區崇和國民小學","臺南市關廟區文和實驗小學","臺南市關廟區新光國民小學","臺南市關廟區深坑國民小學","臺南市關廟區關廟國民小學","臺南市鹽水區仁光國民小學","臺南市鹽水區坔頭港國民小學","臺南市鹽水區岸內國民小學","臺南市鹽水區文昌國民小學","臺南市鹽水區月津國民小學","臺南市鹽水區歡雅國民小學","臺南市鹽水區竹埔國民小學","臺南市鹽水區鹽水國民小學","臺南市麻豆區北勢國民小學","臺南市麻豆區培文國民小學","臺南市麻豆區大山國民小學","臺南市麻豆區安業國民小學","臺南市麻豆區文正國民小學","臺南市麻豆區港尾國民小學","臺南市麻豆區紀安國民小學","臺南市麻豆區麻豆國民小學","臺南市龍崎區龍崎國民小學","臺南師範學院","臺南海事","臺南第一高級中學","臺南第二高級中學","臺南藝術大學","臺南藝術學院","臺南護理專科學校","臺南附中","臺南高商","臺南高工","臺南高等工業學校","臺南高級商業職業學校","臺南高級工業職業學校","臺南高級海事水產職業學校","臺坂國小","臺坂國民小學","臺大","臺東大學","臺東大學附屬特殊教育學校","臺東大學附屬體育高級中學","臺東大學附設實驗國民小學","臺東女中","臺東女子高中","臺東女子高級中學","臺東女高","臺東專科學校","臺東市立托兒所","臺東師範學院","臺東幼稚園","臺東縣卑南鄉初鹿國民小學","臺東縣卑南鄉利嘉國民小學","臺東縣卑南鄉大南國民小學","臺東縣卑南鄉太平國民小學","臺東縣卑南鄉富山國民小學","臺東縣卑南鄉東成國民小學","臺東縣卑南鄉溫泉國民小學","臺東縣卑南鄉立托兒所","臺東縣卑南鄉賓朗國民小學","臺東縣大武鄉大武國民小學","臺東縣大武鄉大鳥國民小學","臺東縣大武鄉尚武國民小學","臺東縣大武鄉立托兒所","臺東縣太麻里鄉三和國民小學","臺東縣太麻里鄉大溪國民小學","臺東縣太麻里鄉大王國民小學","臺東縣太麻里鄉立托兒所","臺東縣太麻里鄉美和國民小學","臺東縣太麻里鄉香蘭國民小學","臺東縣延平鄉桃源國民小學","臺東縣延平鄉武陵國民小學","臺東縣延平鄉立托兒所","臺東縣延平鄉紅葉國民小學","臺東縣延平鄉鸞山國民小學","臺東縣成功鎮三仙國民小學","臺東縣成功鎮三民國民小學","臺東縣成功鎮信義國民小學","臺東縣成功鎮忠孝國民小學","臺東縣成功鎮成功國民小學","臺東縣成功鎮立托兒所","臺東縣東河鄉北源國民小學","臺東縣東河鄉東河國民小學","臺東縣東河鄉泰源國民小學","臺東縣東河鄉立托兒所","臺東縣東河鄉興隆國民小學","臺東縣東河鄉都蘭國民小學","臺東縣池上鄉大坡國民小學","臺東縣池上鄉福原國民小學","臺東縣池上鄉立托兒所","臺東縣池上鄉萬安國民小學","臺東縣海端鄉初來國民小學","臺東縣海端鄉加拿國民小學","臺東縣海端鄉崁頂國民小學","臺東縣海端鄉廣原國民小學","臺東縣海端鄉海端國民小學","臺東縣海端鄉立托兒所","臺東縣海端鄉錦屏國民小學","臺東縣海端鄉霧鹿國民小學","臺東縣私立公東高級工業職業學校","臺東縣私立育仁高級中學","臺東縣立初鹿國民中學","臺東縣立卑南國民中學","臺東縣立大武國民中學","臺東縣立大王國民中學","臺東縣立寶桑國民中學","臺東縣立托兒所","臺東縣立新港國民中學","臺東縣立新生國民中學","臺東縣立新生幼兒園","臺東縣立東海國民中學","臺東縣立桃源國民中學","臺東縣立池上國民中學","臺東縣立泰源國民中學","臺東縣立海端國民中學","臺東縣立瑞源國民中學","臺東縣立知本國民中學","臺東縣立綠島國民中學","臺東縣立蘭嶼高級中學","臺東縣立豐田國民中學","臺東縣立賓茂國民中學","臺東縣立都蘭國民中學","臺東縣立長濱國民中學","臺東縣立關山國民中學","臺東縣立鹿野國民中學","臺東縣綠島鄉公館國民小學","臺東縣綠島鄉立托兒所","臺東縣綠島鄉綠島國民小學","臺東縣臺東市仁愛國民小學","臺東縣臺東市光明國民小學","臺東縣臺東市卑南國民小學","臺東縣臺東市南王國民小學","臺東縣臺東市富岡國民小學","臺東縣臺東市寶桑國民小學","臺東縣臺東市岩灣國民小學","臺東縣臺東市康樂國民小學","臺東縣臺東市建和國民小學","臺東縣臺東市復興國民小學","臺東縣臺東市新園國民小學","臺東縣臺東市新生國民小學","臺東縣臺東市東海國民小學","臺東縣臺東市知本國民小學","臺東縣臺東市立托兒所","臺東縣臺東市臺東幼稚園","臺東縣臺東市豐年國民小學","臺東縣臺東市豐榮國民小學","臺東縣臺東市豐源國民小學","臺東縣臺東市豐田國民小學","臺東縣臺東市豐里國民小學","臺東縣臺東市馬蘭國民小學","臺東縣蘭嶼鄉朗島國民小學","臺東縣蘭嶼鄉東清國民小學","臺東縣蘭嶼鄉椰油國民小學","臺東縣蘭嶼鄉立托兒所","臺東縣蘭嶼鄉蘭嶼國民小學","臺東縣達仁鄉土坂國民小學","臺東縣達仁鄉安朔國民小學","臺東縣達仁鄉立托兒所","臺東縣達仁鄉臺坂國民小學","臺東縣金峰鄉介達國民小學","臺東縣金峰鄉嘉蘭國民小學","臺東縣金峰鄉新興國民小學","臺東縣金峰鄉立托兒所","臺東縣金峰鄉賓茂國民小學","臺東縣長濱鄉三間國民小學","臺東縣長濱鄉寧埔國民小學","臺東縣長濱鄉樟原國民小學","臺東縣長濱鄉立托兒所","臺東縣長濱鄉竹湖國民小學","臺東縣長濱鄉長濱國民小學","臺東縣關山鎮德高國民小學","臺東縣關山鎮月眉國民小學","臺東縣關山鎮立托兒所","臺東縣關山鎮關山國民小學","臺東縣關山鎮電光國民小學","臺東縣鹿野鄉永安國民小學","臺東縣鹿野鄉瑞源國民小學","臺東縣鹿野鄉瑞豐國民小學","臺東縣鹿野鄉立托兒所","臺東縣鹿野鄉鹿野國民小學","臺東縣鹿野鄉龍田國民小學","臺東體中","臺東高商","臺東高級中學","臺東高級商業職業學校","臺灣大學","臺灣工業技術學院","臺灣師範大學","臺灣師範大學附屬高級中學","臺灣戲曲學院","臺灣教育學院","臺灣海洋大學","臺灣海洋學院","臺灣科技大學","臺灣藝術大學","臺灣藝術學院","臺灣觀光學院","臺灣警察專科學校","臺灣體大","臺灣體育學院","臺灣體育運動大學","臺科","臺科大","臺興國小","臺興國民小學","臺藝大","臺西國中","臺西國小","臺西國民中學","臺西國民小學","臺西鄉立托兒所","臺觀","臺觀院","臺體","興中國中","興中國小","興中國民中學","興中國民小學","興仁國中","興仁國小","興仁國民中學","興仁國民小學","興化國小","興化國民小學","興南國中","興南國小","興南國民中學","興南國民小學","興嘉國小","興嘉國民小學","興國國小","興國國民小學","興國管理學院","興大","興大附中","興大附農","興安國小","興安國民小學","興德國小","興德國民小學","興昌國小","興昌國民小學","興田國小","興田國民小學","興福國中","興福國小","興福國民中學","興福國民小學","興穀","興穀國民小學","興糖國小","興糖國民小學","興華國小","興華國民小學","興華高中","興華高級中學","興達國小","興達國民小學","興隆國小","興隆國民小學","興雅國中","興雅國小","興雅國民中學","興雅國民小學","舊城國小","舊城國民小學","舊寮國小","舊寮國民小學","舊庄國小","舊庄國民小學","舊社國小","舊社國民小學","舊莊國小","舊莊國民小學","舊館國小","舊館國民小學","舞鶴國小","舞鶴國民小學","芎林國中","芎林國小","芎林國民中學","芎林國民小學","芎林鄉立托兒所","芙朝國小","芙朝國民小學","芝山國小","芝山國民小學","芬園國中","芬園國小","芬園國民中學","芬園國民小學","芬園鄉立托兒所","芭里國小","芭里國民小學","花中","花園國小","花園國民小學","花壇國中","花壇國小","花壇國民中學","花壇國民小學","花壇鄉立托兒所","花崗國中","花崗國民中學","花嶼國小","花嶼國民小學","花師","花蓮商","花蓮女中","花蓮女子高中","花蓮女子高級中學","花蓮女高","花蓮市立托兒所","花蓮師範學院","花蓮特教","花蓮特殊教育學校","花蓮縣光復鄉光復國民小學","花蓮縣光復鄉大興國民小學","花蓮縣光復鄉大進國民小學","花蓮縣光復鄉太巴塱國民小學","花蓮縣光復鄉立托兒所","花蓮縣光復鄉西富國民小學","花蓮縣卓溪鄉卓楓國民小學","花蓮縣卓溪鄉卓樂國民小學","花蓮縣卓溪鄉卓清國民小學","花蓮縣卓溪鄉卓溪國民小學","花蓮縣卓溪鄉古風國民小學","花蓮縣卓溪鄉太平國民小學","花蓮縣卓溪鄉崙山國民小學","花蓮縣卓溪鄉立山國民小學","花蓮縣卓溪鄉立托兒所","花蓮縣吉安鄉光華國民小學","花蓮縣吉安鄉化仁國民小學","花蓮縣吉安鄉北昌國民小學","花蓮縣吉安鄉南華國民小學","花蓮縣吉安鄉吉安國民小學","花蓮縣吉安鄉太昌國民小學","花蓮縣吉安鄉宜昌國民小學","花蓮縣吉安鄉稻香國民小學","花蓮縣吉安鄉立托兒所","花蓮縣壽豐鄉壽豐國民小學","花蓮縣壽豐鄉平和國民小學","花蓮縣壽豐鄉志學國民小學","花蓮縣壽豐鄉月眉國民小學","花蓮縣壽豐鄉水璉國民小學","花蓮縣壽豐鄉溪口國民小學","花蓮縣壽豐鄉立托兒所","花蓮縣壽豐鄉豐山國民小學","花蓮縣壽豐鄉豐裡國民小學","花蓮縣富里鄉吳江國民小學","花蓮縣富里鄉學田國民小學","花蓮縣富里鄉富里國民小學","花蓮縣富里鄉明里國民小學","花蓮縣富里鄉東竹國民小學","花蓮縣富里鄉東里國民小學","花蓮縣富里鄉永豐國民小學","花蓮縣富里鄉立托兒所","花蓮縣富里鄉萬寧國民小學","花蓮縣新城鄉北埔國民小學","花蓮縣新城鄉嘉里國民小學","花蓮縣新城鄉康樂國民小學","花蓮縣新城鄉新城國民小學","花蓮縣新城鄉立托兒所","花蓮縣玉里鎮三民國民小學","花蓮縣玉里鎮中城國民小學","花蓮縣玉里鎮大禹國民小學","花蓮縣玉里鎮德武國民小學","花蓮縣玉里鎮春日國民小學","花蓮縣玉里鎮松浦國民小學","花蓮縣玉里鎮樂合國民小學","花蓮縣玉里鎮源城國民小學","花蓮縣玉里鎮玉里國民小學","花蓮縣玉里鎮觀音國民小學","花蓮縣玉里鎮長良國民小學","花蓮縣玉里鎮高寮國民小學","花蓮縣瑞穗鄉奇美國民小學","花蓮縣瑞穗鄉富源國民小學","花蓮縣瑞穗鄉瑞北國民小學","花蓮縣瑞穗鄉瑞穗國民小學","花蓮縣瑞穗鄉瑞美國民小學","花蓮縣瑞穗鄉立托兒所","花蓮縣瑞穗鄉舞鶴國民小學","花蓮縣瑞穗鄉鶴岡國民小學","花蓮縣秀林鄉三棧國民小學","花蓮縣秀林鄉佳民國民小學","花蓮縣秀林鄉和平國民小學","花蓮縣秀林鄉富世國民小學","花蓮縣秀林鄉崇德國民小學","花蓮縣秀林鄉文蘭國民小學","花蓮縣秀林鄉景美國民小學","花蓮縣秀林鄉水源國民小學","花蓮縣秀林鄉秀林國民小學","花蓮縣秀林鄉立托兒所","花蓮縣秀林鄉西寶國民小學","花蓮縣秀林鄉銅蘭國民小學","花蓮縣秀林鄉銅門國民小學","花蓮縣私立中華高級工商職業學校","花蓮縣私立四維高級中學","花蓮縣私立國光高級商工職業學校","花蓮縣私立海星高級中學","花蓮縣立三民國民中學","花蓮縣立光復國民中學","花蓮縣立化仁國民中學","花蓮縣立南平中學","花蓮縣立吉安國民中學","花蓮縣立國風國民中學","花蓮縣立壽豐國民中學","花蓮縣立宜昌國民中學","花蓮縣立富北國民中學","花蓮縣立富源國民中學","花蓮縣立富里國民中學","花蓮縣立平和國民中學","花蓮縣立新城國民中學","花蓮縣立東里國民中學","花蓮縣立玉東國民中學","花蓮縣立玉里國民中學","花蓮縣立瑞穗國民中學","花蓮縣立秀林國民中學","花蓮縣立美崙國民中學","花蓮縣立自強國民中學","花蓮縣立花崗國民中學","花蓮縣立萬榮國民中學","花蓮縣立豐濱國民中學","花蓮縣立體育高級中等學校","花蓮縣立鳳林國民中學","花蓮縣縣立托兒所","花蓮縣花蓮市中原國民小學","花蓮縣花蓮市中正國民小學","花蓮縣花蓮市中華國民小學","花蓮縣花蓮市信義國民小學","花蓮縣花蓮市北濱國民小學","花蓮縣花蓮市國福國民小學","花蓮縣花蓮市復興國民小學","花蓮縣花蓮市忠孝國民小學","花蓮縣花蓮市明廉國民小學","花蓮縣花蓮市明恥國民小學","花蓮縣花蓮市明禮國民小學","花蓮縣花蓮市明義國民小學","花蓮縣花蓮市立托兒所","花蓮縣花蓮市鑄強國民小學","花蓮縣萬榮鄉明利國民小學","花蓮縣萬榮鄉立托兒所","花蓮縣萬榮鄉紅葉國民小學","花蓮縣萬榮鄉萬榮國民小學","花蓮縣萬榮鄉西林國民小學","花蓮縣萬榮鄉見晴國民小學","花蓮縣萬榮鄉馬遠國民小學","花蓮縣豐濱鄉新社國民小學","花蓮縣豐濱鄉港口國民小學","花蓮縣豐濱鄉立托兒所","花蓮縣豐濱鄉豐濱國民小學","花蓮縣豐濱鄉靜浦國民小學","花蓮縣鳳林鎮北林國民小學","花蓮縣鳳林鎮大榮國民小學","花蓮縣鳳林鎮林榮國民小學","花蓮縣鳳林鎮立托兒所","花蓮縣鳳林鎮長橋國民小學","花蓮縣鳳林鎮鳳仁國民小學","花蓮縣鳳林鎮鳳林國民小學","花蓮高商","花蓮高工","花蓮高級中學","花蓮高級商業職業學校","花蓮高級工業職業學校","花蓮高級農業職業學校","花蓮高職","花蓮高農","芳和國中","芳和實中","芳和實驗國民中學","芳苑國中","芳苑國小","芳苑國民中學","芳苑國民小學","芳苑鄉立托兒所","苑裡國小","苑裡國民小學","苑裡鎮立托兒所","苑裡高級中學","苑高","苓和國小","苓和國民小學","苓洲國小","苓洲國民小學","苓雅國中","苓雅國民中學","苗栗國中","苗栗國民中學","苗栗特教","苗栗特殊教育學校","苗栗縣三灣鄉三灣國民小學","苗栗縣三灣鄉立幼兒園","苗栗縣三灣鄉立托兒所","苗栗縣三義鄉僑成國民小學","苗栗縣三義鄉建中國民小學","苗栗縣三義鄉立托兒所","苗栗縣三義鄉育英國民小學","苗栗縣三義鄉鯉魚國民小學","苗栗縣公館鄉五穀國民小學","苗栗縣公館鄉仁愛國民小學","苗栗縣公館鄉公館國民小學","苗栗縣公館鄉南河國民小學","苗栗縣公館鄉福基國民小學","苗栗縣公館鄉開礦國民小學","苗栗縣公館鄉鶴岡國民小學","苗栗縣卓蘭鎮內灣國民小學","苗栗縣卓蘭鎮卓蘭國民小學","苗栗縣卓蘭鎮坪林國民小學","苗栗縣卓蘭鎮景山國民小學","苗栗縣卓蘭鎮立托兒所","苗栗縣卓蘭鎮豐田國民小學","苗栗縣卓蘭鎮雙連國民小學","苗栗縣南庄鄉南埔國民小學","苗栗縣南庄鄉南庄國民小學","苗栗縣南庄鄉東河國民小學","苗栗縣南庄鄉田美國民小學","苗栗縣南庄鄉立托兒所","苗栗縣南庄鄉蓬萊國民小學","苗栗縣大湖鄉南湖國民小學","苗栗縣大湖鄉大南國民小學","苗栗縣大湖鄉大湖國民小學","苗栗縣大湖鄉新開國民小學","苗栗縣大湖鄉東興國民小學","苗栗縣大湖鄉栗林國民小學","苗栗縣大湖鄉武榮國民小學","苗栗縣大湖鄉立托兒所","苗栗縣大湖鄉華興國民小學","苗栗縣後龍鎮中和國民小學","苗栗縣後龍鎮同光國民小學","苗栗縣後龍鎮外埔國民小學","苗栗縣後龍鎮大山國民小學","苗栗縣後龍鎮後龍國民小學","苗栗縣後龍鎮成功國民小學","苗栗縣後龍鎮海寶國民小學","苗栗縣後龍鎮溪洲國民小學","苗栗縣後龍鎮立托兒所","苗栗縣後龍鎮龍坑國民小學","苗栗縣泰安鄉士林國民小學","苗栗縣泰安鄉梅園國民小學","苗栗縣泰安鄉汶水國民小學","苗栗縣泰安鄉泰興國民小學","苗栗縣泰安鄉清安國民小學","苗栗縣泰安鄉立托兒所","苗栗縣泰安鄉象鼻國民小學","苗栗縣獅潭鄉永興國民小學","苗栗縣獅潭鄉獅潭國民小學","苗栗縣獅潭鄉豐林國民小學","苗栗縣私立中興高級商工職業學校","苗栗縣私立仁德高級醫事職業學校","苗栗縣私立君毅高級中學","苗栗縣私立大成高級中學","苗栗縣私立建台高級中學","苗栗縣私立建臺高級中學","苗栗縣私立育民高級工業家事職業學校","苗栗縣私立賢德高級工商職業學校","苗栗縣私立龍德家事商業職業學校","苗栗縣立三灣國民中學","苗栗縣立三義高級中學","苗栗縣立公館國民中學","苗栗縣立南和國民中學","苗栗縣立南庄國民中學","苗栗縣立南湖國民中學","苗栗縣立啟新國民中學","苗栗縣立大倫國民中學","苗栗縣立大同高級中學","苗栗縣立大湖國民中學","苗栗縣立大西國民中學","苗栗縣立建國國民中學","苗栗縣立後龍國民中學","苗栗縣立文林國民中學","苗栗縣立文英國民中學","苗栗縣立新港國民中小學","苗栗縣立明仁國民中學","苗栗縣立泰安國民中小學","苗栗縣立烏眉國民中學","苗栗縣立照南國民中學","苗栗縣立獅潭國民中學","苗栗縣立福興武術國民中小學","苗栗縣立竹南國民中學","苗栗縣立維真國民中學","苗栗縣立致民國民中學","苗栗縣立興華高級中學","苗栗縣立苑裡高級中學","苗栗縣立苗栗國民中學","苗栗縣立西湖國民中學","苗栗縣立通霄國民中學","苗栗縣立造橋國民中學","苗栗縣立頭份國民中學","苗栗縣立頭屋國民中學","苗栗縣立鶴岡國民中學","苗栗縣竹南鎮大埔國民小學","苗栗縣竹南鎮山佳國民小學","苗栗縣竹南鎮新南國民小學","苗栗縣竹南鎮海口國民小學","苗栗縣竹南鎮照南國民小學","苗栗縣竹南鎮立托兒所","苗栗縣竹南鎮竹南國民小學","苗栗縣竹南鎮竹興國民小學","苗栗縣竹南鎮頂埔國民小學","苗栗縣苑裡鎮中山國民小學","苗栗縣苑裡鎮中正國民小學","苗栗縣苑裡鎮客庄國民小學","苗栗縣苑裡鎮山腳國民小學","苗栗縣苑裡鎮文苑國民小學","苗栗縣苑裡鎮林森國民小學","苗栗縣苑裡鎮立托兒所","苗栗縣苑裡鎮苑裡國民小學","苗栗縣苑裡鎮蕉埔國民小學","苗栗縣苑裡鎮藍田國民小學","苗栗縣苗栗市僑育國民小學","苗栗縣苗栗市啟文國民小學","苗栗縣苗栗市大同國民小學","苗栗縣苗栗市建功國民小學","苗栗縣苗栗市文山國民小學","苗栗縣苗栗市文華國民小學","苗栗縣苗栗市新英國民小學","苗栗縣苗栗市福星國民小學","苗栗縣西湖鄉五湖國民小學","苗栗縣西湖鄉僑文國民小學","苗栗縣西湖鄉立托兒所","苗栗縣西湖鄉西湖國民小學","苗栗縣通霄鎮五福國民小學","苗栗縣通霄鎮南和國民小學","苗栗縣通霄鎮啟明國民小學","苗栗縣通霄鎮圳頭國民小學","苗栗縣通霄鎮坪頂國民小學","苗栗縣通霄鎮城中國民小學","苗栗縣通霄鎮新埔國民小學","苗栗縣通霄鎮烏眉國民小學","苗栗縣通霄鎮立托兒所","苗栗縣通霄鎮通霄國民小學","苗栗縣造橋鄉僑樂國民小學","苗栗縣造橋鄉立托兒所","苗栗縣造橋鄉談文國民小學","苗栗縣造橋鄉造橋國民小學","苗栗縣造橋鄉錦水國民小學","苗栗縣造橋鄉龍昇國民小學","苗栗縣銅鑼鄉中興國民小學","苗栗縣銅鑼鄉九湖國民小學","苗栗縣銅鑼鄉文峰國民小學","苗栗縣銅鑼鄉新隆國民小學","苗栗縣銅鑼鄉立托兒所","苗栗縣銅鑼鄉興隆國民小學","苗栗縣銅鑼鄉銅鑼國民小學","苗栗縣頭份市信德國民小學","苗栗縣頭份市信義國民小學","苗栗縣頭份市僑善國民小學","苗栗縣頭份市六合國民小學","苗栗縣頭份市后庄國民小學","苗栗縣頭份市尖山國民小學","苗栗縣頭份市建國國民小學","苗栗縣頭份市斗煥國民小學","苗栗縣頭份市新興國民小學","苗栗縣頭份市永貞國民小學","苗栗縣頭份市蟠桃國民小學","苗栗縣頭份市頭份國民小學","苗栗縣頭份鎮立托兒所","苗栗縣頭屋鄉明德國民小學","苗栗縣頭屋鄉頭屋國民小學","苗栗農工","苗栗高中","苗栗高商","苗栗高級中學","苗栗高級商業職業學校","苗栗高級農工職業學校","英明國中","英明國民中學","茂林國中","茂林國小","茂林國民中學","茂林國民小學","茄拔國小","茄拔國民小學","茄苳國小","茄苳國民小學","茄荖國小","茄荖國民小學","茄萣國中","茄萣國小","茄萣國民中學","茄萣國民小學","茶山國小","茶山國民小學","草埔國小","草埔國民小學","草屯商工","草屯國中","草屯國小","草屯國民中學","草屯國民小學","草屯鎮立托兒所","草屯高級商工職業學校","草嶺生態地質國小","草嶺生態地質國民小學","草港國小","草港國民小學","草湖國中","草湖國小","草湖國民中學","草湖國民小學","草漯國中","草漯國小","草漯國民中學","草漯國民小學","荖濃國小","荖濃國民小學","莊敬國小","莊敬國民小學","莒光國小","莒光國民小學","莿桐國中","莿桐國小","莿桐國民中學","莿桐國民小學","莿桐鄉立托兒所","菁埔國小","菁埔國民小學","菁寮國中","菁寮國小","菁寮國民中學","菁寮國民小學","菁桐國小","菁桐國民小學","菓林國小","菓林國民小學","華僑高中","華僑高級中等學校","華勛國小","華勛國民小學","華南商職","華南國小","華南國民小學","華南實小","華南實驗國民小學","華南高商","華南高級商業職業學校","華商","華夏技術學院","華山國中","華山國小","華山國民中學","華山國民小學","華岡藝校","華岡藝術學校","華德福","華德福實小","華德福實驗學校","華東台商子女學校","華東學校","華東臺商子女學校","華梵","華梵人文科技學院","華梵大學","華江國小","華江國民小學","華江高中","華江高級中學","華洲工家","華洲高級工業家事職業學校","華興國小","華興國民小學","華興小學","華興高中","華興高級中學","華藝","華龍國小","華龍國民小學","萬丹國中","萬丹國小","萬丹國民中學","萬丹國民小學","萬來國小","萬來國民小學","萬合國小","萬合國民小學","萬和國中","萬和國民中學","萬大國小","萬大國民小學","萬安國小","萬安國民小學","萬富國小","萬富國民小學","萬寧國小","萬寧國民小學","萬巒國中","萬巒國小","萬巒國民中學","萬巒國民小學","萬巒鄉立托兒所","萬新國中","萬新國民中學","萬榮國中","萬榮國小","萬榮國民中學","萬榮國民小學","萬榮鄉立托兒所","萬福國小","萬福國民小學","萬能工商","萬能技術學院","萬能科大","萬能科技大學","萬能高級工商職業學校","萬興國中","萬興國小","萬興國民中學","萬興國民小學","萬芳國小","萬芳國民小學","萬芳高中","萬芳高級中學","萬華國中","萬華國民中學","萬華幼兒園","萬華托兒所","萬華社區大學","萬華社大","萬豐國小","萬豐國民小學","萬里國中","萬里國小","萬里國民中學","萬里國民小學","萬里幼兒園","萬里托兒所","萬隆國小","萬隆國民小學","葫蘆國小","葫蘆國民小學","葫蘆墩國小","葫蘆墩國民小學","蒜頭國小","蒜頭國民小學","蓬萊國小","蓬萊國民小學","蔡文國小","蔡文國民小學","蔦松國中","蔦松國小","蔦松國民中學","蔦松國民小學","蕉埔國小","蕉埔國民小學","薇閣小學","薇閣高中","薇閣高級中學","藍田國小","藍田國民小學","藝術學院","藝高","蘆中","蘆洲國小","蘆洲國民中學","蘆洲國民小學","蘆洲幼兒園","蘆洲托兒所","蘆竹國小","蘆竹國民小學","蘆竹幼兒園","蘇澳國中","蘇澳國小","蘇澳國民中學","蘇澳國民小學","蘇澳海事","蘇澳鎮立托兒所","蘇澳高級海事水產職業學校","蘭嶼國小","蘭嶼國民小學","蘭嶼鄉立托兒所","蘭嶼高中","蘭嶼高級中學","蘭州國中","蘭州國民中學","蘭潭國中","蘭潭國小","蘭潭國民中學","蘭潭國民小學","蘭院","蘭陽女中","蘭陽女子高級中學","蘭陽學院","蘭陽技術學院","蘭雅國中","蘭雅國小","蘭雅國民中學","蘭雅國民小學","虎井國小","虎井國民小學","虎尾國中","虎尾國小","虎尾國民中學","虎尾國民小學","虎尾技術學院","虎尾技術學院附設進修學院","虎尾科技大學","虎尾農工","虎尾鎮立仁國小","虎尾鎮立仁國民小學","虎尾鎮立托兒所","虎尾高中","虎尾高級中學","虎尾高級農工職業學校","虎山國小","虎山國民小學","虎山實小","虎山實驗小學","虎林國中","虎林國小","虎林國民中學","虎林國民小學","虎科","虎科大","蚵寮國中","蚵寮國小","蚵寮國民中學","蚵寮國民小學","蚵間國小","蚵間國民小學","螢橋國中","螢橋國小","螢橋國民中學","螢橋國民小學","螺陽國小","螺陽國民小學","螺青國小","螺青國民小學","蟠桃國小","蟠桃國民小學","行政學校","行政管理學校","行政院人事行政總處公務人力發展學院","衛理女中","衛理女子高級中學","衛道","衛道高級中學","裕文國小","裕文國民小學","裕民國小","裕民國民小學","裕誠幼兒園","裕誠幼稚園","褒忠國中","褒忠國小","褒忠國民中學","褒忠國民小學","褒忠鄉立托兒所","西勢國小","西勢國民小學","西口國小","西口國民小學","西園國小","西園國民小學","西埔國小","西埔國民小學","西定國小","西定國民小學","西富國小","西富國民小學","西寧國小","西寧國民小學","西寶國小","西寶國民小學","西屯國小","西屯國民小學","西岐國小","西岐國民小學","西嶺國小","西嶺國民小學","西嶼國中","西嶼國民中學","西嶼鄉立托兒所","西松國小","西松國民小學","西松高中","西松高級中學","西林國小","西林國民小學","西港國中","西港國小","西港國民中學","西港國民小學","西湖國中","西湖國小","西湖國民中學","西湖國民小學","西湖鄉立托兒所","西溪國小","西溪國民小學","西苑高中","西苑高級中學","西螺國中","西螺國民中學","西螺農工","西螺鎮立托兒所","西螺高級農工職業學校","西門國小","西門國民小學","西門實小","西門實驗小學","見晴國小","見晴國民小學","親愛國小","親愛國民小學","親民工商專科學校","觀亭國小","觀亭國民小學","觀音國中","觀音國小","觀音國民中學","觀音國民小學","觀音幼兒園","觀音高中","觀音高級中等學校","誠正中學","誠正國中","誠正國小","誠正國民中學","誠正國民小學","談文國小","談文國民小學","講美國小","講美國民小學","警大","警察專科學校","警專","豐丘國小","豐丘國民小學","豐南國中","豐南國民中學","豐原商業高級中等學校","豐原國中","豐原國小","豐原國民中學","豐原國民小學","豐原托兒所","豐原高中","豐原高商","豐原高級中等學校","豐安國小","豐安國民小學","豐山國小","豐山國民小學","豐山實驗教育學校","豐崙國小","豐崙國民小學","豐年國小","豐年國民小學","豐村國小","豐村國民小學","豐東國中","豐東國民中學","豐林國小","豐林國民小學","豐榮國小","豐榮國民小學","豐洲國小","豐洲國民小學","豐源國小","豐源國民小學","豐濱國中","豐濱國小","豐濱國民中學","豐濱國民小學","豐濱鄉立托兒所","豐珠中小學","豐珠國民中小學","豐田國中","豐田國小","豐田國民中學","豐田國民小學","豐裡國小","豐裡國民小學","豐里國小","豐里國民小學","豐陽國中","豐陽國民中學","象鼻國小","象鼻國民小學","財團法人雲林縣私立福智高級中學","貢中","貢寮國小","貢寮國民中學","貢寮國民小學","貢寮幼兒園","貢寮托兒所","貴林國小","貴林國民小學","賀建國小","賀建國民小學","賓朗國小","賓朗國民小學","賓茂國中","賓茂國小","賓茂國民中學","賓茂國民小學","賢北國小","賢北國民小學","賢庵國小","賢庵國民小學","賢德工商","賢德高級工商職業學校","賴厝國小","賴厝國民小學","賽嘉國小","賽嘉國民小學","赤山國小","赤山國民小學","赤崁國小","赤崁國民小學","路上國小","路上國民小學","路竹國小","路竹國民小學","路竹幼兒園","路竹高中","路竹高級中學","車城國中","車城國小","車城國民中學","車城國民小學","車城鄉立托兒所","車籠埔國小","車籠埔國民小學","軍事情報學校","軍功國小","軍功國民小學","軍樂學校","軍法學校","軍醫學校","軍需學校","載熙","載熙國民小學","載興國小","載興國民小學","輔仁大學","輔仁高中","輔仁高級中學","輔大","輔英技術學院","輔英科大","輔英科技大學","辛亥國小","辛亥國民小學","辰光國小","辰光國民小學","述美國小","述美國民小學","迴龍中小學","迴龍國民中小學","追分國小","追分國民小學","通興國小","通興國民小學","通霄國中","通霄國小","通霄國民中學","通霄國民小學","通霄鎮立托兒所","造橋國中","造橋國小","造橋國民中學","造橋國民小學","造橋鄉立托兒所","逢甲","逢甲國小","逢甲國民小學","逢甲大學","逢甲學院","連江縣北竿鄉塘岐國民小學","連江縣南竿鄉仁愛國民小學","連江縣立中山國民中學","連江縣立中正國民中小學","連江縣立介壽國民中小學","連江縣立敬恆國民中小學","連江縣立東引國民中小學","連江縣莒光鄉東莒國民小學","進學國小","進學國民小學","進德國小","進德國民小學","逸仙國小","逸仙國民小學","過埤國小","過埤國民小學","過嶺國中","過嶺國小","過嶺國民中學","過嶺國民小學","過港幼兒園","過港幼稚園","過溝國中","過溝國小","過溝國民中學","過溝國民小學","過溪國小","過溪國民小學","過路國小","過路國民小學","道明外僑學校","道明外橋","道明高中","道明高級中學","達人女中","達人女子高級中學","達仁鄉立托兒所","達德商工","達德高級商工職業學校","達觀中小學","達觀國民中小學","達邦國小","達邦國民小學","遠東技術學院","遠東科大","遠東科技大學","那拔國小","那拔國民小學","那瑪夏國中","那瑪夏國民中學","郡坑國小","郡坑國民小學","都蘭國中","都蘭國小","都蘭國民中學","都蘭國民小學","都達國小","都達國民小學","鄧公國小","鄧公國民小學","醒吾","醒吾學校財團法人醒吾科技大學","醒吾技術學院","醒吾科技大學","里港國中","里港國小","里港國民中學","里港國民小學","里港鄉立托兒所","重光國小","重光國民小學","重寮國小","重寮國民小學","重慶國中","重慶國小","重慶國民中學","重慶國民小學","重溪國小","重溪國民小學","重興國小","重興國民小學","重陽國小","重陽國民小學","野柳國小","野柳國民小學","金中","金城國中","金城國民中學","金城幼稚園","金大","金寧中小學","金寧國民中小學","金山國小","金山國民小學","金山幼兒園","金山托兒所","金山高中","金山高級中學","金岳國小","金岳國民小學","金峰鄉立托兒所","金沙國中","金沙國小","金沙國民中學","金沙國民小學","金洋國小","金洋國民小學","金湖國中","金湖國小","金湖國民中學","金湖國民小學","金潭國小","金潭國民小學","金甌女中","金甌女子高級中學","金竹國小","金竹國民小學","金美國小","金美國民小學","金華國中","金華國小","金華國民中學","金華國民小學","金門大學","金門技術學院","金門縣烈嶼鄉上岐國民小學","金門縣烈嶼鄉卓環國民小學","金門縣烈嶼鄉西口國民小學","金門縣立烈嶼國民中學","金門縣立金城國民中學","金門縣立金城幼稚園","金門縣立金寧國民中小學","金門縣立金沙國民中學","金門縣立金湖國民中學","金門縣金城鎮中正國民小學","金門縣金城鎮古城國民小學","金門縣金城鎮賢庵國民小學","金門縣金寧鄉古寧國民小學","金門縣金寧鄉湖埔國民小學","金門縣金寧鄉金鼎國民小學","金門縣金沙鎮何浦國民小學","金門縣金沙鎮安瀾國民小學","金門縣金沙鎮述美國民小學","金門縣金沙鎮金沙國民小學","金門縣金湖鎮多年國民小學","金門縣金湖鎮柏村國民小學","金門縣金湖鎮正義國民小學","金門縣金湖鎮金湖國民小學","金門縣金湖鎮開瑄國民小學","金門農工","金門高級中學","金門高級農工職業學校","金鼎國小","金鼎國民小學","金龍國小","金龍國民小學","銅蘭國小","銅蘭國民小學","銅鑼國小","銅鑼國民小學","銅鑼鄉立托兒所","銅門國小","銅門國民小學","銘傳國中","銘傳國小","銘傳國民中學","銘傳國民小學","銘傳大學","銘傳管理學院","錦和國小","錦和國民小學","錦和高中","錦和高級中學","錦屏國小","錦屏國民小學","錦山國小","錦山國民小學","錦水國小","錦水國民小學","錦湖國小","錦湖國民小學","錦興國小","錦興國民小學","鎮北國小","鎮北國民小學","鎮南國小","鎮南國民小學","鎮平國小","鎮平國民小學","鎮昌國小","鎮昌國民小學","鎮東國小","鎮東國民小學","鎮海國中","鎮海國小","鎮海國民中學","鎮海國民小學","鎮西國小","鎮西國民小學","鐵山國小","鐵山國民小學","鑄強國小","鑄強國民小學","長億國小","長億國民小學","長億高中","長億高級中學","長坑國小","長坑國民小學","長安國中","長安國小","長安國民中學","長安國民小學","長平國小","長平國民小學","長庚","長庚國小","長庚國民小學","長庚大學","長庚技術學院","長庚醫學院暨工程學院","長春國小","長春國民小學","長榮大學","長榮女中","長榮女子高級中學","長榮百合國小","長榮百合國民小學","長榮管理學院","長榮高中","長榮高級中學","長樂國小","長樂國民小學","長橋國小","長橋國民小學","長治國中","長治國民中學","長流國小","長流國民小學","長濱國中","長濱國小","長濱國民中學","長濱國民小學","長濱鄉立托兒所","長福國小","長福國民小學","長興國小","長興國民小學","長良國小","長良國民小學","開元國小","開元國民小學","開南","開南商工","開南大學","開南管理學院","開南高級商工職業學校","開平餐飲","開平餐飲職業學校","開瑄國小","開瑄國民小學","開礦國小","開礦國民小學","關埔國小","關埔國民小學","關山國中","關山國小","關山國民中學","關山國民小學","關山工商","關山鎮立托兒所","關山高級工商職業學校","關廟國中","關廟國小","關廟國民中學","關廟國民小學","關廟幼兒園","關廟托兒所","關東國小","關東國民小學","關渡國中","關渡國小","關渡國民中學","關渡國民小學","關西國中","關西國小","關西國民中學","關西國民小學","關西鎮立托兒所","關西高中","關西高級中學","阿蓮國中","阿蓮國小","阿蓮國民中學","阿蓮國民小學","阿里山中小學","阿里山國民中小學","阿里山鄉立托兒所","陜西國小","陜西國民小學","陳平國小","陳平國民小學","陳康國小","陳康國民小學","陸戰隊士官學校","陸興高中","陸興高級中學","陸豐國小","陸豐國民小學","陸軍專科學校","陸軍特戰學校","陸軍理工學院","陸軍聯合參謀大學","陸軍財務經理學校","陸軍軍官學校","陽交大","陽光國小","陽光國民小學","陽明","陽明交大","陽明交通大學","陽明國中","陽明國小","陽明國民中學","陽明國民小學","陽明大學","陽明山國小","陽明山國民小學","陽明醫學院","陽明高中","陽明高級中學","陽明高級中等學校","隆田國小","隆田國民小學","隆聖國小","隆聖國民小學","隆華國小","隆華國民小學","隘寮國小","隘寮國民小學","隘門國小","隘門國民小學","隙頂國小","隙頂國民小學","雄中","雄商","雄女","集來國小","集來國民小學","集美國小","集美國民小學","集集國中","集集國小","集集國民中學","集集國民小學","集集鎮立托兒所","雙中","雙冬國小","雙冬國民小學","雙十國中","雙十國民中學","雙園國中","雙園國小","雙園國民中學","雙園國民小學","雙園托兒所","雙城國小","雙城國民小學","雙峰國小","雙峰國民小學","雙春國小","雙春國民小學","雙湖國小","雙湖國民小學","雙溪國小","雙溪國民小學","雙溪幼兒園","雙溪托兒所","雙溪高級中學","雙蓮國小","雙蓮國民小學","雙連國小","雙連國民小學","雙龍國小","雙龍國民小學","雨聲國小","雨聲國民小學","雨農國小","雨農國民小學","雲林國中","雲林國小","雲林國民中學","雲林國民小學","雲林技術學院","雲林特教","雲林特殊教育學校","雲林科技大學","雲林縣二崙鄉三和國民小學","雲林縣二崙鄉二崙國民小學","雲林縣二崙鄉來惠國民小學","雲林縣二崙鄉大同國民小學","雲林縣二崙鄉旭光國民小學","雲林縣二崙鄉永定國民小學","雲林縣二崙鄉油車國民小學","雲林縣二崙鄉立托兒所","雲林縣二崙鄉義賢國民小學","雲林縣元長鄉仁德國民小學","雲林縣元長鄉仁愛國民小學","雲林縣元長鄉信義國民小學","雲林縣元長鄉元長國民小學","雲林縣元長鄉和平國民小學","雲林縣元長鄉客厝國民小學","雲林縣元長鄉山內國民小學","雲林縣元長鄉忠孝國民小學","雲林縣元長鄉新生國民小學","雲林縣元長鄉立托兒所","雲林縣北港鎮僑美國民小學","雲林縣北港鎮北辰國民小學","雲林縣北港鎮南陽國民小學","雲林縣北港鎮好收國民小學","雲林縣北港鎮朝陽國民小學","雲林縣北港鎮東榮國民小學","雲林縣北港鎮立托兒所","雲林縣北港鎮育英國民小學","雲林縣北港鎮辰光國民小學","雲林縣口湖鄉下崙國民小學","雲林縣口湖鄉口湖國民小學","雲林縣口湖鄉台興國民小學","雲林縣口湖鄉崇文國民小學","雲林縣口湖鄉成龍國民小學","雲林縣口湖鄉文光國民小學","雲林縣口湖鄉立托兒所","雲林縣口湖鄉臺興國民小學","雲林縣口湖鄉興南國民小學","雲林縣口湖鄉金湖國民小學","雲林縣口湖鄉頂湖國民小學","雲林縣古坑鄉新光國民小學","雲林縣古坑鄉東和國民小學","雲林縣古坑鄉桂林國民小學","雲林縣古坑鄉棋山國民小學","雲林縣古坑鄉水碓國民小學","雲林縣古坑鄉永光國民小學","雲林縣古坑鄉立托兒所","雲林縣古坑鄉興昌國民小學","雲林縣古坑鄉草嶺生態地質國民小學","雲林縣古坑鄉華南實驗國民小學","雲林縣古坑鄉華山國民小學","雲林縣台西鄉台西國民小學","雲林縣台西鄉尚德國民小學","雲林縣台西鄉崙豐國民小學","雲林縣台西鄉新興國民小學","雲林縣台西鄉泉州國民小學","雲林縣台西鄉立托兒所","雲林縣四湖鄉三崙國民小學","雲林縣四湖鄉內湖國民小學","雲林縣四湖鄉南光國民小學","雲林縣四湖鄉四湖國民小學","雲林縣四湖鄉建華國民小學","雲林縣四湖鄉建陽國民小學","雲林縣四湖鄉明德國民小學","雲林縣四湖鄉東光國民小學","雲林縣四湖鄉林厝國民小學","雲林縣四湖鄉立托兒所","雲林縣四湖鄉飛沙國民小學","雲林縣四湖鄉鹿場國民小學","雲林縣土庫鎮土庫國民小學","雲林縣土庫鎮埤腳國民小學","雲林縣土庫鎮宏崙國民小學","雲林縣土庫鎮後埔國民小學","雲林縣土庫鎮新庄國民小學","雲林縣土庫鎮秀潭國民小學","雲林縣土庫鎮立托兒所","雲林縣土庫鎮馬光國民小學","雲林縣大埤鄉仁和國民小學","雲林縣大埤鄉嘉興國民小學","雲林縣大埤鄉大埤國民小學","雲林縣大埤鄉立托兒所","雲林縣大埤鄉聯美國民小學","雲林縣大埤鄉舊庄國民小學","雲林縣山峰華德福教育實驗國民小學","雲林縣崙背鄉中和國民小學","雲林縣崙背鄉大有國民小學","雲林縣崙背鄉崙背國民小學","雲林縣崙背鄉東興國民小學","雲林縣崙背鄉立托兒所","雲林縣崙背鄉豐榮國民小學","雲林縣崙背鄉陽明國民小學","雲林縣斗六市久安國民小學","雲林縣斗六市保長國民小學","雲林縣斗六市公誠國民小學","雲林縣斗六市斗六國民小學","雲林縣斗六市林頭國民小學","雲林縣斗六市梅林國民小學","雲林縣斗六市溝壩國民小學","雲林縣斗六市溪洲國民小學","雲林縣斗六市石榴國民小學","雲林縣斗六市立托兒所","雲林縣斗六市鎮南國民小學","雲林縣斗六市鎮東國民小學","雲林縣斗六市鎮西國民小學","雲林縣斗六市雲林國民小學","雲林縣斗南鎮僑真國民小學","雲林縣斗南鎮大東國民小學","雲林縣斗南鎮文安國民小學","雲林縣斗南鎮斗南國民小學","雲林縣斗南鎮石龜國民小學","雲林縣斗南鎮立幼兒園","雲林縣斗南鎮立托兒所","雲林縣斗南鎮重光國民小學","雲林縣東勢鄉同安國民小學","雲林縣東勢鄉安南國民小學","雲林縣東勢鄉明倫國民小學","雲林縣東勢鄉東勢國民小學","雲林縣東勢鄉立托兒所","雲林縣東勢鄉龍潭國民小學","雲林縣林內鄉九芎國民小學","雲林縣林內鄉成功國民小學","雲林縣林內鄉林中國民小學","雲林縣林內鄉林內國民小學","雲林縣林內鄉民生國民小學","雲林縣林內鄉立托兒所","雲林縣林內鄉重興國民小學","雲林縣水林鄉中興國民小學","雲林縣水林鄉大興國民小學","雲林縣水林鄉宏仁國民小學","雲林縣水林鄉尖山國民小學","雲林縣水林鄉文正國民小學","雲林縣水林鄉水燦林國民小學","雲林縣水林鄉立托兒所","雲林縣水林鄉蔦松國民小學","雲林縣水林鄉誠正國民小學","雲林縣潮厝華德福教育實驗國民小學","雲林縣私立大德工業商業職業學校","雲林縣私立大成高級商工職業學校","雲林縣私立崇先高級中學","雲林縣私立巨人高級中學","雲林縣私立揚子高級中學","雲林縣私立文生高級中學","雲林縣私立東南國民中學","雲林縣私立正心高級中學","雲林縣私立永年高級中學","雲林縣私立淵明國民中學","雲林縣私立益新高級工商職業學校","雲林縣私立維多利亞國民小學","雲林縣私立義峰高級中學","雲林縣立二崙國民中學","雲林縣立元長國民中學","雲林縣立北港國民中學","雲林縣立口湖國民中學","雲林縣立古坑國民中小學","雲林縣立古坑華德福實驗高級中學","雲林縣立台西國民中學","雲林縣立四湖國民中學","雲林縣立土庫國民中學","雲林縣立大埤國民中學","雲林縣立宜梧國民中學","雲林縣立崇德國民中學","雲林縣立崙背國民中學","雲林縣立建國國民中學","雲林縣立斗六國民中學","雲林縣立斗南高級中學","雲林縣立東仁國民中學","雲林縣立東勢國民中學","雲林縣立東和國民中學","雲林縣立東明國民中學","雲林縣立林內國民中學","雲林縣立樟湖生態國民中小學","雲林縣立水林國民中學","雲林縣立石榴國民中學","雲林縣立臺西國民中學","雲林縣立莿桐國民中學","雲林縣立蔦松國民中學","雲林縣立虎尾國民中學","雲林縣立褒忠國民中學","雲林縣立西螺國民中學","雲林縣立雲林國民中學","雲林縣立飛沙國民中學","雲林縣立馬光國民中學","雲林縣立麥寮高級中學","雲林縣臺西鄉尚德國民小學","雲林縣臺西鄉崙豐國民小學","雲林縣臺西鄉新興國民小學","雲林縣臺西鄉泉州國民小學","雲林縣臺西鄉立托兒所","雲林縣臺西鄉臺西國民小學","雲林縣莿桐鄉僑和國民小學","雲林縣莿桐鄉六合國民小學","雲林縣莿桐鄉大美國民小學","雲林縣莿桐鄉立托兒所","雲林縣莿桐鄉育仁國民小學","雲林縣莿桐鄉莿桐國民小學","雲林縣莿桐鄉饒平國民小學","雲林縣虎尾鎮中正國民小學","雲林縣虎尾鎮中溪國民小學","雲林縣虎尾鎮光復國民小學","雲林縣虎尾鎮大屯國民小學","雲林縣虎尾鎮安慶國民小學","雲林縣虎尾鎮平和國民小學","雲林縣虎尾鎮廉使國民小學","雲林縣虎尾鎮惠來國民小學","雲林縣虎尾鎮拯民國民小學","雲林縣虎尾鎮立仁國民小學","雲林縣虎尾鎮立托兒所","雲林縣虎尾鎮虎尾國民小學","雲林縣褒忠鄉復興國民小學","雲林縣褒忠鄉立托兒所","雲林縣褒忠鄉褒忠國民小學","雲林縣褒忠鄉龍巖國民小學","雲林縣西螺鎮中山國民小學","雲林縣西螺鎮吳厝國民小學","雲林縣西螺鎮大新國民小學","雲林縣西螺鎮安定國民小學","雲林縣西螺鎮廣興國民小學","雲林縣西螺鎮文昌國民小學","雲林縣西螺鎮文興國民小學","雲林縣西螺鎮文賢國民小學","雲林縣西螺鎮立托兒所","雲林縣麥寮鄉明禮國民小學","雲林縣麥寮鄉橋頭國民小學","雲林縣麥寮鄉立托兒所","雲林縣麥寮鄉興華國民小學","雲林縣麥寮鄉豐安國民小學","雲林縣麥寮鄉麥寮國民小學","雲海國小","雲海國民小學","雲特","雲科","雲科大","電光國小","電光國民小學","電雷學校","霄裡國小","霄裡國民小學","霞雲國小","霞雲國民小學","霧台國小","霧台國民小學","霧台鄉立托兒所","霧峰國中","霧峰國小","霧峰國民中學","霧峰國民小學","霧峰幼兒園","霧峰托兒所","霧峰農工","霧峰農業工業高級中等學校","霧臺國小","霧臺國民小學","霧臺鄉立托兒所","霧鹿國小","霧鹿國民小學","青埔國中","青埔國小","青埔國民中學","青埔國民小學","青山中小學","青山國小","青山國民中小學","青山國民小學","青年國中","青年國民中學","青溪國中","青溪國小","青溪國民中學","青溪國民小學","青潭國小","青潭國民小學","青草國小","青草國民小學","青草湖國小","青草湖國民小學","青葉國小","青葉國民小學","靜修國小","靜修國民小學","靜修女中","靜修女子高級中學","靜宜","靜宜大學","靜宜女子文理學院","靜宜女子英語專科學校","靜心國中","靜心國民中學","靜心小學","靜浦國小","靜浦國民小學","韓僑小學","頂六國小","頂六國民小學","頂埔國小","頂埔國民小學","頂庄國小","頂庄國民小學","頂洲國小","頂洲國民小學","頂湖國小","頂湖國民小學","頂溪國小","頂溪國民小學","頂番國小","頂番國民小學","頂社國小","頂社國民小學","順天國中","順天國小","順天國民中學","順天國民小學","順安國中","順安國小","順安國民中學","順安國民小學","頭份國中","頭份國小","頭份國民中學","頭份國民小學","頭份鎮立托兒所","頭前國中","頭前國小","頭前國民中學","頭前國民小學","頭城商職","頭城國中","頭城國小","頭城國民中學","頭城國民小學","頭城家商","頭城鎮立托兒所","頭城高商","頭城高級家事商業職業學校","頭城高職","頭家國小","頭家國民小學","頭屋國中","頭屋國小","頭屋國民中學","頭屋國民小學","頭汴國小","頭汴國民小學","頭洲國小","頭洲國民小學","頭湖國小","頭湖國民小學","頭社國小","頭社國民小學","顯宮國小","顯宮國民小學","風櫃國小","風櫃國民小學","飛沙國中","飛沙國小","飛沙國民中學","飛沙國民小學","餉潭國小","餉潭國民小學","餐大附中","饒平國小","饒平國民小學","饒明","饒明國民小學","香中","香山區托兒所","香山國小","香山國民小學","香山高級中學","香林國小","香林國民小學","香田國小","香田國民小學","香蘭國小","香蘭國民小學","馬偕護理專科學校","馬偕醫專","馬光國中","馬光國小","馬光國民中學","馬光國民小學","馬公國中","馬公國小","馬公國民中學","馬公國民小學","馬公市立托兒所","馬公高級中學","馬祖高中","馬祖高級中學","馬興國小","馬興國民小學","馬蘭國小","馬蘭國民小學","馬賽國小","馬賽國民小學","馬遠國小","馬遠國民小學","馬高","馬鳴國小","馬鳴國民小學","體大","體師","體育大學","體育學院","體育師範專科學校","體育高中","體育高級中等學校","高原國小","高原國民小學","高士國小","高士國民小學","高大","高寮國小","高寮國民小學","高峰國小","高峰國民小學","高師","高師大","高師附中","高朗國小","高朗國民小學","高榮國小","高榮國民小學","高樹國中","高樹國小","高樹國民中學","高樹國民小學","高樹鄉立托兒所","高泰國中","高泰國民中學","高海","高海科大","高科","高科大","高級中學","高美國小","高美國民小學","高美醫專","高美醫護管理專科學校","高義國小","高義國民小學","高苑技術學院","高苑科大","高苑科技大學","高醫","高雄大學","高雄女子高級中學","高雄工學院","高雄市三民區三民國民小學","高雄市三民區光武國民小學","高雄市三民區十全國民小學","高雄市三民區博愛國民小學","高雄市三民區愛國國民小學","高雄市三民區東光國民小學","高雄市三民區正興國民小學","高雄市三民區民族國民小學","高雄市三民區河堤國民小學","高雄市三民區河濱國民小學","高雄市三民區獅湖國民小學","高雄市三民區莊敬國民小學","高雄市三民區陽明國民小學","高雄市三民區鼎金國民小學","高雄市仁武區仁武國民小學","高雄市仁武區八卦國民小學","高雄市仁武區灣內國民小學","高雄市仁武區烏林國民小學","高雄市仁武區登發國民小學","高雄市仁武區竹後國民小學","高雄市內門區內門國民小學","高雄市內門區景義國民小學","高雄市內門區木柵國民小學","高雄市內門區溝坪國民小學","高雄市內門區西門國民小學","高雄市內門區觀亭國民小學","高雄市內門區金竹國民小學","高雄市六龜區六龜國民小學","高雄市六龜區寶來國民小學","高雄市六龜區新威國民小學","高雄市六龜區新發國民小學","高雄市六龜區荖濃國民小學","高雄市六龜區龍興國民小學","高雄市前金區前金國民小學","高雄市前金區建國國民小學","高雄市前鎮區仁愛國民小學","高雄市前鎮區佛公國民小學","高雄市前鎮區光華國民小學","高雄市前鎮區前鎮國民小學","高雄市前鎮區復興國民小學","高雄市前鎮區愛群國民小學","高雄市前鎮區明正國民小學","高雄市前鎮區樂群國民小學","高雄市前鎮區民權國民小學","高雄市前鎮區獅甲國民小學","高雄市前鎮區瑞祥國民小學","高雄市前鎮區瑞豐國民小學","高雄市前鎮區紅毛港國民小學","高雄市前鎮區鎮昌國民小學","高雄市大寮區中庄國民小學","高雄市大寮區大寮國民小學","高雄市大寮區山頂國民小學","高雄市大寮區後庄國民小學","高雄市大寮區忠義國民小學","高雄市大寮區昭明國民小學","高雄市大寮區永芳國民小學","高雄市大寮區溪寮國民小學","高雄市大寮區潮寮國民小學","高雄市大寮區翁園國民小學","高雄市大樹區九曲國民小學","高雄市大樹區大樹國民小學","高雄市大樹區姑山國民小學","高雄市大樹區小坪國民小學","高雄市大樹區水寮國民小學","高雄市大樹區溪埔國民小學","高雄市大樹區興田國民小學","高雄市大樹區龍目國民小學","高雄市大社區嘉誠國民小學","高雄市大社區大社國民小學","高雄市大社區觀音國民小學","高雄市小港區二苓國民小學","高雄市小港區坪頂國民小學","高雄市小港區太平國民小學","高雄市小港區小港國民小學","高雄市小港區明義國民小學","高雄市小港區桂林國民小學","高雄市小港區港和國民小學","高雄市小港區漢民國民小學","高雄市小港區華山國民小學","高雄市小港區青山國民小學","高雄市小港區鳳林國民小學","高雄市小港區鳳陽國民小學","高雄市小港區鳳鳴國民小學","高雄市岡山區兆湘國民小學","高雄市岡山區前峰國民小學","高雄市岡山區和平國民小學","高雄市岡山區嘉興國民小學","高雄市岡山區壽天國民小學","高雄市岡山區岡山國民小學","高雄市岡山區後紅國民小學","高雄市岡山區竹圍國民小學","高雄市左營區勝利國民小學","高雄市左營區屏山國民小學","高雄市左營區左營國民小學","高雄市左營區文府國民小學","高雄市左營區新上國民小學","高雄市左營區新光國民小學","高雄市左營區新民國民小學","高雄市左營區新莊國民小學","高雄市左營區明德國民小學","高雄市左營區永清國民小學","高雄市左營區福山國民小學","高雄市左營區舊城國民小學","高雄市巴楠花部落小學","高雄市彌陀區南安國民小學","高雄市彌陀區壽齡國民小學","高雄市彌陀區彌陀國民小學","高雄市新興區七賢國民小學","高雄市新興區信義國民小學","高雄市新興區大同國民小學","高雄市新興區新興國民小學","高雄市旗山區圓潭國民小學","高雄市旗山區嶺口國民小學","高雄市旗山區旗尾國民小學","高雄市旗山區旗山國民小學","高雄市旗山區溪洲國民小學","高雄市旗山區鼓山國民小學","高雄市旗津區中洲國民小學","高雄市旗津區大汕國民小學","高雄市旗津區旗津國民小學","高雄市杉林區上平國民小學","高雄市杉林區新庄國民小學","高雄市杉林區月美國民小學","高雄市杉林區杉林國民小學","高雄市杉林區集來國民小學","高雄市林園區中芸國民小學","高雄市林園區林園國民小學","高雄市林園區汕尾國民小學","高雄市林園區港埔國民小學","高雄市林園區王公國民小學","高雄市林園區金潭國民小學","高雄市桃源區寶山國民小學","高雄市桃源區建山國民小學","高雄市桃源區桃源國民小學","高雄市桃源區樟山國民小學","高雄市桃源區興中國民小學","高雄市梓官區梓官國民小學","高雄市梓官區蚵寮國民小學","高雄市楠梓區加昌國民小學","高雄市楠梓區右昌國民小學","高雄市楠梓區後勁國民小學","高雄市楠梓區援中國民小學","高雄市楠梓區楠梓國民小學","高雄市楠梓區楠陽國民小學","高雄市楠梓區油廠國民小學","高雄市楠梓區莒光國民小學","高雄市橋頭區五林國民小學","高雄市橋頭區仕隆國民小學","高雄市橋頭區橋頭國民小學","高雄市橋頭區甲圍國民小學","高雄市橋頭區興糖國民小學","高雄市永安區新港國民小學","高雄市永安區永安國民小學","高雄市永安區維新國民小學","高雄市湖內區三侯國民小學","高雄市湖內區大湖國民小學","高雄市湖內區文賢國民小學","高雄市湖內區明宗國民小學","高雄市湖內區海埔國民小學","高雄市燕巢區安招國民小學","高雄市燕巢區橫山國民小學","高雄市燕巢區深水國民小學","高雄市燕巢區燕巢國民小學","高雄市燕巢區金山國民小學","高雄市燕巢區鳳雄國民小學","高雄市田寮區崇德國民小學","高雄市田寮區新興國民小學","高雄市甲仙區寶隆國民小學","高雄市甲仙區小林國民小學","高雄市甲仙區甲仙國民小學","高雄市私立三信家事商業職業學校","高雄市私立中華高級藝術職業學校","高雄市私立優佳國民中學","高雄市私立國光國民中學","高雄市私立國光高級中學","高雄市私立國際高級商工職業學校","高雄市私立大榮高級中學","高雄市私立復華高級中學","高雄市私立日僑小學","高雄市私立明誠高級中學","高雄市私立樹德高級家事商業職業學校","高雄市私立立志高級中學","高雄市私立美國學校","高雄市私立道明外僑學校","高雄市私立道明高級中學","高雄市私立韓僑小學","高雄市私立高鳳高級工業家事職業學校","高雄市立一甲國民中學","高雄市立七賢國民中學","高雄市立三民國民中學","高雄市立三民高級中學","高雄市立三民高級家事商業職業學校","高雄市立中山國民中學","高雄市立中山高級中學","高雄市立中崙國民中學","高雄市立中庄國民中學","高雄市立中正高級中學","高雄市立中正高級工業職業學校","高雄市立中芸國民中學","高雄市立五甲國民中學","高雄市立五福國民中學","高雄市立仁武幼兒園","高雄市立仁武特殊教育學校","高雄市立仁武高級中學","高雄市立光華國民中學","高雄市立內門國民中學","高雄市立內門幼兒園","高雄市立六龜幼兒園","高雄市立六龜高級中學","高雄市立前峰國民中學","高雄市立前金國民中學","高雄市立前金幼兒園","高雄市立前金幼稚園","高雄市立前鎮國民中學","高雄市立前鎮高級中學","高雄市立南隆國民中學","高雄市立右昌國民中學","高雄市立嘉興國民中學","高雄市立國昌國民中學","高雄市立圓富國民中學","高雄市立壽山國民中學","高雄市立大仁國民中學","高雄市立大寮國民中學","高雄市立大寮幼兒園","高雄市立大樹國民中學","高雄市立大洲國民中學","高雄市立大灣國民中學","高雄市立大社國民中學","高雄市立大義國民中學","高雄市立寶來國民中學","高雄市立小港國民中學","高雄市立小港高級中學","高雄市立岡山國民中學","高雄市立左營國民中學","高雄市立左營高級中學","高雄市立彌陀國民中學","高雄市立後勁國民中學","高雄市立忠孝國民中學","高雄市立成功特殊教育學校","高雄市立文山高級中學","高雄市立文府國民中學","高雄市立新興高級中學","高雄市立新莊高級中學","高雄市立旗山國民中學","高雄市立旗津國民中學","高雄市立明義國民中學","高雄市立明華國民中學","高雄市立杉林國民中學","高雄市立林園高級中學","高雄市立桃源國民中學","高雄市立梓官國民中學","高雄市立楠梓國民中學","高雄市立楠梓特殊學校","高雄市立楠梓高級中學","高雄市立橋頭國民中學","高雄市立正興國民中學","高雄市立民族國民中學","高雄市立永安國民中學","高雄市立海青高級工商職業學校","高雄市立湖內國民中學","高雄市立湖內幼兒園","高雄市立溪埔國民中學","高雄市立潮寮國民中學","高雄市立燕巢國民中學","高雄市立獅甲國民中學","高雄市立瑞祥高級中學","高雄市立瑞豐國民中學","高雄市立田寮國民中學","高雄市立甲仙國民中學","高雄市立福山國民中學","高雄市立福誠高級中學","高雄市立空中大學","高雄市立立德國民中學","高雄市立美濃國民中學","高雄市立翠屏國民中小學","高雄市立興仁國民中學","高雄市立苓雅國民中學","高雄市立英明國民中學","高雄市立茂林國民中學","高雄市立茄萣國民中學","高雄市立蚵寮國民中學","高雄市立裕誠幼兒園","高雄市立裕誠幼稚園","高雄市立路竹幼兒園","高雄市立路竹高級中學","高雄市立那瑪夏國民中學","高雄市立阿蓮國民中學","高雄市立陽明國民中學","高雄市立青年國民中學","高雄市立高雄女子高級中學","高雄市立高雄特殊教育學校","高雄市立高雄高級中學","高雄市立高雄高級商業職業學校","高雄市立高雄高級工業職業學校","高雄市立鳥松國民中學","高雄市立鳥松幼兒園","高雄市立鳳山國民中學","高雄市立鳳林國民中學","高雄市立鳳甲國民中學","高雄市立鳳翔國民中學","高雄市立鳳西國民中學","高雄市立鹽埕國民中學","高雄市立鼎金國民中學","高雄市立鼓山高級中學","高雄市立龍肚國民中學","高雄市立龍華國民中學","高雄市美濃區中壇國民小學","高雄市美濃區吉東國民小學","高雄市美濃區吉洋國民小學","高雄市美濃區廣興國民小學","高雄市美濃區東門國民小學","高雄市美濃區福安國民小學","高雄市美濃區美濃國民小學","高雄市美濃區龍山國民小學","高雄市美濃區龍肚國民小學","高雄市苓雅區中正國民小學","高雄市苓雅區五權國民小學","高雄市苓雅區凱旋國民小學","高雄市苓雅區四維國民小學","高雄市苓雅區成功國民小學","高雄市苓雅區福康國民小學","高雄市苓雅區福東國民小學","高雄市苓雅區苓洲國民小學","高雄市茂林區多納國民小學","高雄市茂林區茂林國民小學","高雄市茄萣區成功國民小學","高雄市茄萣區砂崙國民小學","高雄市茄萣區興達國民小學","高雄市茄萣區茄萣國民小學","高雄市路竹區一甲國民小學","高雄市路竹區三埤國民小學","高雄市路竹區下坑國民小學","高雄市路竹區北嶺國民小學","高雄市路竹區大社國民小學","高雄市路竹區竹滬國民小學","高雄市路竹區蔡文國民小學","高雄市路竹區路竹國民小學","高雄市那瑪夏區民權國民小學","高雄市那瑪夏區民生國民小學","高雄市阿蓮區中路國民小學","高雄市阿蓮區復安國民小學","高雄市阿蓮區阿蓮國民小學","高雄市鳥松區仁美國民小學","高雄市鳥松區大華國民小學","高雄市鳥松區鳥松國民小學","高雄市鳳山區中山國民小學","高雄市鳳山區中崙國民小學","高雄市鳳山區中正國民小學","高雄市鳳山區五甲國民小學","高雄市鳳山區五福國民小學","高雄市鳳山區南成國民小學","高雄市鳳山區大東國民小學","高雄市鳳山區忠孝國民小學","高雄市鳳山區文山國民小學","高雄市鳳山區文德國民小學","高雄市鳳山區文華國民小學","高雄市鳳山區新甲國民小學","高雄市鳳山區曹公國民小學","高雄市鳳山區正義國民小學","高雄市鳳山區瑞興國民小學","高雄市鳳山區福誠國民小學","高雄市鳳山區誠正國民小學","高雄市鳳山區過埤國民小學","高雄市鳳山區鎮北國民小學","高雄市鳳山區鳳山國民小學","高雄市鳳山區鳳翔國民小學","高雄市鳳山區鳳西國民小學","高雄市鹽埕區光榮國民小學","高雄市鹽埕區忠孝國民小學","高雄市鹽埕區鹽埕國民小學","高雄市鼓山區中山國民小學","高雄市鼓山區九如國民小學","高雄市鼓山區內惟國民小學","高雄市鼓山區壽山國民小學","高雄市鼓山區鼓山國民小學","高雄市鼓山區鼓岩國民小學","高雄市鼓山區龍華國民小學","高雄師範大學","高雄師範大學附屬高級中學","高雄師範學院","高雄師院","高雄應用科大附設進修學院","高雄應用科技大學附設進修學院","高雄技術學院","高雄海洋技術學院","高雄特殊教育學校","高雄科學技術學院","高雄科學技術學院附設進修學院","高雄科技大學","高雄美國學校","高雄醫學大學","高雄醫學院","高雄餐旅大學","高雄餐旅大學附屬餐旅高級中等學校","高雄餐旅學院","高雄高商","高雄高工","高雄高級中學","高雄高級商業職業學校","高雄高級工業職業學校","高雄高職","高餐","高餐大","高餐附中","高鳳工家","高鳳技術學院","高鳳數位內容學院","高鳳高級工業家事職業學校","魚池國中","魚池國小","魚池國民中學","魚池國民小學","魚池鄉立托兒所","鯉魚國小","鯉魚國民小學","鯤鯓國小","鯤鯓國民小學","鳥嶼國中","鳥嶼國小","鳥嶼國民中學","鳥嶼國民小學","鳥松國中","鳥松國小","鳥松國民中學","鳥松國民小學","鳥松幼兒園","鳳仁國小","鳳仁國民小學","鳳凰國小","鳳凰國民小學","鳳山商工","鳳山國中","鳳山國小","鳳山國民中學","鳳山國民小學","鳳山高中","鳳山高級中學","鳳山高級商工職業學校","鳳岡國中","鳳岡國小","鳳岡國民中學","鳳岡國民小學","鳳新高中","鳳新高級中學","鳳林國中","鳳林國小","鳳林國民中學","鳳林國民小學","鳳林鎮立托兒所","鳳甲國中","鳳甲國民中學","鳳翔國中","鳳翔國小","鳳翔國民中學","鳳翔國民小學","鳳西國中","鳳西國小","鳳西國民中學","鳳西國民小學","鳳陽國小","鳳陽國民小學","鳳雄國小","鳳雄國民小學","鳳霞國小","鳳霞國民小學","鳳鳴國中","鳳鳴國小","鳳鳴國民中學","鳳鳴國民小學","鶯歌國中","鶯歌國小","鶯歌國民中學","鶯歌國民小學","鶯歌工商","鶯歌幼兒園","鶯歌托兒所","鶯歌高級工商職業學校","鶴岡國中","鶴岡國小","鶴岡國民中學","鶴岡國民小學","鶴聲國中","鶴聲國小","鶴聲國民中學","鶴聲國民小學","鷺中","鷺江國小","鷺江國民中學","鷺江國民小學","鸞山國小","鸞山國民小學","鹽中","鹽埔國中","鹽埔國小","鹽埔國民中學","鹽埔國民小學","鹽埔鄉立托兒所","鹽埕國小","鹽埕國民中學","鹽埕國民小學","鹽水國中","鹽水國小","鹽水國民中學","鹽水國民小學","鹽洲國小","鹽洲國民小學","鹿場國小","鹿場國民小學","鹿寮國中","鹿寮國民中學","鹿峰國小","鹿峰國民小學","鹿東國小","鹿東國民小學","鹿江中小學","鹿江國際中小學","鹿港國中","鹿港國小","鹿港國民中學","鹿港國民小學","鹿港鎮立托兒所","鹿港高級中學","鹿滿國小","鹿滿國民小學","鹿草國中","鹿草國小","鹿草國民中學","鹿草國民小學","鹿谷國中","鹿谷國小","鹿谷國民中學","鹿谷國民小學","鹿谷鄉立托兒所","鹿野國中","鹿野國小","鹿野國民中學","鹿野國民小學","鹿野鄉立托兒所","鹿高","鹿鳴國中","鹿鳴國民中學","麒麟國小","麒麟國民小學","麗園國小","麗園國民小學","麗山國中","麗山國小","麗山國民中學","麗山國民小學","麗山高中","麗山高級中學","麗林","麗林國民小學","麗湖國小","麗湖國民小學","麟洛國中","麟洛國小","麟洛國民中學","麟洛國民小學","麟洛鄉立托兒所","麥中","麥寮國小","麥寮國民小學","麥寮鄉立托兒所","麥寮高級中學","麻園國小","麻園國民小學","麻豆國中","麻豆國小","麻豆國民中學","麻豆國民小學","麻豆幼兒園","麻豆托兒所","黃竹國小","黃竹國民小學","黎明國中","黎明國小","黎明國民中學","黎明國民小學","黎明學院","黎明技術學院","鼎金國中","鼎金國小","鼎金國民中學","鼎金國民小學","鼓中","鼓山國小","鼓山國民小學","鼓山高級中學","鼓岩國小","鼓岩國民小學","鼻頭國小","鼻頭國民小學","龍井國中","龍井國小","龍井國民中學","龍井國民小學","龍井托兒所","龍坑國小","龍坑國民小學","龍埔國小","龍埔國民小學","龍壽國小","龍壽國民小學","龍安國小","龍安國民小學","龍山國中","龍山國小","龍山國民中學","龍山國民小學","龍山托兒所","龍岡國中","龍岡國小","龍岡國民中學","龍岡國民小學","龍峰國小","龍峰國民小學","龍崎國中","龍崎國小","龍崎國民中學","龍崎國民小學","龍崎托兒所","龍崗國小","龍崗國民小學","龍巖國小","龍巖國民小學","龍德家事商業職業學校","龍德家商","龍昇國小","龍昇國民小學","龍星國小","龍星國民小學","龍泉國小","龍泉國民小學","龍津國小","龍津國民小學","龍津高中","龍津高級中等學校","龍海國小","龍海國民小學","龍港國小","龍港國民小學","龍源國小","龍源國民小學","龍潭國中","龍潭國小","龍潭國民中學","龍潭國民小學","龍潭幼兒園","龍潭高中","龍潭高級中等學校","龍田國小","龍田國民小學","龍目國小","龍目國民小學","龍肚國中","龍肚國小","龍肚國民中學","龍肚國民小學","龍興國中","龍興國小","龍興國民中學","龍興國民小學","龍華","龍華國中","龍華國小","龍華國民中學","龍華國民小學","龍華技術學院","龍華科技大學","龍門國中","龍門國小","龍門國民中學","龍門國民小學","龜山國中","龜山國小","龜山國民中學","龜山國民小學","龜山幼兒園"],"_schoolDegree":["博一","博七","博三","博二","博五","博六","博四","國一","國三","國二","大一","大七","大三","大二","大五","大六","大四","小一","小三","小二","小五","小六","小四","碩一","碩三","碩二","碩五","碩四","高一","高三","高二"]}

# 將符合句型的參數列表印出。這是 debug 或是開發用的。
def debugInfo(inputSTR, utterance):
    if DEBUG_Zai_QMarker:
        print("[Zai_QMarker] {} ===> {}".format(inputSTR, utterance))

def getResult(inputSTR, utterance, args, resultDICT):
    debugInfo(inputSTR, utterance)
    if utterance == "在什麼位置":
        resultDICT["Zai"].append("QMarker")

    if utterance == "在哪邊嗎":
        resultDICT["Zai"].append("QMarker")

    if utterance == "在嗎":
        resultDICT["Zai"].append("QMarker")

    return resultDICT